#ifndef _DFLORA_AT_H
#define _DFLORA_AT_H
#include "uartUSB.h"
#include "timer.h"
#include "CH58x_common.h"
#include <stdlib.h>

#define LDBG(...){Serial.print("["); Serial.print(__FUNCTION__); Serial.print("(): "); Serial.print(__LINE__); Serial.print(" ] "); Serial.println(__VA_ARGS__);}
#define ATNUM   62
#define SENUM   6
#define UART_INTTERFACE_NUM    0
#define USB1_INTTERFACE_NUM    1
#define LORAWAN_INTTERFACE_NUM    2
#define USB2_INTTERFACE_NUM    3
#define RS485_INTTERFACE_NUM    4
#define NFC_INTTERFACE_NUM    5

#define WWDG_ENABLE
#define  NFC_ENABLE
#define  SLEEP_ENABLE
//#define  CSV_ENABLE
//#define  SPI_FLASH_LOG
#define  FLASH_ROM_LOG
//#define  UNIX_EPOCH_ENABLE
static uint16_t bufferLenth=0;
#define GPIOA 1
#define GPIOB 2
struct AtCmd;

extern volatile uint8_t tansFlag ;
extern volatile uint8_t transImmed ;
typedef enum {

    TRANSEND,  //透传结束
    TRANSTART,//传输开始
    RETRANS,
    TRANSSUCCESS,

}eTransState_t;

extern uint8_t AT;

#define SETMODE(AORB,PIN,MODE)\
   if(AORB == GPIOA){ \
       GPIOA_ModeCfg(PIN,MODE);\
   }else if(AORB == GPIOB){ \
       GPIOB_ModeCfg(PIN,MODE); \
   }

#define SETHIGH(AORB,PIN) \
        if(AORB == GPIOA){ \
            GPIOA_SetBits(PIN);\
        }else if(AORB == GPIOB){ \
            GPIOB_SetBits(PIN); \
        }
#define SETLOW(AORB,PIN) \
        if(AORB == GPIOA){ \
            GPIOA_ResetBits(PIN);\
        }else if(AORB == GPIOB){ \
            GPIOB_ResetBits(PIN); \
        }

typedef void (*pAtprint)(const char *data);
typedef void (*pAtwrite8)(uint8_t data);
typedef void (*pAtwrite16)(uint16_t data);
typedef void (*pAtflush)(void * data);

extern TimerEvent_t sleepTimer;
extern TimerEvent_t hardwareLoopTimer;
extern TimerEvent_t transTimer;
extern  TimerEvent_t unixEpochTimer;
// 定义环形缓冲区的大小
#define BUFFER_SIZE 64


typedef struct
{
  char name[32];
  int age;
}tSysInfo, *pSysInfo;
extern tSysInfo si;
extern char str[1024];
enum REGION{
	CN470 = 1,
	US915 = 2,
	EU868 = 3 
};

enum UPLINKTYPE{
	CONFIRMED = 1,
	UNCONFIRMED = 2
};

enum PORT{
    PORT1 = 1,
    PORT2 = 2,
    PORT3 = 3	
};
struct msgBuf
{
  struct msgBuf *next;
  uint16_t handle;  //!< Handle of the attribute that has been changed (must be first field)
  uint16_t len;     //!< Length of value
  uint8_t value[0]; //!< New value of the attribute
};

typedef struct
{
  uint8_t OTAA;
  uint8_t DEVEUI[8];
  uint8_t APPEUI[8];
  uint8_t APPKEY[16];
  uint8_t DEVADDR[8];
  uint8_t APPSKEY[16];
  uint32_t NETID;
  uint8_t PORT;
  uint8_t JOIN;
  uint32_t FREQS[9];
  uint8_t REGION;
  uint8_t SUBBAND;
  uint8_t ANTENNAGAIN;
  float EIRP;
  uint8_t DATARATE;
  uint8_t UPLINKTYPE;
}tLoraInfo, *pLoraInfo;

extern tLoraInfo lora;


typedef struct DFSerial{
	unsigned char recvBuffer[150];
    pAtprint print;
	pAtwrite8 write8;
	pAtwrite16 write16;
	pAtflush flush;
	uint16_t  datalength;
    uint8_t regLen;
	uint8_t dataFlag;
	uint8_t id;
	uint8_t index;
}tDFSerial, *pDFSerial;



typedef struct
{
    uint8_t enable;
    uint8_t voltage;
    int32_t supplyTime;
    uint8_t group;
    uint32_t pin;
}sOutpower1_t;
typedef struct
{
    uint8_t enable; //使能adc1的采集
    uint16_t adc1Value;//采集到的adc的值
    uint8_t adcFlag; //表示电压还是电流
    uint32_t pin;  //表示引脚
    uint8_t channel;

    int16_t min;
    int16_t max;
}sAdc1Control_t;

typedef struct
{
    uint8_t enable; //使能
    uint16_t ioCmd[12];//采集到的adc的值,可以有四条指令
    uint32_t pin;  //表示引脚
    uint8_t cmdlen; //指令长度
}sIoControl_t;

typedef struct
{
    uint8_t enable; //使能
    uint8_t mode; //使能
    uint8_t intType;
    uint8_t pinValue; //
    uint8_t changeFlag; //
    uint32_t pin;  //表示引脚

}sIoInControl_t;

// 定义一个结构体，用于存储IO计数器的相关信息
typedef struct {
    //char io[2]; // IO口名称
    char mode[10]; // 计数模式（上升沿、下降沿、停止等）
    uint16_t Count; // 计数值
    uint8_t reset; // 是否在计数前复位计数器
    uint8_t enable;
    uint8_t stop;
    uint32_t pin;  //表示引脚
    uint intType;
    uint8_t intTyp;
} sIoCounter_t;


// 定义一个结构体，存储经度、纬度和海拔数据
typedef struct {
    float longitude;
    float latitude;
    int altitude;
} sCoordinates_t;

typedef struct{
 uint8_t enable;
 uint8_t len ;
 uint8_t  item[15];
 uint16_t nextDelay;
 uint8_t startAddr;
 uint8_t regNum;
 uint8_t byteOrder;
 int8_t dataType;
 int8_t am;
 int8_t param1_str[10];
 int8_t param2_str[10];
 float param1;
 float param2;
}sRs485Control_t;

typedef struct{
    union {
    uint8_t value;

    struct{
      uint8_t dataBits: 2;
      uint8_t stopBits: 1;
      uint8_t parityEn: 1;
      uint8_t parity:2;
      uint8_t bit6:1;
      uint8_t bit7:1;
    };
    };
}sUartConfig_t;

typedef struct sLoraWanPara
{
  uint8_t joinType;
  uint8_t devEui[8];
  uint8_t joinEui[8];
  uint8_t appKey[16];
  uint32_t  devAddr;
  uint8_t appSKey[16];
  uint8_t nwkSKey[16];
  uint32_t netId;
  uint32_t singleFreq;
  uint8_t joinState;
  uint8_t port ;
  uint8_t uartPort[4];
  uint8_t uartMode;
  uint8_t channelsMask[6];
  float antennGain;
  uint8_t  eirp;
  uint8_t txpower;
  BOOL firstPacket;
  int8_t dataRate;
  uint8_t nbtrials;
  uint8_t class;
  BOOL adr;
  uint8_t region;
  uint8_t subBand;
  int8_t snr;
  int16_t rssi;
  uint8_t isTxConfirmed;
  uint8_t macVersion[3];
  uint32_t frequency[6];
  uint32_t frameCounter;
  uint16_t pid;
  uint16_t vid;
}LoraWanPara_t;



typedef struct sLoraWanAt
{

  uint8_t coding;
  BOOL powerChangeFlag;
  uint8_t mcuSleep;
  uint32_t baud[4];
  sUartConfig_t uartConfig[4];
  uint8_t atChannel;

  char hwVersion[10];
  uint8_t sn[8];
  uint8_t passwd[6];  //delete
  uint32_t dutyTime;
  uint32_t loopTime;
  float loss;
  char name[20];
  char product[15];  //delete
  //char pname[20];
  uint8_t pdate[4];
  uint32_t unixEpoch;  //delete
 // uint8_t seriesNum[8];

  uint32_t rowCsv;  //delete
  uint32_t addr;
  uint8_t udiskMode; //delete
}sHardware_t;


typedef struct
{
    sOutpower1_t power[2]; //改为power[1]
    uint8_t adcMask;
    sAdc1Control_t adc[2]; //改为adc[1]
    uint8_t countMask;
    sIoCounter_t ioCount[4];  //改为ioCount[2]
    uint8_t ioMask;
    uint8_t ioValue;
    sIoControl_t io[4];      //改为io[2]
    sIoInControl_t ioIn[4];  //改为io[2]
    uint32_t rs485Baud[9];
    sUartConfig_t rs485Config[9];
    uint16_t rs485Mask;
    sCoordinates_t coord;
    uint8_t rs485Retry;
    sRs485Control_t rs485item[9];
    uint32_t lastRow;
    BOOL reTrans;
    uint32_t reTranNum;
    uint32_t loopTime1;
    uint16_t reTranTimes;
    uint8_t fakeJoinFlag;  //delete
    BOOL adnDisable;
}sLoopOperation_t;
typedef struct{

    union {
      uint16_t value;
      struct{
        uint16_t joinType: 1;
        uint16_t devEui: 1;
        uint16_t joinEui: 1;
        uint16_t appKey: 1;
        uint16_t devAddr: 1;
        uint16_t appSKey: 1;
        uint16_t nwkSkey: 1;
        uint16_t region: 1;
        uint16_t antenngain: 1;
        uint16_t eirp: 1;
        uint16_t nbtrials: 1;
        uint16_t class: 1;
        uint16_t adr: 1;
        uint16_t dataRate: 1;
        uint16_t recovery: 1;
        uint16_t bit16: 1;
      };
    };
} sLoraWanMask_t;

typedef struct{
    uint16_t cfgCnt;
    uint8_t logPrint;
    uint8_t saveConfig;

    uint16_t crc;
    LoraWanPara_t loraWanPara;
    sLoraWanMask_t saveMask;
    sHardware_t hardPara;


    sLoopOperation_t loopOperation;
    uint8_t defaultLogPrint;
    uint8_t lmRecover;
}sDtuConfig_t;


extern sDtuConfig_t dtuConfig;

extern tDFSerial dfserials[SENUM];





typedef BOOL (*pAtFunc)(void *param, pDFSerial s);

typedef struct AtCmd
{
  const char *cmdStr;
  
  pAtFunc func;

}tAtCmd,*pAtCmd;


typedef struct{

  char latitude[9];
  char longitude[9];
  char altitude[9];
  char source[10];
  uint8_t accuracy;

}sNfcCoord_t;


typedef struct{
 uint32_t cfgCnt;
 char name[12];
 char MFGData[12];
 char sn[16];
 char product[12];
 char fVersion[10];
 char hVersion[10];
 BOOL active;
 uint8_t battery;
 sNfcCoord_t coord;
 char class;

 uint8_t devEUI[8];
 uint8_t joinEUI[8];
 uint8_t appkey[16];
 uint32_t devAddr;
 uint8_t appSkey[16];
 uint8_t nwkSKey[16];
 uint8_t nbtrials;

 char macVersion[8];
 BOOL joinType;
 uint8_t aport;
 char region[10];
 uint8_t subband;
 uint32_t upInterval;
 uint8_t txpower;
 uint8_t datarate;
 BOOL confirm;
 BOOL reJoin;
 BOOL adr;
 uint16_t pid;
 uint16_t vid;
 uint8_t recover;
}sNfcBasic_t;


typedef struct{
  int power12V;
  int power5V;
}sNfcPower_t;




typedef struct{
   uint8_t id;
   BOOL enable;
   uint8_t mode;
   uint8_t upLoad;
   uint8_t trigger;

}sDigitalINChannel_t;

typedef struct{
 uint8_t id;
 BOOL enable;
 uint16_t min;
 uint16_t max;
}sNfcAnalogChannel_t;


typedef struct{
 uint8_t id;
 int baudrate;
 BOOL enable;
 uint8_t  databits;
 uint8_t stopbit;
 char parity[10];
 uint8_t retry;
 uint16_t timeOut;
 uint8_t byteOrder;

 uint8_t writeCmd[15];
}sNfcModbus_t;

typedef struct{
    sNfcPower_t power;
    sDigitalINChannel_t digitalChannel[4];
    sNfcAnalogChannel_t analogChannel[2];
    sNfcModbus_t modBus[8];
}sNfcCollect_t;






typedef struct{

    sNfcBasic_t basic;
    sNfcCollect_t collect;

}nfcConfig_t;



void AtBegin();

uint16_t readADC(uint8_t ch,uint32_t gpio,int16_t min,int16_t max);
BOOL doOtaa(void *param, pDFSerial s);
BOOL doDeveui(void *param, pDFSerial s);
BOOL doAppeui(void *param, pDFSerial s);
BOOL doAppkey(void *param, pDFSerial s);
BOOL doDevaddr(void *param, pDFSerial s);
BOOL doAppSkey(void *param, pDFSerial s);
BOOL doNwkSkey(void *param, pDFSerial s);
BOOL doNetID(void *param, pDFSerial s);
BOOL doPort(void *param, pDFSerial s);
BOOL doJoin(void *param, pDFSerial s);
BOOL doFreqs(void *param, pDFSerial s);
BOOL doRegion(void *param, pDFSerial s);
BOOL doSubband(void *param, pDFSerial s);
BOOL doAntennagain(void *param, pDFSerial s);
BOOL doEirp(void *param, pDFSerial s);
BOOL doDatarate(void *param, pDFSerial s);
BOOL doUplinktype(void *param, pDFSerial s);
BOOL doGpio(void *param, pDFSerial s);
void execAtCmd(pDFSerial serialData);
pDFSerial recvAt();

BOOL startAt(const char *p, pDFSerial s);
BOOL doSave(const char *p, pDFSerial s);
void syncConfig();
BOOL doPulse(void *param, pDFSerial s);
BOOL doText(void *param, pDFSerial s);
BOOL doNbtrials(void *param, pDFSerial s);
BOOL doClass(void *param, pDFSerial s);
BOOL doADR(void *param, pDFSerial s);
BOOL doSend(void *param, pDFSerial s);
BOOL doRecv(void *param, pDFSerial s);
BOOL doSleep(void *param, pDFSerial s);
BOOL doPname(void *param, pDFSerial s);
BOOL doRejoin(void *param, pDFSerial s);
BOOL doBaud(void *param, pDFSerial s);
BOOL doATChan(void *param, pDFSerial s);
BOOL doManufacturer(void *param, pDFSerial s);
BOOL doProduct(void *param, pDFSerial s);
BOOL doSWVersion(void *param, pDFSerial s);
//BOOL doHWVersion(void *param, pDFSerial s);
BOOL doSnr(void *param, pDFSerial s);
BOOL doRssi(void *param, pDFSerial s);
BOOL doSN(void *param, pDFSerial s);
BOOL doRecovery(void *param, pDFSerial s);
BOOL doReboot(void *param, pDFSerial s);
BOOL doUpgrade(void *param, pDFSerial s);
BOOL doRS485Baud(void *param, pDFSerial s);
BOOL doRs485send(void *param, pDFSerial s);
BOOL doLora485(void *param, pDFSerial s);
BOOL doRs485Retry(void *param, pDFSerial s);
BOOL doRs485Item(void *param, pDFSerial s);
BOOL doOutpower(void *param, pDFSerial s);
BOOL doHwversion(void *param, pDFSerial s);
//BOOL doSWVersion(void *param, pDFSerial s);
BOOL doMD5(void *param, pDFSerial s);
BOOL doAdc(void *param, pDFSerial s);
BOOL doIoOut(void *param, pDFSerial s);
BOOL doIoOut2(void *param, pDFSerial s);
BOOL doIoIn(void *param, pDFSerial s);
BOOL doLOSS(void *param, pDFSerial s);
BOOL doLog(void *param, pDFSerial s);
BOOL doIocnt(void *param, pDFSerial s);
BOOL doIocnt2(void *param, pDFSerial s);
BOOL doCoord(void *param, pDFSerial s);

BOOL doBattery(void *param, pDFSerial s);
BOOL doPasswd(void *param, pDFSerial s);
BOOL doName(void *param, pDFSerial s);
BOOL doLoop(void *param, pDFSerial s);
uint8_t readData(char* data, uint8_t length);
void writeData(const char* data, int length);

BOOL doPdata(void *param, pDFSerial s);
BOOL doSerialnum(void *param, pDFSerial s);
BOOL doMacVersion(void *param, pDFSerial s);
BOOL doPid(void *param, pDFSerial s);
BOOL doVid(void *param, pDFSerial s);
BOOL doUnixEpoch(void *param, pDFSerial s);
BOOL doCSV(void *param, pDFSerial s);
BOOL doUDISK(void *param, pDFSerial s);
BOOL doUartPort(void *param, pDFSerial s);

BOOL doUartMode(void *param, pDFSerial s);
BOOL doBC(void *param, pDFSerial s);
BOOL doADN(void *param, pDFSerial s);

void softReset();
BOOL getMcuSleep();
BOOL resetFactoryData();
void ioOut1(void);
void ioOut2(void);
void IOinit(void);
//float getBattery();
const char* arrtoStr(uint8_t data[], uint8_t num);
const char* inttoStr(uint8_t data);
bool setSubBand(uint8_t subBand);
bool isUartConfigValid(int baudRates,sUartConfig_t config);
void uart1Print(const char *data);
void uart1Write8(uint8_t data);
void uart1Write16(uint16_t data);
void uart1Flush(void *data);
int mapVoltageToRange(float voltage,int minValue,int maxValue);
void USB1Print(const char *data);
void USB1Write8(uint8_t data);
void USB1Write16(uint16_t data);
void USB1Flush(void *data);
void USB2Print(const char *data);
void USB2Write8(uint8_t data);
void USB2Write16(uint16_t data);
void USB2Flush(void *data);
void floatToString(float number, char *str, int precision);
void NFCPrint(void * data);
uint8_t getloraMaxL();
int getTotalDataLength();
void LORAWANPrint(const char *data);
void LORAWANWrite8(uint8_t data);
void LORAWANWrite16(uint16_t data);
void LORAWANFlush(void *data);
void sysLog(const char *data);
void intMessage(const char *data);
uint8_t packetDequeue(uint8_t maxN, uint8_t *buffer);
uint8_t packetDequeue(uint8_t len, uint8_t *result);
BOOL ttlData();
void clearTTL();
void transToTTL(uint8_t *buffer,uint16_t size);
uint8_t  getBatteryLevel();
nfcConfig_t flashToNfc();
void setDataRate(uint8_t dr);
void nfcToFlash(nfcConfig_t nfcConfig);
uint32_t burn_dfbc_file(const void *buf,uint32_t index, uint32_t len);
#define NEW_RS485

#endif
