#include <stdio.h>
#include <time.h>
#include "LoRaMac.h"
TimerEvent_t flashWriteTimer;
#define DF_BOOT_SIZE   8*1024
#include "ff.h"
#include <log.h>
#ifdef  FLASH_ROM_LOG
LogBuffer logBuffer; // 全局变量

#elif defined (SPI_FLASH_LOG)
FATFS fs;                                                   // FatFs文件系统对象
FIL fnew;                                                   // 文件对象
FRESULT res_flash;                                          // 文件操作结果
UINT fnum;                                                  // 文件成功读写数量

volatile uint8_t writeFlag = 0;

const void fatFsWrite(void* context)
{
    writeFlag = 1;

}
#define UDISK_CODE_END_ADDR   0x00032000
#define jumpUDISK            ((void (*)(void))((int *)UDISK_CODE_END_ADDR))


FRESULT open_append (
    FIL* fp,            /* [OUT] File object to create */
    const char* path    /* [IN]  File name to be opened */
)
{
    FRESULT fr;

    /* Opens an existing file. If not exist, creates a new file. */
    fr = f_open(fp, path, FA_WRITE | FA_OPEN_ALWAYS);
    if (fr == FR_OK) {
        /* Seek to end of the file to append data */
        fr = f_lseek(fp, f_size(fp));
        if (fr != FR_OK)
            f_close(fp);
    }

    return fr;
}

#else

#endif
// 初始化日志缓冲区
void initLog() {
#ifdef  FLASH_ROM_LOG
    NvmmRead(&logBuffer,sizeof(logBuffer),DF_BOOT_SIZE+2048);
    if(logBuffer.head  == logBuffer.tail){
      logBuffer.head = 0;
      logBuffer.tail = 0;
      logBuffer.count = 0;
    }

    if(logBuffer.count >= 32){
        logBuffer.count = 0;
    }
    return ;

#elif defined (SPI_FLASH_LOG)
    res_flash = f_mount( &fs, "0:", 1 );
    dtuConfig.logPrint = 1;
    if( res_flash == FR_NO_FILESYSTEM )                     // 没有文件系统
    {
        printf("no fatfs: ");
        DelayMs(100);
        jumpUDISK();
        //BYTE work[FF_MAX_SS];
        //res_flash=f_mkfs( "0:", 0, work, sizeof(work) );    // 格式化 创建创建文件系统
        if( res_flash == FR_OK )
        {
            sysLog("Successful formatting\r\n");
            res_flash = f_mount( NULL, "0:", 1 );           // 格式化后，先取消挂载
            res_flash = f_mount( &fs,  "0:", 1 );           // 重新挂载
        }
        else
        {
            sysLog("Formatting failure\r\n");
            return;
        }
    }

    res_flash = f_mkdir( "logDir1" );

    //f_mount( NULL, "0:", 1 );                                                                       // 卸载文件系统




    TimerInit(&flashWriteTimer,fatFsWrite);
    TimerSetValue( &flashWriteTimer, 3000);
    res_flash= open_append(&fnew, "0:logDir1/logFile1.txt");

#else

#endif
}
// 将毫秒转换为时分秒格式
//__attribute__((section(".highcode")))
void formatTime(unsigned int milliseconds, char* buffer)
{

#ifdef  FLASH_ROM_LOG
    unsigned int hours   =  milliseconds / 3600000;
    unsigned int minutes = (milliseconds / 60000) % 60;
    unsigned int seconds = (milliseconds / 1000) % 60;
    snprintf(buffer, 9, "%02u:%02u:%02u", hours, minutes, seconds);



#elif defined (SPI_FLASH_LOG)
    //if(usbIsConnection()==1){
   //   time_t timestamp = dtuConfig.hardPara.unixEpoch;  // 用你的UNIX时间戳替换这个示例值
   //   struct tm *tm_info;
   //   char date_string[20];  // 用于存储日期时间字符串的缓冲区
   //   tm_info = localtime(&timestamp);
   //   strftime(buffer, 20, "%Y-%m-%d %H:%M:%S", tm_info);  // 你可以根据需要使用不同的日期时间格式
   // }else{
        unsigned int hours   =  milliseconds / 3600000;
        unsigned int minutes = (milliseconds / 60000) % 60;
        unsigned int seconds = (milliseconds / 1000) % 60;
        snprintf(buffer, 9, "%02u:%02u:%02u", hours, minutes, seconds);

  //  }
#else


#endif

}

// 写入日志
//__attribute__((section(".highcode")))
void writeLog(const char* message, unsigned int systemTime) {

    char formattedTime[30];
    //printf("count=%d\n",logBuffer.count);
    formatTime(systemTime, formattedTime);
    if(logBuffer.count >= 32){
        logBuffer.count = 1;
    }
    if(dtuConfig.logPrint == 0) return;
    char str2[128];
#ifdef  FLASH_ROM_LOG
    if (logBuffer.count >= LOG_BUFFER_SIZE) {
        logBuffer.head = (logBuffer.head + 1) % LOG_BUFFER_SIZE;
        logBuffer.count--;
    }




    int length = strlen(message);
    if (length > MAX_LOG_LENGTH) {
        length = MAX_LOG_LENGTH;
    }

    snprintf(logBuffer.entries[logBuffer.tail], MAX_LOG_LENGTH + 1, "%s: %s", formattedTime, message);
    logBuffer.tail = (logBuffer.tail + 1) % LOG_BUFFER_SIZE;
    logBuffer.count++;

    NvmmWrite(&logBuffer,sizeof(LogBuffer),2048+2048+DF_BOOT_SIZE);
    return ;
#elif defined (SPI_FLASH_LOG)
    //res_flash = f_mount( &fs, "0:", 1 );
    //res_flash = f_open( &fnew, "0:logDir1/logFile1.txt",FA_WRITE|FA_OPEN_ALWAYS );
    //res_flash= open_append(&fnew, "0:logDir1/logFile1.txt");
    memset(str2,0,128);
    snprintf(str2, strlen(formattedTime) + strlen(message)+6, "[%s]: %s", formattedTime, message);
    // 以写入方式打开文件，若未发现文件则新建文件

    if( res_flash == FR_OK )
    {
        //printf("str=%d\n",strlen(str2));
        res_flash = f_write( &fnew, str2, strlen(str2), &fnum );                      // 写向文件内写入指定数据

       if( res_flash == FR_OK )
       {
       //     sysLog("[spi]write success\n");

           TimerStart( &flashWriteTimer );
       }else{

            sysLog("[spi]write error\n");
       }


    }else{
        sprintf(str2,"error=%d\n",res_flash);
        sysLog(str2);
    }
    memset(str2,0,128);
    //TimerStop( &flashWriteTimer );

   // f_close(&fnew);
   // f_mount( NULL, "0:", 1 );
#else

#endif
}
#ifdef  FLASH_ROM_LOG
// 查询日志
LogBuffer* queryLog() {
    return &logBuffer;
}
#endif

void resetLog(){

#ifdef  FLASH_ROM_LOG
    logBuffer.head = 0;
    logBuffer.tail = 0;
    logBuffer.count = 0;
    NvmmWrite(&logBuffer,sizeof(LogBuffer),2048+2048+DF_BOOT_SIZE);
#endif

}
void deletLogFile(){


#if defined (SPI_FLASH_LOG)
    res_flash = f_unlink( "0:logDir1/logFile1.txt");
    //printf("del=%d\n",res_flash);
#endif
}
//__attribute__((section(".highcode")))
void SDacceWrite(){
  #if defined (SPI_FLASH_LOG)
  if(writeFlag == 1){
      f_close(&fnew);
      res_flash= open_append(&fnew, "0:logDir1/logFile1.txt");
      TimerStop( &flashWriteTimer );
      writeFlag = 0;
  }
  #endif
}
void closeFalse(){
   #if defined (SPI_FLASH_LOG)
    f_close(&fnew);
   // res_flash= open_append(&fnew, "0:logDir1/logFile1.txt");
   #endif
}
void openFalse(){
   #if defined (SPI_FLASH_LOG)
    res_flash= open_append(&fnew, "0:logDir1/logFile1.txt");
   #endif
}
