#include "NDEF.h"

uint8_t recordType[NDEF_RECORD_TYPE_BUFFER_SIZE];
uint8_t recordId[NDEF_RECORD_ID_BUFFER_SIZE];
uint8_t recordPayload[NDEF_RECORD_PAYLOAD_BUFFER_SIZE];

const uint8_t TLVTerm[0x02] = {TAG_TERMINATOR, 0};

eNDEFStatus_t NDEF_Init(sNDEFTLV_t *pMsg)
{
    uint8_t *pTLVBuf = pMsg->TLVBuf;
    // 获取 NDEF Record 长度
    size_t recordLength = 0, headerLength = 0, payloadLength = 0;
    {
        recordLength++;
        recordLength += (pMsg->record.typeLength + 1);
        if (pMsg->record.shortRecord)
        {
            recordLength += (pMsg->record.payloadLength + 1);
        }
        else
        {
            recordLength += (pMsg->record.payloadLength + 4);
        }
        if (pMsg->record.idLengthFlag)
        {
            recordLength += (pMsg->record.idLength + 1);
        }

        payloadLength = pMsg->record.payloadLength;
        headerLength  = recordLength - payloadLength;

        if (payloadLength > NDEF_RECORD_PAYLOAD_BUFFER_SIZE)
        {
            return NDEF_ERROR_BUFFER_OVERFLOW;
        }
    }

    // 初始化 NDEF Msg TLV
    {
        if (recordLength >= 0xFF)
        {
            *pTLVBuf++   = TAG_NDEF_MSG;
            *pTLVBuf++   = 0xFF;
            *pTLVBuf++   = (recordLength & 0xFF00) >> 8;
            *pTLVBuf++   = (recordLength & 0x00FF);
            pMsg->TLVLen = recordLength + 4;
        }
        else
        {
            *pTLVBuf++   = TAG_NDEF_MSG;
            *pTLVBuf++   = (recordLength & 0x00FF);
            pMsg->TLVLen = recordLength + 2;
        }
    }

    // 初始化 NDEF Record Header
    {
        // Flags
        *pTLVBuf = 0;
        *pTLVBuf |= (pMsg->record.messageBegin) << 7;
        *pTLVBuf |= (pMsg->record.messageEnd) << 6;
        *pTLVBuf |= (pMsg->record.chunkFlag) << 5;
        *pTLVBuf |= (pMsg->record.shortRecord) << 4;
        *pTLVBuf |= (pMsg->record.idLengthFlag) << 3;
        *pTLVBuf++ |= pMsg->record.typeNameFormat;
        // Type Length
        *pTLVBuf++ = pMsg->record.typeLength;
        // Payload Length
        if (pMsg->record.shortRecord)
        {
            *pTLVBuf++ = pMsg->record.payloadLength;
        }
        else
        {
            *pTLVBuf++ = (pMsg->record.payloadLength) >> 24;
            *pTLVBuf++ = ((pMsg->record.payloadLength) >> 16) & 0xFF;
            *pTLVBuf++ = ((pMsg->record.payloadLength) >> 8) & 0xFF;
            *pTLVBuf++ = (pMsg->record.payloadLength) & 0xFF;
        }
        // ID Length & ID & Type
        if (pMsg->record.idLengthFlag)
        {
            *pTLVBuf++ = pMsg->record.idLength;
            for (uint8_t i = 0; i < pMsg->record.typeLength; i++)
            {
                pMsg->record.type = pTLVBuf;
                *pTLVBuf++        = *(recordType + i);
            }
            for (uint8_t i = 0; i < pMsg->record.idLength; i++)
            {
                pMsg->record.id = pTLVBuf;
                *pTLVBuf++      = *(recordId + i);
            }
        }
        else
        {
            for (uint8_t i = 0; i < pMsg->record.typeLength; i++)
            {
                pMsg->record.type = pTLVBuf;
                pMsg->record.id   = NULL;
                *pTLVBuf++        = *(recordType + i);
            }
        }
    }
    // 初始化 NDEF 负载
    {
        pMsg->record.payload = pTLVBuf;
        memcpy(pTLVBuf, recordPayload, pMsg->record.payloadLength);
        pTLVBuf += pMsg->record.payloadLength;
    }

    return NDEF_SUCCESS;
}

eNDEFStatus_t NDEF_Parsing(sNDEFTLV_t *pMsg)
{
    // 只会解析 NDEF 的 TLV，需要上层检测 CC File 和 Term
    uint8_t *pDat = pMsg->TLVBuf;
    // TLV
    size_t i;
    if (*pDat != 03)
    {
        return NDEF_ERROR_ILLEGAL_FORMAT;
    }
    else
    {
        pDat++;
    }

    size_t recordLength;
    if (*pDat == 0xFF)
    {
        pDat++;
        recordLength = (((uint16_t)*pDat++) << 8) + (*pDat++);
        pMsg->TLVLen = recordLength + 2;
    }
    else
    {
        recordLength = *pDat++;
        pMsg->TLVLen = recordLength + 2;
    }
   // PRINT("Record Length: %u\r\n", recordLength);

    // Record
    memset(&pMsg->record, 0, sizeof(sNDEFRecord_t));
    // Flags
    pMsg->record.messageBegin   = (*pDat & 0x80) >> 7;
    pMsg->record.messageEnd     = (*pDat & 0x40) >> 6;
    pMsg->record.chunkFlag      = (*pDat & 0x20) >> 5;
    pMsg->record.shortRecord    = (*pDat & 0x10) >> 4;
    pMsg->record.idLengthFlag   = (*pDat & 0x08) >> 3;
    pMsg->record.typeNameFormat = *pDat & 0x07;
    pDat++;
    // Type Length
    pMsg->record.typeLength = *pDat++;
    // Payload Length

    if (pMsg->record.shortRecord)
    {
        pMsg->record.payloadLength = *pDat++;
    }
    else
    {
        pMsg->record.payloadLength = (((size_t)(*pDat++)) << 24) + (((size_t)(*pDat++)) << 16) +
                                     (((size_t)(*pDat++)) << 8) + ((size_t)(*pDat++));
    }
    // ID Length & ID & Type
    if (pMsg->record.idLengthFlag)
    {
        pMsg->record.idLength = *pDat++;

        pMsg->record.type = pDat;
        pDat += pMsg->record.typeLength;
        pMsg->record.id = pDat;
        pDat += pMsg->record.idLength;
    }
    else
    {
        pMsg->record.type = pDat;
        pDat += pMsg->record.typeLength;
    }
    // Payload
    pMsg->record.payload = pDat;

    return NDEF_SUCCESS;
}
