#ifndef NDEF_H
#define NDEF_H

#include "CH58x_common.h"
#include <string.h>

#define EMPTY                                 0x00
#define WELL_KNOWN_TYPE                       0x01
#define MULTIPURPOSE_INTERNET_MAIL_EXTENSIONS 0x02
#define ABSOLUTE_URI                          0x03
#define EXTERNAL_TYPE                         0x04
#define UNKNOW                                0x05
#define UNCHANGED                             0x06
#define RESERVED                              0x07

#define TAG_NDEF_MSG    0x03
#define TAG_PROPRIETARY 0xFD
#define TAG_TERMINATOR  0xFE

#define NDEF_RECORD_ID_BUFFER_SIZE      8
#define NDEF_RECORD_TYPE_BUFFER_SIZE    4
#define NDEF_RECORD_PAYLOAD_BUFFER_SIZE 868
#define NDEF_TLV_BUFFER_SIZE            872

typedef enum
{
    NDEF_SUCCESS,
    NDEF_ERROR,
    NDEF_ERROR_ILLEGAL_FORMAT,
    NDEF_ERROR_BUFFER_OVERFLOW,
    NDEF_ERROR_MISMATCHED_TLV_LENGTH
} eNDEFStatus_t;

typedef struct
{
    uint8_t  messageBegin;
    uint8_t  messageEnd;
    uint8_t  chunkFlag;
    uint8_t  shortRecord;
    uint8_t  idLengthFlag;
    uint8_t  typeNameFormat;
    uint8_t  typeLength;
    uint32_t payloadLength;
    uint8_t  idLength;
    uint8_t *type;
    uint8_t *id;
    uint8_t *payload;
} sNDEFRecord_t;

typedef struct
{
    sNDEFRecord_t record;
    uint8_t       TLVBuf[NDEF_TLV_BUFFER_SIZE];
    size_t        TLVLen;
} sNDEFTLV_t;

extern uint8_t recordType[NDEF_RECORD_TYPE_BUFFER_SIZE];
extern uint8_t recordId[NDEF_RECORD_ID_BUFFER_SIZE];
extern uint8_t recordPayload[NDEF_RECORD_PAYLOAD_BUFFER_SIZE];

extern const uint8_t TLVTerm[0x02];

// 初始化一个 NDEF 消息，只支持单个 Record
eNDEFStatus_t NDEF_Init(sNDEFTLV_t *pMsg);
// 解析一个 NDEF 消息，返回包含消息中的第一个 Record
eNDEFStatus_t NDEF_Parsing(sNDEFTLV_t *pMsg);

extern sNDEFTLV_t NDEFMsg;

#endif
