#ifndef NT3H_H
#define NT3H_H

#include "../src/app_i2c.h"
#include "NDEF_TEXT.h"

// #define PRINT_COMM // 控制是否打印 I2C 通信内容

extern uint8_t defaultCCFile[4];

#define NT3H_INT_PIN     GPIO_Pin_9                                   // PB9
#define NT3H_PWR_PIN     GPIO_Pin_1                                   // PA2
#define NT3H_POWER()     ((NT3H_PWR_PIN & R32_PB_OUT) ? TRUE : FALSE) // 检查 NT3H 电源状态
#define NT3H_POWER_ON()  GPIOB_SetBits(NT3H_PWR_PIN)                  // 开启 NT3H 的电源
#define NT3H_POWER_OFF() GPIOB_ResetBits(NT3H_PWR_PIN)                // 关闭 NT3H 的电源

#define NT3H_NXPID_LENGTH 7
#define NT3H_PWD_LENGTH   4
#define NT3H_PACK_LENGTH  2
#define NT3H_BLOCK_SIZE   16

#define NT3H_MAX_NDEF_LENGTH 840

#define NT3H_DEFAULT_BLOCK_INFO                                                                                        \
    {                                                                                                                  \
        0xAA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00                 \
    }

#define NT3H_DEFAULT_BLOCK_LOCK                                                                                        \
    {                                                                                                                  \
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF                 \
    }

#define NT3H_DEFAULT_BLOCK_PWD                                                                                         \
    {                                                                                                                  \
        0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00                 \
    }

#define NT3H_DEFAULT_BLOCK_CONF                                                                                        \
    {                                                                                                                  \
        0x01, 0x00, 0xF8, 0x48, 0x08, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00                 \
    }

#define NT3H_DEFAULT_CC_FILE_1K                                                                                        \
    {                                                                                                                  \
        0xE1, 0x10, 0x6A, 0x00                                                                                         \
    }

#define NT3H_I2C_ADDR    0x55
#define NT3H_I2C_BUFSIZE (NT3H_BLOCK_SIZE + 1)
#define NT3H_I2C_TIMEOUT 1000

#define BIT(x) (1 << x)

typedef struct NT3H_config
{
    uint8_t FD_on_evt;
    uint8_t FD_off_evt;

    void (*onRXConfig)(uint8_t *dat, size_t len);
    void (*onRxPthru)(uint8_t *dat, size_t len);

} NT3H_config_t;

enum NT3H_ERR_NUM
{
    NT3H_SUCCESS                     = 0,
    NT3H_ERR_E2P_NOTREADY            = -1,
    NT3H_ERR_BUF_OVERFLOW            = -2,
    NT3H_ERR_E2P_OVERFLOW            = -3,
    NT3H_ERR_ILLEGAL_MEMADRR         = -4,
    NT3H_ERR_SET_REG_FAILURE         = -5,
    NT3H_ERR_SET_E2P_FAILURE         = -6,
    NT3H_ERR_ILLEGAL_NXP_ID          = -7,
    NT3H_ERR_NDEF_UNDETECTED         = -8,
    NT3H_ERR_NDEF_PARSING_FAILURE    = -9,
    NT3H_ERR_NDEF_MSG_INIT_FAILURE   = -10,
    NT3H_ERR_I2C_TRANSMITTED_FAILURE = -11,

};

enum NT3H_BLOCKS
{

    NT3H_BLOCK_INFO       = 0x00,
    NT3H_BLOCK_USER0      = 0x01,
    NT3H_BLOCK_USER0_TERM = 0x37,
    NT3H_BLOCK_LOCK       = 0x38,
    NT3H_BLOCK_PWD        = 0x39,
    NT3H_BLOCK_CONF       = 0x3A,
    NT3H_BLOCK_USER1      = 0x40,
    NT3H_BLOCK_USER1_TERM = 0x7F,
    NT3H_BLOCK_SRAM       = 0xF8,
    NT3H_BLOCK_SRAM_TERM  = 0xFB,
    NT3H_BLOCK_SESS       = 0xFE
};

enum NT3H_CONF_SESS_REGS
{
    NT3H_REG_NC_REG            = 0,
    NT3H_REG_LAST_NDEF_BLOCK   = 1,
    NT3H_REG_SRAM_MIRROR_BLOCK = 2,
    NT3H_REG_WDT_LS            = 3,
    NT3H_REG_WDT_MS            = 4,
    NT3H_REG_I2C_CLOCK_STR     = 5,
    NT3H_REG_REGNS_LOCK        = 6,
};

enum NT3H_AUTH_BYTES
{
    NT3H_AUTH_ACCESS = 0,
    NT3H_AUTH_PWD    = 4,
    NT3H_AUTH_PACK   = 8,
    NT3H_AUTH_PT_I2C = 12,
    NT3H_AUTH_AUTH   = 15
};

enum NT3H_FD_ON_EVT
{
    NT3H_FD_FIELD_OPEN          = 0 | 0,           // NFC 场开启
    NT3H_FD_COMMUNICATION_START = 0 | BIT(2),      // NFC 通信开始
    NT3H_FD_TAG_SELECTED        = BIT(3) | 0,      // NFC TAG 选中
    NT3H_FD_PT_DATA_READY       = BIT(3) | BIT(2), // 直通模式数据就绪
};

enum NT3H_FD_OFF_EVT
{
    NT3H_FD_FIELD_CLOSE     = 0 | 0,      // NFC 场关闭
    NT3H_FD_STATE_HALT      = 0 | BIT(4), // 标签为 HALT （通常意味着 NFC 终端完成了对标签的操作）
    NT3H_FD_LASTPAGE_READED = BIT(5) | 0, // 读取了寄存器 LAST_NDEF_BLOCK 中设置的页面
    NT3H_FD_PT_READ_COMPLET = BIT(5) | BIT(4), // 直通模式数据读取完毕
    // 当 NT3H_FD_ON_EVT 不为 NT3H_FD_PT_DATA_READY 时，NT3H_FD_PT_READ_COMPLET 与 NT3H_FD_FIELD_CLOSE 相同
};

enum NT3H_BIT_ACCESS
{
    NT3H_ACCESS_NFC_PORT     = BIT(7), // NFC 端口的密码验证              | 0：写入保护 | 1：读写保护 |
    NT3H_ACCESS_NFC_DIS_SEC1 = BIT(5), // NFC 扇区 1（0x40-0x7F）访问保护 | 0：可访问   | 1:不可访问  |
    // NT3H_ACCESS_AUTHLIM 密码最大尝试次数 2^(NT3H_PWD_AUTHLIM)
};

enum NT3H_BIT_PT_I2C
{
    NT3H_PT_2K_PORT   = BIT(3),         // 扇区 1 密码保护 | 0：禁用 | 1：开启 |
    NT3H_PT_SRAM_PORT = BIT(2),         // SRAM  密码保护  | 0：禁用 | 1：开启 |
    NT3H_PT_I2C_PORT  = BIT(1) | BIT(0) // I2C 访问密码保护区权 00：读写 01：只读 1X:不可读写
};

enum NT3H_PTHRU_DIR
{
    I2C_TO_NFC = 0,
    NFC_TO_I2C = 1
};

/***************************************************
 *                 Initialization
 ***************************************************/
int NT3H_init(void);              // 初始化中断和电源引脚
int NT3H_check_id(uint8_t *pBuf); // 检查制造商 ID，如果制造商 ID 正确且 pBuf != NULL，则在 pBuf 中返回 NXP ID。
int NT3H_check_cc(uint8_t *pDat); // 检查 CC File，如果与传入的 CC 不同，则用新配置覆盖。
int NT3H_FD_config(uint8_t onEvt, uint8_t offEvt, uint8_t lastNDEFPage); // 配置 FD 引脚电平事件，用于中断
int NT3H_auth(uint8_t auth, uint8_t access, uint8_t *pwd, uint8_t *pack,
              uint8_t pt);                                               // 将 auth 后的所有页加密

/***************************************************
 *                 Check NTag Status
 ***************************************************/
int NT3H_is_E2PROM_available(void);  // 检查E2PROM 就绪，可以写入
int NT3H_is_radio_field_exist(void); // 检查是否存在无线电场
int NT3H_is_RF_locked(void);         // 检查 RF_LOCK 寄存器，I2C 是否可用
int NT3H_is_write_ready(void);       // 检查 NTag 模式下是否可写

/***************************************************
 *                       NDEF
 ***************************************************/
int NT3H_write_NDEF(uint8_t memAddr, uint8_t *pDat, size_t len); // 写入一个 NDEF
int NT3H_read_NDEF(uint8_t memAddr, uint8_t *pBuf, size_t len);  // 读取并解析一个 NDEF

/***************************************************
 *                    Pass-through
 ***************************************************/
int NT3H_open_pthru(uint8_t dir);    // 开启一次透传通信，dir 为方向
int NT3H_close_pthru(void);          // 关闭透传
int NT3H_is_pthru_write_ready(void); // 检查是否可写
int NT3H_is_pthru_read_ready(void);  // 检查是否可读
int NT3H_pthru_write(uint8_t *pDat); // 透传写，一次 64 Bytes
int NT3H_pthru_read(uint8_t *pBuf);  // 透传读，一次 64 Bytes

/***************************************************
 *                       E2PROM
 ***************************************************/
int NT3H_write(uint8_t memAddr, uint8_t *pDat, size_t len);
int NT3H_read(uint8_t memAddr, uint8_t *pBuf, size_t len);
int NT3H_write_block(uint8_t memAddr, uint8_t *pDat, size_t len);
int NT3H_read_block(uint8_t memAddr, uint8_t *pBuf, size_t len);
int NT3H_write_reg(uint8_t regAddr, uint8_t regMask, uint8_t regData);
int NT3H_read_reg(uint8_t regAddr, uint8_t *pBuf);
int NT3H_clear_blocks(uint8_t memAddr, uint8_t endMema); // 清理 A 到 B 之间的所有 Block

/***************************************************
 *                     Debug Tools
 ***************************************************/
void printMem(uint8_t beginAddr, uint8_t endAddr);
void printBlock(uint8_t *pBuf, size_t len);
#endif
