#include "NT3H_COMM.h"

static eNT3HState_t state;
static sNT3HCB_t    callbacks;
static sNT3HFlags_t flags;

void nfcInit(sNT3HCB_t *cbs)
{
    state              = NFC_IDLE;
    flags.interrupt    = false;
    flags.switch2Read  = false;
    flags.switch2Write = false;
    memcpy(&callbacks, cbs, sizeof(sNT3HCB_t));
    uint8_t nxpId[7];
    NT3H_init();
    NT3H_POWER_ON();
    DelayMs(10);

    // 检查 ID
    while (NT3H_SUCCESS != NT3H_check_id(nxpId))
    {
        PRINT("Check ID Fail, Retry!\r\n");
        DelayMs(1000);
    }

    // 写入 CC File
    uint8_t CCFile[4] = NT3H_DEFAULT_CC_FILE_1K;
    while (NT3H_SUCCESS != NT3H_check_cc(CCFile))
    {
        PRINT("Init CC File Fail, Retry!\r\n");
        DelayMs(1000);
    }

    // 配置 FD 引脚事件
    while (NT3H_SUCCESS != NT3H_FD_config(NT3H_FD_COMMUNICATION_START, NT3H_FD_STATE_HALT, 0x37))
    {
        PRINT("Init FDPin Fail, Retry!\r\n");
        DelayMs(1000);
    }

    // 配置加密
    uint8_t password[NT3H_PWD_LENGTH], pack[NT3H_PACK_LENGTH];
    PWDGenerater(nxpId, password, pack);
    while (NT3H_SUCCESS != NT3H_auth(0xFE, NT3H_ACCESS_NFC_PORT, password, pack, 0))
    {
        PRINT("Init Fail, Retry!\r\n");
        DelayMs(1000);
    }

    // 清空虚拟寄存器
    while (NT3H_SUCCESS != NT3H_clear_blocks(0x37, 0x37))
    {
        PRINT("Clean fail, retrying\r\n");
        DelayMs(1000);
    }

    // 完成，关闭电源
    NT3H_POWER_OFF();
    PRINT("Init Done\r\n");
}

void processNfc(void)
{
}

void nfcFSM(void)
{
    switch (state)
    {
    case NFC_IDLE:
        // 触发中断时，切换到 NTAG 状态
        if (TRUE == flags.interrupt)
        {
            flags.interrupt = FALSE;
            NT3H_POWER_ON();
            DelayMs(10);
            if (NT3H_is_radio_field_exist())
            {
                PRINT("IDLE->NTAG\r\n");
                state = NFC_NTAG;
            }
        }
        break;
    case NFC_NTAG:
        // NTAG 状态下，轮询 0x37 Block，当检测到标志位变化时，切换至相应的模式
        if (NT3H_is_radio_field_exist())
        {
            uint8_t recvBuf[16];
            NT3H_read_block(0x37, recvBuf, 16);
            DelayMs(100);
        }
        else
        {
            PRINT("NTAG->IDLE\r\n");
            state = NFC_IDLE;
        }
        break;
    case NFC_NTAG_HALT:
        // HALT 状态将等待 NFC 离场后读取配置信息
        if (NT3H_is_radio_field_exist())
        {
            // 不需要做任何事
        }
        else
        {
            uint8_t buf[NT3H_MAX_NDEF_LENGTH] = {0};
            size_t  length                    = NT3H_read_NDEF(NT3H_BLOCK_USER0, buf, sizeof(buf));
            if (length > 0)
            {
                callbacks.onRxNTag(buf, length);
            }
            PRINT("HALT->IDLE\r\n");
            state = NFC_IDLE;
        }
        break;
    case NFC_PTHRU_READ:
        // 接收透传数据
        if (NT3H_is_radio_field_exist())
        {
            if (TRUE == flags.interrupt)
            {
                flags.interrupt = FALSE;
                if (NT3H_is_pthru_read_ready())
                {
                    uint8_t buf[64];
                    NT3H_pthru_read(buf);
                    callbacks.onRxPthru(buf);
                    if (TRUE == flags.switch2Read)
                    {
                        flags.switch2Read = FALSE;
                        NT3H_close_pthru();
                        NT3H_open_pthru(I2C_TO_NFC);
                        PRINT("PTHRU_READ->PTHRU_WRITE\r\n");
                        state = NFC_PTHRU_WRITE;
                    }
                    printBlock(buf, sizeof(buf));
                }
            }
        }
        else
        {
            PRINT("PTHRU_READ->IDLE\r\n");
            state = NFC_IDLE;
        }
        break;
    case NFC_PTHRU_WRITE:
        if (NT3H_is_radio_field_exist())
        {
            if (TRUE == flags.interrupt)
            {
                if (NT3H_is_pthru_write_ready())
                {
                    // Todo: 发送队列或缓存中的数据
                    PRINT("PTHRU SEND COMPLETED\r\n");
                    if (TRUE == flags.switch2Read)
                    {
                        flags.switch2Read = FALSE;
                        NT3H_close_pthru();
                        NT3H_open_pthru(NFC_TO_I2C);
                        PRINT("PTHRU_READ->PTHRU_WRITE\r\n");
                        state = NFC_PTHRU_WRITE;
                    }
                }
            }
        }
        else
        {
            PRINT("PTHRU_READ->IDLE\r\n");
            state = NFC_IDLE;
        }
        break;
    }
}
