#include "flash.h"

int _FLASH_SS;
uint8_t       SPI_FLASH_BUF[512];

void flashInit()
{
  GPIOB_SetBits(GPIO_Pin_18);
  GPIOB_ModeCfg(GPIO_Pin_18 , GPIO_ModeOut_PP_5mA);
  GPIOA_ModeCfg(GPIO_Pin_0 | GPIO_Pin_1, GPIO_ModeOut_PP_5mA);
  //SPI1_DataMode(Mode0_LowBitINFront);
  SPI1_CLKCfg(256);
  SPI1_MasterDefInit();
  releasePowerDown();
  writeDisable();
 // printf("id=%d\n",manufacturerID());
  if(manufacturerID() == 0xEF){
      spiFlashWork = TRUE;
  }else{
      spiFlashWork = FALSE;
  }
}


void write(unsigned int page, int pageAddress, int val){


    uint8_t spiBuff[5];
    writeEnable();
    GPIOB_ResetBits(GPIO_Pin_18);
    spiBuff[0] = PAGE_PROGRAM;
    spiBuff[1] =(page >> 8) & 0xFF;
    spiBuff[2] = (page >> 0) & 0xFF;
    spiBuff[3] = pageAddress;
    spiBuff[4] = val;
    SPI1_MasterTrans(spiBuff,5);
    GPIOB_SetBits(GPIO_Pin_18);

    notBusy();
    writeDisable();
}

uint8_t read(unsigned int page, int pageAddress){
    uint8_t spiBuff[4];
    uint8_t spiBuffrev[2];
    GPIOB_ResetBits(GPIO_Pin_18);


    spiBuff[0] = READ_DATA;
    spiBuff[1] = (page >> 8) & 0xFF;
    spiBuff[2] = (page >> 0) & 0xFF;
    spiBuff[3] = pageAddress;
    SPI1_MasterTrans(spiBuff,4);
    uint8_t val = SPI1_MasterRecvByte();
    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();
    return val;
}
void chipErase() {
  writeEnable();
  GPIOB_ResetBits(GPIO_Pin_18);
  SPI1_MasterSendByte(CHIP_ERASE);
  GPIOB_SetBits(GPIO_Pin_18);
  notBusy();
  writeDisable();
}
void releasePowerDown()
{


    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(RELEASE_POWER_DOWN);
    GPIOB_SetBits(GPIO_Pin_18);


    notBusy();

}

void spiFlashPowerDown(){

    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(POWER_DOWN);
    GPIOB_SetBits(GPIO_Pin_18);
    DelayMs(1);
    //notBusy();
}

uint8_t manufacturerID(){
    uint8_t spiBuff[4];
    uint8_t spiBuffrev[2];
    GPIOB_ResetBits(GPIO_Pin_18);
    spiBuff[0] = MANUFACTURER_ID;
    spiBuff[1] = 0;
    spiBuff[2] = 0;
    spiBuff[3] = 0;
    SPI1_MasterTrans(spiBuff,4);
    SPI1_MasterRecv(spiBuffrev,2);
    GPIOB_SetBits(GPIO_Pin_18);


    return spiBuffrev[0];
}

void notBusy(){


      GPIOB_ResetBits(GPIO_Pin_18);
      SPI1_MasterSendByte(READ_STATUS_REGISTER_1);

      uint8_t state = SPI1_MasterRecvByte();
      uint8_t timeout = 100;
      while (state& 1) {
          DelayMs(1);
          state = SPI1_MasterRecvByte();
          timeout--;
          if(timeout == 0)break;
      }
      GPIOB_SetBits(GPIO_Pin_18);


}
void writeEnable(){
    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(WRITE_ENABLE);
    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();
}
void writeDisable(){

    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(WRITE_DISABLE);
    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();

}
void readFlash(uint32_t addr,void*pBuf,uint16_t len)
{
    GPIOB_ResetBits(GPIO_Pin_18);

    if((addr > 0xFFFFFF)||(pBuf == NULL)||(len == 0)) return;
    uint8_t *buf = (uint8_t *)pBuf;

    SPI1_MasterSendByte(READ_DATA);
    SPI1_MasterSendByte((uint8_t)(addr>>16));
    SPI1_MasterSendByte((uint8_t)(addr>>8));
    SPI1_MasterSendByte((uint8_t)(addr));
    for(uint16_t i = 0; i < len; i++){
        buf[i] = SPI1_MasterRecvByte();
    }


    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();
}
void writeFlash(uint32_t addr,void*pBuf,uint16_t len)
{

    if((addr > W25Q_BLOCK_ADDR_END)||(pBuf == NULL) || (len == 0)) return;
    uint8_t *buf = (uint8_t *)pBuf;
    uint16_t left = len;
    while(left){
        if(left > W25Q_PAGE_SIZE){

            len = W25Q_PAGE_SIZE ;
        }else{

            len = left;
        }

        writeEnable();
        GPIOB_ResetBits(GPIO_Pin_18);
        SPI1_MasterSendByte(PAGE_PROGRAM);
        SPI1_MasterSendByte((uint8_t)(addr>>16));
        SPI1_MasterSendByte((uint8_t)(addr>>8));
        SPI1_MasterSendByte((uint8_t)(addr));
        for(uint16_t i = 0; i < len; i++){
            SPI1_MasterSendByte(buf[i]);
        }
        GPIOB_SetBits(GPIO_Pin_18);
        notBusy();
        buf += len;
        left -= len;
        addr += len;
    }

}

void eraseSector(uint32_t addr, BOOL isFirstAddr)
{
  if((addr > W25Q_BLOCK_ADDR_END)||(isFirstAddr && (addr%W25Q_SECTOR_SIZE)))
      return;

  writeEnable();
  GPIOB_ResetBits(GPIO_Pin_18);
  SPI1_MasterSendByte(W25Q_CMD_SECTOR_ERASE);
  SPI1_MasterSendByte((uint8_t)(addr>>16));
  SPI1_MasterSendByte((uint8_t)(addr>>8));
  SPI1_MasterSendByte((uint8_t)(addr));
  GPIOB_SetBits(GPIO_Pin_18);


}

void SPI_Flash_Erase_Sector(uint32_t Dst_Addr)
{
   // Dst_Addr *= 4096;
    writeEnable();;
    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(W25Q_CMD_SECTOR_ERASE);
    SPI1_MasterSendByte((uint8_t)((Dst_Addr) >> 16));
    SPI1_MasterSendByte((uint8_t)((Dst_Addr) >> 8));
    SPI1_MasterSendByte((uint8_t)Dst_Addr);
    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();
}


void SPI_Flash_Read(uint8_t *pBuffer, uint32_t ReadAddr, uint16_t size)
{
    uint16_t i;

    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(READ_DATA);
    SPI1_MasterSendByte((uint8_t)((ReadAddr) >> 16));
    SPI1_MasterSendByte((uint8_t)((ReadAddr) >> 8));
    SPI1_MasterSendByte((uint8_t)ReadAddr);

    for(i = 0; i < size; i++)
    {
        pBuffer[i] = SPI1_MasterRecvByte();
    }

    GPIOB_SetBits(GPIO_Pin_18);

}

void SPI_Flash_Write_Page(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t size)
{
    uint16_t i;

    writeEnable();
    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(PAGE_PROGRAM);
    SPI1_MasterSendByte((uint8_t)((WriteAddr) >> 16));
    SPI1_MasterSendByte((uint8_t)((WriteAddr) >> 8));
    SPI1_MasterSendByte((uint8_t)WriteAddr);

    for(i = 0; i < size; i++)
    {
        SPI1_MasterSendByte(pBuffer[i]);
    }

    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();
}
void SPI_Flash_Write_NoCheck(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t size)
{
    uint16_t pageremain;

    pageremain = 256 - WriteAddr % 256;

    if(size <= pageremain)
        pageremain = size;

    while(1)
    {
        SPI_Flash_Write_Page(pBuffer, WriteAddr, pageremain);

        if(size == pageremain)
        {
            break;
        }
        else
        {
            pBuffer += pageremain;
            WriteAddr += pageremain;
            size -= pageremain;

            if(size > 256)
                pageremain = 256;
            else
                pageremain = size;
        }
    }
}


void SPI_Flash_Write(uint8_t *pBuffer, uint32_t WriteAddr, uint16_t size)
{
    uint32_t secpos;
    uint16_t secoff;
    uint16_t secremain;
    uint16_t i;

    secpos = WriteAddr / 512;
    secoff = WriteAddr % 512;
    secremain = 512 - secoff;

    if(size <= secremain)
        secremain = size;

    while(1)
    {
        SPI_Flash_Read(SPI_FLASH_BUF, secpos * 512, 512);

        for(i = 0; i < secremain; i++)
        {
            if(SPI_FLASH_BUF[secoff + i] != 0XFF)
                break;
        }

        if(i < secremain)
        {
            SPI_Flash_Erase_Sector(secpos);

            for(i = 0; i < secremain; i++)
            {
                SPI_FLASH_BUF[i + secoff] = pBuffer[i];
            }

            SPI_Flash_Write_NoCheck(SPI_FLASH_BUF, secpos * 512, 512);
        }
        else
        {
            SPI_Flash_Write_NoCheck(pBuffer, WriteAddr, secremain);
        }

        if(size == secremain)
        {
            break;
        }
        else
        {
            secpos++;
            secoff = 0;

            pBuffer += secremain;
            WriteAddr += secremain;
            size -= secremain;

            if(size > 512)
            {
                secremain = 512;
            }
            else
            {
                secremain = 512;
            }
        }
    }
}

void SPI_Flash_Erase_Chip(void)
{
    writeEnable();
    GPIOB_ResetBits(GPIO_Pin_18);
    SPI1_MasterSendByte(CHIP_ERASE);
    GPIOB_SetBits(GPIO_Pin_18);
    notBusy();
    writeDisable();
}


