#include "dfbc_vm.h"
#include "comm.h"
#include <sys/time.h>   // fixme: get current time

/**************************************************
 *                LOCAL VARIABLES
 **************************************************/
static btcd_t btcd={0};
static venv_t venv={0};


struct timeval st={0};

/**************************************************
 *               LOCAL DECLARATION
 **************************************************/

// static void parse(btcd_t *bc);
// static void call(venv_t *vm, btcd_t *bc, int fnIdx, uint32_t argv);

/**************************************************
 *                   FUNCTIONS
 **************************************************/
bool dfbc_init(void) {
    // init buffer
    uint8_t *code = DFBC_ADDR;
    uint16_t size = code[0] + code[1]*256 + 4;
    btcd_t *bc = &btcd;
    SG_INIT(bc->segm);

    // free buffer if reinit
    if (SG_GET_HEAD(GLOB_SEGM) != NULL) {
        free(SG_GET_HEAD(GLOB_SEGM));
    }
    if (bc->extn) {
        LIST_FREE(bc->extn, exfn_t);
    }
    if (bc->cstr) {
        LIST_FREE(bc->cstr, cstr_t);
    }
    if (bc->func) {
        LIST_FREE(bc->func, uint32_t);
    }

    if (bc->sche) {
        sche_t *sche = NULL;
        int     iter = 0;
        list_iter_init(bc->sche);
        while (list_iter(bc->sche, (void **)&sche, &iter)) {
            LIST_FREE(sche->bind, bind_t);
        }
        LIST_FREE(bc->sche, sche_t);
    }

    memset(&btcd, 0, sizeof(btcd_t));

    BUFF_SET_HEAD(bc->code, code);
    BUFF_SET_CURR(bc->code, code);
    BUFF_SET_SIZE(bc->code, size);
    (void)BUFF_GET4_MV(bc->code);   // total length


    for (int i = 0; i < SEGM_NUMB; i++) {
        SG_SET_SIZE(i, BUFF_GET4_MV(bc->code));
        SG_SET_HEAD(i, BUFF_GET_CURR(bc->code));
        SG_SET_CURR(i, BUFF_GET_CURR(bc->code));
        BUFF_MOVE(bc->code, SG_GET_SIZE(i));
    }

    uint8_t *glob_segm_temp;
    glob_segm_temp = malloc(SG_GET_SIZE(GLOB_SEGM));
    memcpy(glob_segm_temp, SG_GET_HEAD(GLOB_SEGM), SG_GET_SIZE(GLOB_SEGM));
    SG_GET_HEAD(GLOB_SEGM) = SG_GET_CURR(GLOB_SEGM) = glob_segm_temp;

    // analyze bytecode
    if (!dfbc_vm_parse(bc)) {
        return false;
    }

    // init virtual environment
    memset(&venv, 0, sizeof(venv_t));
    venv.sp = -1;

    if (gettimeofday(&st, NULL) == -1) {
        DDBG("GET CURRENT TIME ERROR");
    }   // fixme: get program init timestamp

    return true;
}

void dfbc_loop() {
    uint32_t time;   // all event in a precess-cycle use one same timestamp to managed centrally
    time = getSysTime();

    DDBG(">>> --- LOOP START --- <<<                time: %d\n", time);

    // traversal all scheduled trigger event, find if anyone need process
    sche_t *sche_iter = NULL;
    int     i         = 0;
    list_iter_init(btcd.sche);
    while (list_iter(btcd.sche, (void **)&sche_iter, &i)) {
        bind_t  *bind_iter;
        int      argc = 0, i = 0;
        bool     need_proc = true;
        uint32_t argv[4]   = {0, 0, 0, 0};

        DDBG("process trigger event: %.*s()", sche_iter->evnt->symbol.size, sche_iter->evnt->symbol.data);
        list_iter_init(sche_iter->bind);
        while (list_iter(sche_iter->bind, (void **)&bind_iter, &i)) {
            if ((bind_iter->intv < 0) /*pulse trigger & timer event*/ ||
                ((bind_iter->last_run + bind_iter->intv) <= time) /*repeated trigger*/ ||
                (bind_iter->last_run < 0) /* first run */) {
                // process trigger event, get parameters. run once each event
                if (need_proc) {
                    argc = sche_iter->evnt->func(&btcd, &argv);
                    need_proc = false;
                    if (argc < 0) {
                        DDBG(TAB2"trigger event not happend, no need to process");
                        break;
                    }
                }
                DDBG(TAB "process bind event: func_0x%.8X() %ldms", bind_iter->func, bind_iter->intv);

                // execute user tasks
                dfbc_vm_call(&btcd, &venv, bind_iter->func, argv);
                // update last running timestamp
                bind_iter->last_run = time;
            }
        }
        if(need_proc) {
            DDBG(TAB"no bind event timeout");
        }
        DDBG("");

        // garbage collection, free string list memory
        size_t cstr_len = list_len(btcd.cstr);
        while (cstr_len > btcd.cstr_len) {
            cstr_t *cstr = NULL;
            list_get(btcd.cstr, cstr_len - 1, (void *)&cstr);
            if (cstr != NULL) {
                DDBG("GC: %d.%.*s", cstr_len, cstr->size, cstr->data);
                if (cstr->data != NULL) {
                    free(cstr->data);
                }
                free(cstr);
            }
            list_pop(btcd.cstr, cstr_len - 1);
            cstr_len--;
        }
    }

    DDBG("\n>>> ---  LOOP END  --- <<<\n\n\n");

}
size_t dfbc_meta(uint32_t *addr) {
    SG_INIT(btcd.segm);
    *addr = SG_GET_HEAD(META_SEGM);
    return SG_GET_SIZE(META_SEGM);
}

uint8_t dfbc_valid(){
	static uint8_t ret = 0;
	uint16_t len;
	uint16_t calc_cs=0, cs;
	uint8_t *p = DFBC_ADDR;
	if(ret) return ret;
	len = p[0] + p[1]*256;
	for(uint16_t i=0; i<len+4; i++){
		calc_cs += p[i];
	}
	cs = p[2044]+p[2045]*256;
	if((calc_cs == cs) && (p[2046] == 'O') && (p[2047] == 'K')){
	    DDBG("calc_cs=0x%04X  cs=%04X\n",calc_cs,cs);
	    //parse_bc_file(&vm);
	    //dump_bc_file();
		//dfbc_run(&vm);
		ret = 1;
	}else{
	    DDBG("len=0x%04X\n",len);
	    DDBG("calc_cs=0x%04X  cs=%04X\n",calc_cs,cs);
	    DDBG("dfbc tag=%c%c\n",p[2046],p[2047]);
	    ret = 0;
	}
	return ret;
}
/**************************************************
 *                 LOCAL FUNCTIONS
 **************************************************/
