#ifndef __DFBC_H__
#define __DFBC_H__

#include "dfbc_defs.h"

/**************************************************
 *                   FUNCTIONS
 **************************************************/

/// @brief      init df bytecode virtual machine
/// @param code the bytecode buffer
/// @param size sizeof buffer
bool dfbc_init(void);

/// @brief process virtual machine event
void dfbc_loop();
size_t dfbc_meta(uint32_t *addr);

typedef union {
    uint8_t all_bits;
    struct {
        uint8_t EPOCH  : 1; // bit0
        uint8_t IOOUT0 : 1; // bit1
        uint8_t IOIN0  : 1; // bit2
        uint8_t IOIN1  : 1; // bit3
        uint8_t IOCNT0 : 1; // bit4
        uint8_t IOCNT1 : 1; // bit5
        uint8_t ADC0   : 1; // bit6
        uint8_t BAT    : 1; // bit7
    } bits;
} sHW0Masks_t;

typedef struct {
	sHW0Masks_t hw0Mask;
	uint8_t modbusMask;
} sHWMasks_t;

typedef struct{
	sHWMasks_t masks;
    uint16_t IOCNT0;
    uint16_t IOCNT1;
    uint16_t prevIOCNT0;
    uint16_t prevIOCNT1;
    uint16_t ADC0;
	uint32_t EPOCH;
    uint8_t IOOUT0;
    uint8_t IOIN0;
    uint8_t IOIN1;
    uint8_t prevIOIN0;
    uint8_t prevIOIN1;
    uint8_t BAT;
    float RS485Value[8];
    uint8_t LORAWAN_RECV;
    char lorawan_buf[20];
}sDFBCEvents_t;

extern volatile sDFBCEvents_t dfbcEvents;

#endif // __DFBC_H__
