#include "dfbc_defs.h"
#include "dfbc.h"
#include <timer.h>
#include "hw.h"
#include "utils.h"
#include "DFLora_AT.h"
/**************************************************
 *               TIMER IMPLEMENTION
 **************************************************/


struct sTimerEvent {
    int           index;
    int           active;
    //struct event *ev;
};

struct sTimer {
    struct sTimerEvent event[8];
    TimerEvent_t dfbc_timer[8];
} tm;

int tm_init() {
    memset(&tm, 0, sizeof(tm));
    //tm.base = event_base_new();
    //if (!tm.base) {
    //    DDBG("timer init fail!");
    //    return -1;
    //}
    return 0;
}


void tm_callback(void *arg) {
	uint32_t index = (uint32_t)arg;
	DDBG("%s()----------index=%d\n",__FUNCTION__,index);
    struct sTimerEvent *info = &tm.event[index];
    info->active             = 1;
}

int tm_start(int index, int timeout) {
    if (index < 0 || index >= 8) {
        DDBG("Index out of bounds");
        return -1;
    }
    return 0;
}

void tm_stop(void) {
    uint8_t i=0;
	for(i=0;i<8;i++){
		if(tm.dfbc_timer[i].IsStarted){
			TimerStop(&tm.dfbc_timer[i]);
		}
	}
}

void builtin_init() {
    tm_init();
}

void builtin_loop() {
    //event_base_dispatch(tm.base);
}

#define CSTR(x)                                                                                                        \
    { #x, sizeof(#x) - 1 }

#define BIND_EVNT(x)                                                                                                   \
    { CSTR(x), evnt_##x }

#define EXTN_FUNC(x)                                                                                                   \
    { CSTR(x), wrap_##x }

int counter[2] = {0, 0};

// clang-format off
// bindable event
int timer (int  *data) {
    //if (!tm.base) {
    //    while(tm_init() < 0);
    //}
    for(int i=0; i<8;i++){
        if(tm.event[i].active) {
            tm.event[i].active = 0;
            *data = i;
            return 1;
        }
    }
    return -1;
}

int32_t on_di1_counter_change (int32_t  *data) {
	int32_t ret = 0;
	if(dfbcEvents.masks.hw0Mask.bits.IOCNT0){
		*data = dfbcEvents.IOCNT0;
		ret = 1;
	}
	DDBG(TAB"on_di1_counter_change (%d) return %d", *data,ret);
	return ret;
}

int32_t on_di2_counter_change (int32_t  *data) {
	int32_t ret = 0;
		*data = dfbcEvents.IOCNT1;
		ret = 1;
	DDBG(TAB"on_di2_counter_change (%d) return %d", *data, ret);
	return ret;
}

int32_t on_di1_volt_change    (int32_t  *data) {
	int32_t ret = 0;
	if(dfbcEvents.masks.hw0Mask.bits.IOIN0){
		*data = dfbcEvents.IOIN0;
		ret = 1;
	}
	DDBG(TAB"on_di1_volt_change (%d) return %d",    *data, ret);
	return ret;
}

int32_t on_di2_volt_change    (int32_t  *data) {
	int32_t ret = 0;
	if(dfbcEvents.masks.hw0Mask.bits.IOIN1){
		*data = dfbcEvents.IOIN1;
		ret = 1;
	}
	DDBG(TAB"on_di2_volt_change (%d) return %d",    *data, ret);
	return ret;
}

int32_t on_ai1_volt_change    (int32_t  *data)   {
	int32_t ret = 0;
	if(dfbcEvents.masks.hw0Mask.bits.ADC0){
		*data = dfbcEvents.ADC0;
		ret = 1;
	}
	DDBG(TAB"on_ai1_volt_change (%d) return %d",    *data, ret);
	return ret;
}

int on_lora_recv          (char *data)   {
	uint8_t len = 0;
	if(dfbcEvents.LORAWAN_RECV){
		dfbcEvents.LORAWAN_RECV = 0;
		len = strlen(dfbcEvents.lorawan_buf);
		memcpy(data, dfbcEvents.lorawan_buf, len);
	}

    DDBG(TAB"on_lora_recv (%.*s) return %d", len, data, len);
    return len;
}
int on_modbus_read        (int i, float *data) { \
	*data = dfbcEvents.RS485Value[i];\
    DDBG(TAB"%s index= %d (%f)",__FUNCTION__, i, *data);  \
    return sizeof("0123456789ABCDEF") - 1;  \
}

// built-in functions
int digital_read  (int   pin){
	DDBG("digital_read (%d) return %d", pin, rand()%2);
	return rand()%2;
}

int digital_write (int   pin, int value)   {
	if(pin == 1){
		relay(value);
	}
	DDBG("digital_write (%d, %d) return 1", pin, value);
	return 1;
}

int count_read    (int   pin)              {
	DDBG("count_read (%d) return %d", pin, counter[pin-1]);
	return counter[pin-1] ++;
}

int count_write   (int   pin, int value)   {
	DDBG("count_write (%d, %d->%d) return 1", pin, dtuConfig.loopOperation.ioCount[pin-1].Count,value);
	dtuConfig.loopOperation.ioCount[pin-1].Count = value;
	return 1;
}

int analog_read   (int   pin) {
	DDBG("analog_read (%d) return %d", pin, rand()%500);
	return rand()%500;
} // the analog is a float of 0.01 precision and less than 5. Which can be represented by an int of 0-500

int lora_send_str (char *data, int size)   {
	uint8_t total = 2;
	char AppDataBuffer[20]={0};
	memcpy(AppDataBuffer, data, size);
	lora_packet_list_append((uint8_t *)AppDataBuffer,size, 0);
	return 1;
}

int lora_send_modbus (int index)   {
	uint8_t total = 2;
	char AppDataBuffer[10];
	memset(AppDataBuffer,0,sizeof(AppDataBuffer));
	index--;
	DDBG("lora_send_modbus (%d) mask= %02X", index, dfbcEvents.masks.modbusMask);
	if(dfbcEvents.masks.modbusMask & (1<<index)){
		DDBG("lora_send_modbus (%d) %.2f return 1", index, dfbcEvents.RS485Value[index]);
	    sHWMasks_t *appmask = AppDataBuffer;
	    uint8_t *appdata = AppDataBuffer+2;
	    total += packModbusN(appmask,appdata,index);
	    lora_packet_list_append((uint8_t *)AppDataBuffer,total, 0);
	}

	return 1;
}

int lora_send_count   (int pin) {
	uint8_t total = 0;

	char AppDataBuffer[10];
    memset(AppDataBuffer,0,sizeof(AppDataBuffer));
    sHWMasks_t *appmask = AppDataBuffer;
    uint8_t *appdata = AppDataBuffer+2;
	if((pin == 1) && dfbcEvents.masks.hw0Mask.bits.IOCNT0){
		total = 2;
	}else if((pin == 2) && dfbcEvents.masks.hw0Mask.bits.IOCNT1) {
		total = 2;
	}
	if(total){
		total += packCountN(appmask,appdata,pin-1);
		lora_packet_list_append((uint8_t *)AppDataBuffer, total, 0);
	}
	DDBG("lora_send_count (%d) return 1", pin);
	return 1;
}

int lora_send_digital (int pin) {
	uint8_t total = 0;

	char AppDataBuffer[10];
    memset(AppDataBuffer,0,sizeof(AppDataBuffer));
    sHWMasks_t *appmask = AppDataBuffer;
    uint8_t *appdata = AppDataBuffer+2;
	if((pin == 1) && dfbcEvents.masks.hw0Mask.bits.IOIN0){
		total = 2;
	}else if((pin == 2) && dfbcEvents.masks.hw0Mask.bits.IOIN1) {
		total = 2;
	}
	if(total){
		total += packIOINN(appmask,appdata,pin-1);
		lora_packet_list_append((uint8_t *)AppDataBuffer, total, 0);
	}
	return 1;
}

int lora_send_adc (int pin) {
	uint8_t total = 0;

	char AppDataBuffer[10];
    memset(AppDataBuffer,0,sizeof(AppDataBuffer));
    sHWMasks_t *appmask = AppDataBuffer;
    uint8_t *appdata = AppDataBuffer+2;
	if(dfbcEvents.masks.hw0Mask.bits.ADC0){
		total = 2;
	}
	if(total){
		total += packAdc(appmask,appdata);
		lora_packet_list_append((uint8_t *)AppDataBuffer, total, 0);
	}
	return 1;
}

int modbus_send (int   baud, char *args, char *data, int size)  {
	char buf[80]={0};
	uint32_t tmp8;
	char *p = data;
	uint8_t len=0;

	while(len*2 < size){
		sscanf(p,"%02X",&tmp8);
		buf[len] = tmp8;
		len++;
		p+=2;
	}
	enable3V3();
	DelayMs(50);
    uart0Iint( baud, 3);
    rs485Flush();
    UART0_SendString(buf, len);
	//doRs485Item(buf,NULL);
    DDBG("modbus_send (%d, %.3s, %.*s) return 1", baud, args, size, data); 
    disable3V3();
    return 1; 
}

int compare_float_str (char* oper, char *str1, int len1, char *str2, int len2)  {
    DBG_NLN("compare_float_str (%c%c, %.*s, %.*s)", *oper, (*(oper+1)=='=')?'=':0, len1, str1, len2, str2);
    char buf[20]={0};
    memcpy(buf,str1,len1);
    DDBG("str1=%s\n",buf);
    memcpy(buf,str2,len2); buf[len2] = 0;
    DDBG("str2=%s\n",buf);
    float f1 = atof(str1), f2 = atof(str2);
    printf("-------------------%f\n",f2);
    DBG("f1=%f\n", f1);
    DBG("f2=%f\n", f2);
    switch(*oper){
        case '=': DDBG(" return %d", f1 == f2); return f1 == f2;
        case '<': DDBG(" return %d", (*(oper+1)=='=')?f1<=f2:f1<f2); return (*(oper+1)=='=')?f1<=f2:f1<f2;
        case '>': DDBG(" return %d", (*(oper+1)=='=')?f1>=f2:f1>f2); return (*(oper+1)=='=')?f1>=f2:f1>f2;
        default:  DDBG(" return 0"); return 0;
    }
}

int timer_start (int index, int timeout) {
    DDBG("timer_start(%d, %d)", index, timeout);
	TimerInit(&tm.dfbc_timer[index], tm_callback);
	TimerSetContext(&tm.dfbc_timer[index],(void *)index);
    TimerSetValue( &tm.dfbc_timer[index], timeout);
    TimerStart(&tm.dfbc_timer[index]);
    return 1;
}

// wrapped
int evnt_timer                   (btcd_t *bc, void *argv) { (void)bc; return timer((int *)argv);}
int evnt_on_di1_counter_change   (btcd_t *bc, void *argv) { (void)bc; if(on_di1_counter_change((int *)argv)) return 1; return -1;}
int evnt_on_di2_counter_change   (btcd_t *bc, void *argv) { (void)bc; if(on_di2_counter_change((int *)argv)) return 1; return -1;}
int evnt_on_di1_volt_change      (btcd_t *bc, void *argv) { (void)bc; if(on_di1_volt_change((int *)argv))    return 1; return -1;}
int evnt_on_di2_volt_change      (btcd_t *bc, void *argv) { (void)bc; if(on_di2_volt_change((int *)argv))    return 1; return -1;}
int evnt_on_ai1_volt_change      (btcd_t *bc, void *argv) { (void)bc; if(on_ai1_volt_change((int *)argv))    return 1; return -1;}
int evnt_on_lora_recv            (btcd_t *bc, void *argv) {
    char buff[64]; 
    size_t size = 0;
    if(size = on_lora_recv(buff)){
        cstr_t cstr = {
            data : (char *)malloc(sizeof(char) * size),
            size : size,
        };
        memcpy(cstr.data, buff, size);
        LIST_APPEND(bc->cstr, cstr_t, &cstr);
        *(uint32_t *)argv = (uint32_t)list_len(bc->cstr)-1;
        return 1;
    }   
    return -1;
}

#define evnt_on_modbus_read(x) \
int evnt_on_modbus_read_##x          (btcd_t *bc, void *argv) { \
    char buff[64]={0}; \
    float data;\
    if(on_modbus_read(x-1, &data)){\
		sprintf(buff,"%.2f",data); \
		size_t size = strlen(buff);\
        cstr_t cstr = {\
            data : (char *)malloc(size),\
            size : size,\
        };\
        memcpy(cstr.data, buff, size);\
        LIST_APPEND(bc->cstr, cstr_t, &cstr);\
        *((int *)(argv)) = list_len(bc->cstr) - 1;\
        return 2;\
    }   \
    return -1;\
}

evnt_on_modbus_read(1)
evnt_on_modbus_read(2)
evnt_on_modbus_read(3)
evnt_on_modbus_read(4)
evnt_on_modbus_read(5)
evnt_on_modbus_read(6)
evnt_on_modbus_read(7)
evnt_on_modbus_read(8)


int wrap_digital_read            (btcd_t *bc, void *argv) { (void)bc; return digital_read  (*(int *)argv); }
int wrap_digital_write           (btcd_t *bc, void *argv) { (void)bc; return digital_write (*(int *)argv, *(((int *)argv)+1)); }
int wrap_count_read              (btcd_t *bc, void *argv) { (void)bc; return count_read    (*(int *)argv); }
int wrap_count_write             (btcd_t *bc, void *argv) { (void)bc; return count_write   (*(int *)argv, *(((int *)argv)+1)); }
int wrap_analog_read             (btcd_t *bc, void *argv) { (void)bc; return analog_read   (*(int *)argv); }
int wrap_lora_send_str           (btcd_t *bc, void *argv) {
    cstr_t *cstr = NULL;
    list_get(bc->cstr, *(int *)argv, (void *)&cstr);
    if (cstr == NULL) {
        DBG_DTL("get list item error");
        return -1;
    }
    return lora_send_str(cstr->data, cstr->size);
}

int wrap_lora_send_count         (btcd_t *bc, void *argv) {
	(void)bc;
	return lora_send_count(*(int *)argv);
}

int wrap_lora_send_digital       (btcd_t *bc, void *argv) {
	(void)bc;
	return lora_send_digital (*(int *)argv);
}

int wrap_lora_send_adc           (btcd_t *bc, void *argv) {
	(void)bc;
	return lora_send_adc(*(int *)argv);
}

int wrap_lora_send_modbus        (btcd_t *bc, void *argv) {
	return lora_send_modbus  (*(int *)argv);
}

int wrap_modbus_send             (btcd_t *bc, void *argv) {
    int baud = *(int *)argv;
    cstr_t *cstr[2] = {NULL, NULL};
    for(int i=0; i<2; i++){
        list_get(bc->cstr, *(((int *)argv)+1+i), (void *)&cstr[i]);
        if (cstr[i] == NULL) {
            DBG_DTL("get list item error");
            return -1;
        }
    }
    return modbus_send(baud, cstr[0]->data, cstr[1]->data, cstr[1]->size);
}

int wrap_compare_float_str       (btcd_t *bc, void *argv) {
    cstr_t *cstr[3] = {NULL, NULL, NULL};
    for(int i=0; i<3; i++){
        list_get(bc->cstr, *(((int *)argv)+i), (void *)&cstr[i]);
        if (cstr[i] == NULL) {
            DBG_DTL("get list item error");
            return -1;
        }
    }
    int retval = compare_float_str(cstr[0]->data, cstr[1]->data, cstr[1]->size, cstr[2]->data, cstr[2]->size);
    return retval;
}

int wrap_timer_start             (btcd_t *bc, void *argv) { (void)bc; return timer_start(*(int *)argv, *(((int *)argv)+1));}

// callback
void aecb_timer (void * bind) {
    
}

// clang-format on      

exfn_t bind_evnt_list[] = {
    BIND_EVNT(timer),               
    BIND_EVNT(on_di1_counter_change),
    BIND_EVNT(on_di2_counter_change),
    BIND_EVNT(on_di1_volt_change),
    BIND_EVNT(on_di2_volt_change),
    BIND_EVNT(on_ai1_volt_change),
    BIND_EVNT(on_lora_recv),
    BIND_EVNT(on_modbus_read_1),
	BIND_EVNT(on_modbus_read_2),
	BIND_EVNT(on_modbus_read_3),
	BIND_EVNT(on_modbus_read_4),
	BIND_EVNT(on_modbus_read_5),
	BIND_EVNT(on_modbus_read_6),
	BIND_EVNT(on_modbus_read_7),
	BIND_EVNT(on_modbus_read_8),
};

exfn_t extn_func_list[] = {
    EXTN_FUNC(digital_read),  
    EXTN_FUNC(digital_write),     
    EXTN_FUNC(count_read),
    EXTN_FUNC(count_write),   
    EXTN_FUNC(analog_read),       
    EXTN_FUNC(lora_send_str), 
    EXTN_FUNC(lora_send_modbus),
    EXTN_FUNC(lora_send_count),
    EXTN_FUNC(lora_send_digital),
    EXTN_FUNC(lora_send_adc),
    EXTN_FUNC(lora_send_modbus),
    EXTN_FUNC(modbus_send),
    EXTN_FUNC(compare_float_str), 
    EXTN_FUNC(timer_start),
};

int bind_evnt_list_size = sizeof(bind_evnt_list)/sizeof(exfn_t);
int extn_func_list_size = sizeof(extn_func_list)/sizeof(exfn_t);


