#ifndef __DFBC_DEFS_H__
#define __DFBC_DEFS_H__

#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "dfbc_utils.h"

#define WRAP(func) wrapped_##func
#define EXPORT_SYMBOL()

/**************************************************
 *                   CONSTANTS
 **************************************************/

#define DFBC_VM_STACK_SIZE     128
#define DFBC_VM_CALLC_REG_SIZE 4

typedef enum eVaribalType {
    INT,
    STR,
} type_t;

typedef enum eMnemonics {
    MVG = 0x01,
    PUSH,
    PUSH32,
    PUSHR,
    PUSHG,
    POP,
    POP32,
    // POPR,
    // LOAD_ENV,
    // STOR_ENV,

    ADD = 0x11,
    SUB,
    MUL,
    DIV,
    MOD,

    LOR = 0x20,
    LAN,
    AND,
    OR,
    NOT,
    XOR,

    EQ = 0x31,
    EQ_STR,
    NE,
    LT,
    // LT_STR,
    GT,
    // GT_STR,
    LE,
    GE,
    SHL,
    SHR,

    JUMP = 0x41,
    JNZ,
    JZ,
    CALL,
    CALL_C,
    DEFFN,
    RSTCD,
    PRINT,
    PRINTS,

    RETURN = 0xE1,
    NOP    = 0xE2,
    EXIT   = 0xF9,

} mnemonic_t;

typedef enum {
    GLOB_SEGM,  //全局变量
    EXTN_SEGM,  //外部声明的函数  内置函数   
    CSTR_SEGM,  //常量字符串
    EVNT_SEGM,  //绑定事件
    FUNC_SEGM,  //字节码函数
	META_SEGM,  //元数据段

    SEGM_NUMB,
} segment_t;

/**************************************************
 *                VARIABLES
 **************************************************/

typedef struct sBytecode {
    buff_t code, segm[SEGM_NUMB];

    list_t *extn;   // exfn_t   external function
    list_t *cstr;   // cstr_t   bytecode strings
    list_t *func;   // uint32_t bytecode fucntion address
    list_t *sche;   // sche_t   sheduler of attach event

    size_t cstr_len;   // vm will malloc memory for string args at end of the cstr_list, the memory will be free after
                       // each loop

} btcd_t;   // bytecode

typedef int (*wrap_t)(btcd_t *bc, void *);
typedef void (*vmcb_t)(void *);

typedef struct sExternalFunction {
    cstr_t symbol;
    wrap_t func;

} exfn_t;

typedef struct sBind {
    exfn_t  *evnt;     // trigger event
    uint32_t func;     // user function address
    time_t   intv;     // minimum trigger invterval
    time_t   last_run; // last run time
} bind_t;

typedef struct sScheduler {
    exfn_t *evnt;
    list_t *bind;   // bind_t
} sche_t;

typedef struct sVirtualEnvironment {
    uint8_t  stack[DFBC_VM_STACK_SIZE];
    uint32_t reg[DFBC_VM_CALLC_REG_SIZE];
    int32_t  pc, sp, lv /* call level */ ;
    bool     hasReturn; // for we don't have stack frame to process return value, we use this and call level instead

} venv_t;

extern exfn_t bind_evnt_list[];
extern exfn_t extn_func_list[];

extern int bind_evnt_list_size;
extern int extn_func_list_size;

#endif
