#include "dfbc_vm.h"

#define VM_INIT(vm)            venv_t *_vm = vm
#define VM_GET_REG(idx)        (_vm->reg[idx])
#define VM_SET_REG(idx, value) (_vm->reg[idx] = value)
#define VM_GET_PC()            (_vm->pc)
#define VM_SET_PC(addr)        (_vm->pc = addr)
#define VM_GET_LV()            (_vm->lv)
#define VM_SET_LV(level)       (_vm->lv = level)
#define VM_ADD_LV()            (++(_vm->lv))
#define VM_DEC_LV()            (--(_vm->lv))
#define VM_GETV_MV()           *(SG_GET_HEAD(FUNC_SEGM) + _vm->pc++)
#define VM_GET4_MV()                                                                                                   \
    ({                                                                                                                 \
        uint32_t ret = U8A2U32(SG_GET_HEAD(FUNC_SEGM) + _vm->pc);                                                      \
        _vm->pc += 4;                                                                                                  \
        ret;                                                                                                           \
    })
#define VM_PUSH(val) (_vm->stack[++_vm->sp] = val)
#define VM_PUSH4(val)                                                                                                  \
    do {                                                                                                               \
        VM_PUSH(U322U8A(val, 0));                                                                                      \
        VM_PUSH(U322U8A(val, 1));                                                                                      \
        VM_PUSH(U322U8A(val, 2));                                                                                      \
        VM_PUSH(U322U8A(val, 3));                                                                                      \
    } while (0)
#define VM_POP()                                                                                                       \
    ({                                                                                                                 \
        if (_vm->sp < -1)                                                                                              \
            DDBG("STACK UNDERFLOW");                                                                                    \
        _vm->stack[_vm->sp--];                                                                                         \
    })
#define VM_POP4()               ((uint32_t)VM_POP() << 24 | (uint32_t)VM_POP() << 16 | (uint32_t)VM_POP() << 8 | (uint32_t)VM_POP())
#define VM_STACK_IS_UNDERFLOW() (_vm->sp < -1)
#define VM_STACK_IS_EMPTY()     (_vm->sp == -1)


bool dfbc_vm_parse(btcd_t *bc) {
    // extern function declaration
    LIST_INIT(bc->extn);
    SG_INIT(bc->segm);
    SG_RST(EXTN_SEGM);
    while (SG_OFFSET(EXTN_SEGM) < SG_GET_SIZE(EXTN_SEGM)) {
        exfn_t *extn_func = NULL;
        cstr_t  extn_symb;
        size_t  sizlen = 0;

        if (!(sizlen = tocstr(SG_GET_CURR(EXTN_SEGM), &extn_symb))) {
            DDBG("fail to parse extern function symbol: %.*s", (int)extn_symb.size, extn_symb.data);
            return false;
        }

        if (0 == extn_symb.size) {
            (void)SG_GETV_MV(EXTN_SEGM);
            continue;
        }

        // try search the symbol in external function list
        bool found = false;
        for (int i = 0; i < bind_evnt_list_size; i++) {
            if (bind_evnt_list[i].symbol.size == extn_symb.size &&
                memcmp(bind_evnt_list[i].symbol.data, extn_symb.data, extn_symb.size) == 0) {
                extn_func = &bind_evnt_list[i];
                found     = true;
                break;
            }
        }

        // try search the symbol in bindable event list,
        if (!found) {
            for (int i = 0; i < extn_func_list_size; i++) {
                if (extn_func_list[i].symbol.size == extn_symb.size &&
                    memcmp(extn_func_list[i].symbol.data, extn_symb.data, extn_symb.size) == 0) {
                    extn_func = &extn_func_list[i];
                    break;
                }
            }
        }

        if (NULL == extn_func) {
            DDBG("ERROR: unsupport external function declaration: %.*s", (int)extn_symb.size, extn_symb.data);
            return false;
        }

        LIST_APPEND(bc->extn, exfn_t, extn_func);
        SG_MOVE(EXTN_SEGM, sizlen + extn_symb.size);
    }

    // string analysis
    LIST_INIT(bc->cstr);
    SG_RST(CSTR_SEGM);
    DBG_HEX(SG_GET_HEAD(CSTR_SEGM), SG_GET_SIZE(CSTR_SEGM));
    while (SG_OFFSET(CSTR_SEGM) < SG_GET_SIZE(CSTR_SEGM)) {
        cstr_t cstr_symb;
        size_t sizlen = 0;
        sizlen        = tocstr(SG_GET_CURR(CSTR_SEGM), &cstr_symb);
        LIST_APPEND(bc->cstr, cstr_t, &cstr_symb);
        SG_MOVE(CSTR_SEGM, sizlen + cstr_symb.size);
    }
    bc->cstr_len = list_len(bc->cstr);

    // function definition (user's tasks)
    uint8_t instruction = 0;
    LIST_INIT(bc->func);
    SG_RST(FUNC_SEGM);
    while ((instruction = SG_GETV_MV(FUNC_SEGM)) && ((size_t)SG_OFFSET(FUNC_SEGM) <= SG_GET_SIZE(FUNC_SEGM))) {
        // clang-format off
        switch (instruction) {
        case 0: /* align 0 */; break;
        case ADD: case SUB: case MUL: case DIV: case MOD: case AND: case OR:  case NOT:
        case XOR: case EQ:  case NE:  case LT:  case GT:  case LE:  case GE:  case EQ_STR:
        /* case LT_STR: case GT_STR: */         case SHL: case SHR: case POP: case POP32:
        /* case POPR: */    case RETURN:        case PRINT:         case PRINTS:   
            break;                                 // move 0
        case PUSH: case PUSHR: case CALL_C: case CALL:
            (void) SG_GETV_MV(FUNC_SEGM); break; // move 1
        case PUSH32: case PUSHG: case JUMP: case JNZ: case JZ:
            (void) SG_GET4_MV(FUNC_SEGM); break; // move 4
        case MVG:
            (void) SG_GET4_MV(FUNC_SEGM); (void) SG_GET4_MV(FUNC_SEGM); break; // move 8
        case DEFFN: {
            uint32_t addr = SG_OFFSET(FUNC_SEGM) - 1;
            LIST_APPEND(bc->func, uint32_t, &addr);
            (void) SG_GETV_MV(FUNC_SEGM); 
        } break;                                   // move 1
        default:
            DDBG("ERROR, unknown instruction: %02x in %d", instruction, SG_OFFSET(FUNC_SEGM)-1);
            break; // should never reach here
        }
        // clang-format on
    }

    // bind event and user tasks
    LIST_INIT(bc->sche);
    SG_RST(EVNT_SEGM);
    while (SG_OFFSET(EVNT_SEGM) < SG_GET_SIZE(EVNT_SEGM)) {
        uint32_t  evnt = SG_GET4_MV(EVNT_SEGM), func = SG_GET4_MV(EVNT_SEGM), intv = SG_GET4_MV(EVNT_SEGM);
        exfn_t   *_evnt = NULL;
        uint32_t *_func = NULL;
        list_get(bc->extn, evnt, (void *)&_evnt);
        list_get(bc->func, func, (void *)&_func);
        bind_t bind_evnt = {evnt : _evnt, func : *_func, intv : intv, last_run : -1};

        // put tasks with same trigger event together
        sche_t *sche_iter;
        int     i = 0, found = 0;
        list_iter_init(bc->sche);
        while (list_iter(bc->sche, (void **)&sche_iter, &i)) {
            if (bind_evnt.evnt == sche_iter->evnt) {
                LIST_APPEND(sche_iter->bind, bind_t, &bind_evnt);
                found = 1;
                break;
            }
        }

        // otherwise, create a new scheduled event
        if (!found) {
            sche_t sche_evnt = {
                evnt : bind_evnt.evnt,
            };
            LIST_INIT(sche_evnt.bind);
            LIST_APPEND(sche_evnt.bind, bind_t, &bind_evnt);
            LIST_APPEND(bc->sche, sche_t, &sche_evnt);
        }
    }

#ifdef DEBUG
    DDBG("BYTECODE:" EDL);
    DDBG("EXTERANL FUNCTION:");
    exfn_t *exfn = NULL;
    int     iter = 0;
    list_iter_init(bc->extn);
    while (list_iter(bc->extn, (void **)&exfn, &iter)) {
        DDBG("%d" TAB "%s", iter, exfn->symbol.data);
    }
    DDBG(EDL);

    DDBG("STRINGS:");
    cstr_t *cstr = NULL;
    iter         = 0;
    list_iter_init(bc->cstr);
    while (list_iter(bc->cstr, (void **)&cstr, &iter)) {
        if (cstr->size)
            DDBG("%d" TAB "%.*s", iter, (int)cstr->size, cstr->data);
        else
            DDBG("%d" TAB "(empty string)", iter);
    }
    DDBG(EDL);

    DDBG("ATTACH EVENT");
    sche_t *sche = NULL;
    iter         = 0;
    list_iter_init(bc->sche);
    while (list_iter(bc->sche, (void **)&sche, &iter)) {
        DDBG("%d" TAB "%s", iter, sche->evnt->symbol.data);
        bind_t *bind  = NULL;
        int     iter_ = 0;
        list_iter_init(sche->bind);
        while (list_iter(sche->bind, (void **)&bind, &iter_)) {
            DDBG(TAB "%d.%d" TAB "%8d ms" TAB "func_%d()", iter, iter_, (int)bind->intv,
                *(SG_GET_HEAD(FUNC_SEGM) + bind->func + 1));
        }
    }
    DDBG(EDL);
#endif   // DEBUG
    return true;
}

void dfbc_vm_call(btcd_t *bc, venv_t *vm, uint32_t fnAddr, uint32_t *argv) {
#ifdef DEBUG
    // DDBG("STRINGS:");
    // cstr_t *cstr = NULL;
    // int     iter = 0;
    // list_iter_init(bc->cstr);
    // while (list_iter(bc->cstr, (void **)&cstr, &iter)) {
    //     if (cstr->size)
    //         DDBG("%d" TAB"%.*s", iter, (int)cstr->size, cstr->data);
    //     else
    //         DDBG("%d" TAB"(empty string)", iter);
    // }
    // DDBG(EDL);
#endif
    DDBG(TAB2 "call func_%d() {", fnAddr);
    VM_INIT(vm);
    SG_INIT(bc->segm);

    // init environment
    VM_SET_PC(fnAddr);
    if (!VM_STACK_IS_EMPTY()) {
        DDBG(TAB2 "STACK ERROR: NOT EMPTY");
        vm->sp = -1;
    }
    if (VM_GET_LV()) {
        DDBG(TAB2 "CALL LEVEL ERROR: NOT ZERO");
        VM_SET_LV(0);
    }

    VM_SET_REG(0, *(argv + 0));
    VM_SET_REG(1, *(argv + 1));
    VM_SET_REG(2, *(argv + 2));
    VM_SET_REG(3, *(argv + 3));


    // exec bytecode
    uint8_t instruction = VM_GETV_MV(), loop = 1;
    if (instruction != DEFFN) {
        DDBG(TAB2 "wrong function definition");
        return;
    }
    (void)VM_GETV_MV();

    instruction = VM_GETV_MV();
    while (instruction != EXIT) {

        DBG_NLN(TAB3 "PC:0x%2X SP:%2d LV:%2d" TAB, VM_GET_PC(), vm->sp, vm->lv);
        switch (instruction) {
        /*-----------------------------
                variable operation
         -----------------------------*/
        case MVG: {
        	DBG_NLN("\nGLOB SEGM:");
        	DBG_HEX(SG_GET_HEAD(GLOB_SEGM), SG_GET_SIZE(GLOB_SEGM));
            uint32_t globaddr = VM_GET4_MV(), globval = VM_GET4_MV(),oldval = SG_GET4_AT(GLOB_SEGM, globaddr);
            DDBG("GET4_AT 0x%X", SG_GET_HEAD(GLOB_SEGM)+globaddr);
            SG_SET4_AT(GLOB_SEGM, globaddr, globval);
            DDBG(TAB "MVG     (0x%.02X:0x%.02X)(0x%.02X->0x%.02X)", globaddr, globval, oldval, SG_GET4_AT(GLOB_SEGM, globaddr));
            DBG_NLN("GLOB SEGM:");
            DBG_HEX(SG_GET_HEAD(GLOB_SEGM), SG_GET_SIZE(GLOB_SEGM));
        } break;

        /*-----------------------------
                 stack operation
          -----------------------------*/
        case PUSH: {
            uint8_t numval = VM_GETV_MV();
            VM_PUSH(numval);
            DDBG(TAB "PUSH    NUM(0x%.02X)", numval);
        } break;

        case PUSH32: {
            uint32_t numval = VM_GET4_MV();
            VM_PUSH4(numval);
            DDBG(TAB "PUSH32  NUM(0x%.08X)", numval);
        } break;

        case PUSHR: {
            uint8_t  regidx = VM_GETV_MV();
            uint32_t regval = VM_GET_REG(regidx);
            VM_PUSH4(regval);

            DDBG(TAB "PUSHR   REG[%d](0x%.08X)", regidx, regval);
        } break;

        case PUSHG: {
            uint32_t offset = VM_GET4_MV();
            uint32_t varval = SG_GET4_AT(GLOB_SEGM, offset);
            VM_PUSH4(varval);
            DDBG(TAB "PUSHG   GLO[%.08x](%.08x)", offset, varval);
        } break;

        case POP: {
            uint8_t popval = VM_POP();
            DDBG(TAB "POP     NUM(0x%.02X)", popval);
        } break;

        case POP32: {
            uint32_t popval = VM_POP4();
            DDBG(TAB "POP32   NUM(0x%.08X)", popval);
        } break;

            // have not implemented in the compiler yet
            // case POPR: {
            // } break;

            /*-----------------------------
                  arithmetic operation
              -----------------------------*/

        case ADD: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a + b);
            DDBG(TAB "ADD     (%d + %d = %d)", a, b, a + b);
        } break;

        case SUB: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a - b);
            DDBG(TAB "SUB     (%d - %d = %d)", a, b, a - b);
        } break;

        case MUL: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a * b);
            DDBG(TAB "SUB     (%d * %d = %d)", a, b, a * b);
        } break;

        case DIV: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a / b);
            DDBG(TAB "SUB     (%d / %d = %d)", a, b, a / b);
        } break;

        case MOD: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a % b);
            DDBG(TAB "MOD     (%d %% %d = %d)", a, b, a % b);
        } break;

        /*-----------------------------
                  bit operation
          -----------------------------*/
        case AND: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a && b);
            DDBG(TAB "AND     (%d && %d = %d)", a, b, a && b);
        } break;
        case OR: {
            int a = (int)VM_POP4(), b = (int)VM_POP4();
            VM_PUSH4(a || b);
            DDBG(TAB "OR     (%d || %d = %d)", a, b, a || b);
        } break;
        case NOT: {
            int a = (int)VM_POP4();
            VM_PUSH4(!a);
            DDBG(TAB "NOT     (!%d = %d)", a, !a);
        } break;
        case XOR: {
            int a = (int)VM_POP4();
            VM_PUSH4(~a);
            DDBG(TAB "XOR     (~%d = %d)", a, ~a);
        } break;

        /*-----------------------------
                  logic operation
          -----------------------------*/
        case LOR: {
            int b = (int)VM_POP4(), a = (int)VM_POP4(); // stack, FILO
            VM_PUSH4(a || b);
            DDBG(TAB "LOR     (%d || %d)(%s)", a, b, (a || b) ? "true" : "false");
        } break;

        case LAN: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a && b);
            DDBG(TAB "LAN     (%d && %d)(%s)", a, b, (a && b) ? "true" : "false");
        } break;

        case EQ: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a == b);
            DDBG(TAB "EQ      (%d == %d)", a, b);
        } break;

        case EQ_STR: {
            uint32_t strbIdx = VM_POP4(), straIdx = VM_POP4();
            cstr_t  *stra = NULL, *strb = NULL;
            list_get(bc->cstr, straIdx, (void *)&stra);
            list_get(bc->cstr, strbIdx, (void *)&strb);
            bool cmp = (stra->size == strb->size) ? (0 == memcmp(stra->data, strb->data, stra->size)) : false;
            VM_PUSH4(cmp);
            DDBG(TAB "EQ_STR  (\"%.*s\" == \"%.*s\")(%s)", (int)stra->size, stra->data, (int)strb->size, strb->data,
                cmp ? "true" : "false");
        } break;

        case NE: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a != b);
            DDBG(TAB "NE      (%d != %d)", a, b);
        } break;

        case LT: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a < b);
            DDBG(TAB "LT      (%d < %d)", a, b);
        } break;

        case GT: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a > b);
            DDBG(TAB "GT      (%d > %d)", a, b);
        } break;

        case LE: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a <= b);
            DDBG(TAB "LE      (%d <= %d)", a, b);
        } break;

        case GE: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a >= b);
            DDBG(TAB "GE      (%d >= %d)", a, b);
        } break;

        case SHL: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a << b);
            DDBG(TAB "SHL     (%d << %d)", a, b);
        } break;

        case SHR: {
            int b = (int)VM_POP4(), a = (int)VM_POP4();
            VM_PUSH4(a >> b);
            DDBG(TAB "SHR     (%d >> %d)", a, b);
        } break;

        /*-----------------------------
                 vm operation
          -----------------------------*/
        case JUMP: {
            uint32_t addr = VM_GET4_MV();
            VM_SET_PC(addr);
            DDBG(TAB "JUMP    0x%.08X", addr);
        } break;

        case JNZ: {
            uint32_t addr   = VM_GET4_MV();
            int      popval = (int)VM_POP4();
            if (popval != 0) {
                VM_SET_PC(addr);
                DDBG(TAB "JNZ     (JUMP TO 0x%.08X)", addr);
            } else {
                DDBG(TAB "JNZ     (THOUGH)");
            }
        } break;

        case JZ: {
            uint32_t addr   = VM_GET4_MV();
            int      popval = (int)VM_POP4();
            if (popval == 0) {
                VM_SET_PC(addr);
                DDBG(TAB "JZ      (JUMP TO 0x%.08X)", addr);
            } else {
                DDBG(TAB "JZ      (THROUGH)");
            }
        } break;

        case CALL: {
            // parse function parameters
            uint8_t  argc = VM_POP(), funcIdx = VM_GETV_MV();
            uint32_t argv[4] = {0}, *funcAddr;
            list_get(bc->func, funcIdx, (void *)&funcAddr);
            for (int i = 0; i < argc; i++) {
                argv[i] = VM_POP4();
            }

            // save context
            for (int i = 0; i < 4; i++) {
                uint32_t regval = VM_GET_REG(i);
                VM_PUSH4(regval);
                VM_SET_REG(i, argv[i]);
            }
            VM_PUSH4(VM_GET_PC());

            // check if function has return value
            vm->hasReturn = (VM_GETV_MV() == POP32);

            // call function
            VM_SET_PC(*funcAddr);
            VM_ADD_LV();
            DDBG(TAB "CALL    func_%u(%.08X)", funcIdx, (uint32_t)funcAddr);
        } break;

        case CALL_C: {
            uint8_t  argc = VM_POP(), funcIdx = VM_GETV_MV();
            uint32_t argv[4] = {0};
            for (int i = 0; i < argc; i++) {
                argv[argc - 1 - i] = VM_POP4();
            }

            DBG_NLN(TAB "CALL_C  ");

            exfn_t *exfn = NULL;
            list_get(bc->extn, funcIdx, (void *)&exfn);
            uint32_t retval = exfn->func(bc, argv);
            VM_PUSH4(retval);
        } break;

        case DEFFN: {
            int index = VM_GETV_MV();
            DDBG(TAB "DEFFN   %d", index);
        } break;

        case PRINT: {
            int popval = (int)VM_POP4();
            DDBG(TAB "PRINT   %d", popval);
        } break;

        case PRINTS: {
            uint32_t strIdx = VM_POP4();
            cstr_t  *str;
            list_get(bc->cstr, strIdx, (void *)&str);

            DDBG(TAB "PRINTS %.*s", (int)str->size, str->data);
        } break;

        case RETURN: {
            // check call level, return to upper level if lv > 0
            if (VM_GET_LV() > 0) {
                VM_DEC_LV();
                uint32_t retval, addr;
                if (vm->hasReturn) {
                    retval = VM_POP4();
                }
                addr = VM_POP4();
                for (int i = 0; i < 4; i++) {
                    VM_SET_REG(i, VM_POP4());
                }
                VM_SET_PC(addr);
                if (vm->hasReturn) {
                    VM_PUSH4(retval);
                }
                DDBG(TAB "RETURN  (PC:0x%.2X)", addr);
                break;
            }

            // otherwise go through the case then exit program;
            DDBG(TAB "RETURN  (EXIT)");
        }

        case EXIT: {
            instruction = EXIT;
            continue;
        } break;

        default: {
            DDBG(TAB "unknown instruction: %d", instruction);
            exit(-1);
        } break;
        }



        instruction = VM_GETV_MV();
    }

    uint32_t retval = VM_POP4();

    if (!VM_STACK_IS_EMPTY()) {
        DDBG("STACK ERROR, sp = %d", vm->sp);
    }

    DDBG(TAB2 "}");
}
