/*
  ______                              _
 / _____)             _              | |
( (____  _____ ____ _| |_ _____  ____| |__
 \____ \| ___ |    (_   _) ___ |/ ___)  _ \
 _____) ) ____| | | || |_| ____( (___| | | |
(______/|_____)_|_|_| \__)_____)\____)_| |_|
    (C)2013 Semtech

Description: SX126x driver specific target board functions implementation

License: Revised BSD License, see LICENSE.TXT file include in the project

Maintainer: Miguel Luis and Gregory Cristian
*/

/******************************************************************************
 * @file    sx126x-board.c
 * @author  Insight SiP
 * @version V2.0.0
 * @date    30-january-2019
 * @brief   SX126x implementation compatible with LoraWan/semtech drivers.
 *
 * @attention
 *	THIS SOFTWARE IS PROVIDED BY INSIGHT SIP "AS IS" AND ANY EXPRESS
 *	OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *	OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *	DISCLAIMED. IN NO EVENT SHALL INSIGHT SIP OR CONTRIBUTORS BE
 *	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *	GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *	LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *	OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "DFLora_At.h"

#include "CH58x_gpio.h"
#include "CH58x_spi.h"

/*
   GPIOA_SetBits
   GPIOB_ResetBits
   GPIOA_ModeCfg

    GPIO_ModeIN_Floating, //浮空输入
    GPIO_ModeIN_PU,       //上拉输入
    GPIO_ModeIN_PD,       //下拉输入
    GPIO_ModeOut_PP_5mA,  //推挽输出最大5mA
    GPIO_ModeOut_PP_20mA, //推挽输出最大20mA
 * */
#define LORA_SS GPIO_Pin_12
#define LORA_BUSY GPIO_Pin_8
#define LORA_DIO1 GPIO_Pin_7
#define LORA_RST GPIO_Pin_9
#define LORA_ANTPWR GPIO_Pin_8

#define RELAY_GPIO GPIO_Pin_0
//#define LORA_SS  6
#include "CH58x_common.h"

#include "board.h"

#include "sx126x/sx126x.h"
#include "sx126x-board.h"
static RadioOperatingModes_t OperatingMode;

// No need to initialize DIO3 as output everytime, do it once and remember it
bool dio3IsOutput = false;

void SX126xIoInit(void)
{
	//initSPI();

    GPIOA_SetBits(GPIO_Pin_12);
    GPIOA_ModeCfg(GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14, GPIO_ModeOut_PP_5mA);
	SPI0_MasterDefInit();

   // printf("SX126xIoInit\n");
    //GPIOA_ModeCfg(LORA_SS,GPIO_ModeOut_PP_5mA);
    GPIOA_ModeCfg(LORA_BUSY,GPIO_ModeIN_PU);
    GPIOA_ModeCfg(LORA_DIO1,GPIO_ModeIN_PU);
    GPIOA_ModeCfg(LORA_RST,GPIO_ModeOut_PP_5mA);



    GPIOB_ModeCfg(LORA_ANTPWR,GPIO_ModeOut_PP_5mA);
    GPIOB_ResetBits(LORA_ANTPWR);



   // GPIOA_SetBits(LORA_RST);
//    GPIOA_ResetBits(LORA_RST);
    //GPIOA_SetBits(LORA_RST);

    GPIOA_ResetBits(LORA_RST);
    DelayMs(10);
   GPIOA_SetBits(LORA_RST);
    DelayMs(20);


    //GPIOB_ModeCfg(RELAY_GPIO,GPIO_ModeOut_PP_20mA);
    //GPIOB_ResetBits(RELAY_GPIO);
    //while(1){
    //	GPIOB_ResetBits(RELAY_GPIO);
    //	DelayMs(2000);
    //	GPIOB_SetBits(RELAY_GPIO);
    //	DelayMs(2000);
    //}
}

void SX126xIoReInit(void)
{
    GPIOA_SetBits(GPIO_Pin_12);
    GPIOA_ModeCfg(GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14, GPIO_ModeOut_PP_5mA);
    SPI0_MasterDefInit();

	dio3IsOutput = false;

	//GPIOA_ModeCfg(LORA_SS,GPIO_ModeOut_PP_5mA);
   // GPIOA_SetBits(LORA_SS);
   // GPIOA_ModeCfg(LORA_RST,GPIO_ModeOut_PP_5mA);


    GPIOB_ModeCfg(LORA_ANTPWR,GPIO_ModeOut_PP_5mA);
    GPIOB_ResetBits(LORA_ANTPWR);

   // SX126xInit()

}
DioIrqHandler *dioIrq1 =NULL;
void SX126xIoIrqInit(DioIrqHandler dioIrq)
{
    if(dioIrq !=NULL){
        dioIrq1 = dioIrq;
        dioIrq1(NULL);
    }else{
         //printf("diorq nluu\n");
    }

    GPIOA_ModeCfg(LORA_DIO1,GPIO_ModeIN_PU);
    GPIOA_ITModeCfg(LORA_DIO1,GPIO_ITMode_RiseEdge);
    PFIC_EnableIRQ(GPIO_A_IRQn);

	//attachInterrupt(LORA_DIO1, dioIrq, RISING);
}


__INTERRUPT
__HIGH_CODE
void GPIOA_IRQHandler(){
    //printf("irq1=%d\n",GPIOA_ReadITFlagBit(GPIO_Pin_5));
    //printf("-i\n");
    if(GPIOA_ReadITFlagBit(LORA_DIO1 & R16_PA_INT_EN) != 0){
        dioIrq1(NULL);
        GPIOA_ClearITFlagBit(LORA_DIO1);
    }
}

void SX126xIoDeInit(void)
{
	dio3IsOutput = false;

    GPIOA_ModeCfg(LORA_SS,GPIO_ModeIN_PD);
    GPIOB_ModeCfg(LORA_BUSY,GPIO_ModeIN_PU);
    GPIOB_ModeCfg(LORA_DIO1,GPIO_ModeIN_PU);
    GPIOB_ModeCfg(LORA_RST,GPIO_ModeIN_PD);

	//detachInterrupt(LORA_DIO1);

}
uint8_t SX126xGetDeviceId( void )
{
        return SX1262;

}
void SX126xIoTcxoInit( void )
{
    CalibrationParams_t calibParam;

    SX126xSetDio3AsTcxoCtrl( TCXO_CTRL_3_3V, 50 << 6 ); // convert from ms to SX126x time base
    calibParam.Value = 0x7F;
    SX126xCalibrate( calibParam );
}

void SX126xReset(void)
{

	GPIOA_ModeCfg(LORA_RST,GPIO_ModeOut_PP_5mA);
	GPIOA_ResetBits(LORA_RST);
	DelayMs(10);
	GPIOA_SetBits(LORA_RST);
	DelayMs(20);
	dio3IsOutput = false;
}

void SX126xWaitOnBusy(void)
{
	int timeout = 1000;

	while (GPIOA_ReadPortPin(LORA_BUSY) != 0)
	{
	    //printf("busy wait\n");
	    DelayUs(50);
		timeout -= 1;

		if (timeout < 0)
		{
		    //printf("a\n");
			return;
		}
	}
	//printf("b\n");
}
uint32_t SX126xGetDio1PinState( void )
{
    return GPIOA_ReadPortPin(LORA_DIO1);
}
void SX126xWakeup(void)
{
	dio3IsOutput = false;
	//BoardDisableIrq();
    //printf("wakeup\n");
	GPIOA_ResetBits(LORA_SS);


	SPI0_MasterSendByte(RADIO_GET_STATUS);
	SPI0_MasterSendByte(0x00);


	GPIOA_SetBits(LORA_SS);
	// Wait for chip to be ready.
	SX126xWaitOnBusy();

	//BoardEnableIrq();
}

void SX126xWriteCommand(RadioCommands_t command, uint8_t *buffer, uint16_t size)
{
	//printf("%s()\r\n",__FUNCTION__);
	SX126xCheckDeviceReady();

	GPIOA_ResetBits(LORA_SS);


	SPI0_MasterSendByte((uint8_t)command);
	//SPI0_MasterTrans(buffer,size);

    for (uint16_t i = 0; i < size; i++)
    {
        //buffer[i] = SPI0_MasterRecvByte();
        SPI0_MasterSendByte(buffer[i]);
    }


	GPIOA_SetBits(LORA_SS);

	if (command != RADIO_SET_SLEEP)
	{
		SX126xWaitOnBusy();
	}
}

uint8_t SX126xReadCommand(RadioCommands_t opcode, uint8_t *buffer, uint16_t size)
{
	SX126xCheckDeviceReady();

	GPIOA_ResetBits(LORA_SS);

    SPI0_MasterSendByte((uint8_t)opcode);
    SPI0_MasterSendByte(0x00);

    for (uint16_t i = 0; i < size; i++)
	{
		buffer[i] = SPI0_MasterRecvByte();
	}

	//SPI_LORA.endTransaction();
	GPIOA_SetBits(LORA_SS);

	SX126xWaitOnBusy();
}
RadioOperatingModes_t SX126xGetOperatingMode( void )
{
    return OperatingMode;
}

void SX126xSetOperatingMode( RadioOperatingModes_t mode )
{
    OperatingMode = mode;
#if defined( USE_RADIO_DEBUG )
    switch( mode )
    {
        case MODE_TX:
            SX126xDbgPinTxWrite( 1 );
            SX126xDbgPinRxWrite( 0 );
            break;
        case MODE_RX:
        case MODE_RX_DC:
            SX126xDbgPinTxWrite( 0 );
            SX126xDbgPinRxWrite( 1 );
            break;
        default:
            SX126xDbgPinTxWrite( 0 );
            SX126xDbgPinRxWrite( 0 );
            break;
    }
#endif
}
void SX126xWriteRegisters(uint16_t address, uint8_t *buffer, uint16_t size)
{
	SX126xCheckDeviceReady();

	GPIOA_ResetBits(LORA_SS);


	SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
	SPI0_MasterSendByte((address & 0xFF00) >> 8);
	SPI0_MasterSendByte(address & 0x00FF);

    for (uint16_t i = 0; i < size; i++)
    {
        //buffer[i] = SPI0_MasterRecvByte();
        SPI0_MasterSendByte(buffer[i]);
    }

	GPIOA_SetBits(LORA_SS);

	SX126xWaitOnBusy();
}

void SX126xWriteRegister(uint16_t address, uint8_t value)
{
	SX126xWriteRegisters(address, &value, 1);
}

void SX126xReadRegisters(uint16_t address, uint8_t *buffer, uint16_t size)
{
   // printf("%s()\r\n",__FUNCTION__);
	SX126xCheckDeviceReady();
	//printf("12\n");
	GPIOA_ResetBits(LORA_SS);

	//SPI_LORA.beginTransaction(spiSettings);
	SPI0_MasterSendByte(RADIO_READ_REGISTER);
	SPI0_MasterSendByte((address & 0xFF00) >> 8);
	SPI0_MasterSendByte(address & 0x00FF);
	SPI0_MasterSendByte(0x00);
	for (uint16_t i = 0; i < size; i++)
	{
		buffer[i] = SPI0_MasterRecvByte();
	}
	GPIOA_SetBits(LORA_SS);

	SX126xWaitOnBusy();
}

uint8_t SX126xReadRegister(uint16_t address)
{
	uint8_t data;
	SX126xReadRegisters(address, &data, 1);
	return data;
}

void SX126xWriteBuffer(uint8_t offset, uint8_t *buffer, uint8_t size)
{
	SX126xCheckDeviceReady();

	GPIOA_ResetBits(LORA_SS);

	//SPI_LORA.beginTransaction(spiSettings);
	SPI0_MasterSendByte(RADIO_WRITE_BUFFER);
	SPI0_MasterSendByte(offset);
    for (uint16_t i = 0; i < size; i++)
    {
        //buffer[i] = SPI0_MasterRecvByte();
        SPI0_MasterSendByte(buffer[i]);
    }
	//SPI_LORA.endTransaction();
	GPIOA_SetBits(LORA_SS);

	SX126xWaitOnBusy();
}

void SX126xReadBuffer(uint8_t offset, uint8_t *buffer, uint8_t size)
{
	SX126xCheckDeviceReady();

	GPIOA_ResetBits(LORA_SS);

	//SPI_LORA.beginTransaction(spiSettings);
	SPI0_MasterSendByte(RADIO_READ_BUFFER);
	SPI0_MasterSendByte(offset);
	SPI0_MasterSendByte(0x00);
	for (uint16_t i = 0; i < size; i++)
	{
		buffer[i] = SPI0_MasterRecvByte();
	}
	//SPI_LORA.endTransaction();
	GPIOA_SetBits(LORA_SS);

	SX126xWaitOnBusy();
}

void SX126xSetRfTxPower(int8_t power)
{
	SX126xSetTxParams(power, RADIO_RAMP_40_US);
}

uint8_t SX126xGetPaSelect(uint32_t channel)
{

		return SX1262;

}
void SX126xIoRfSwitchInit( void )
{
    SX126xSetDio2AsRfSwitchCtrl( true );
}

uint32_t SX126xGetBoardTcxoWakeupTime( void )
{
    return 5;
}
static void SX126xDio3Control(bool state)
{
	uint8_t reg_0x0580;
	uint8_t reg_0x0583;
	uint8_t reg_0x0584;
	uint8_t reg_0x0585;
	uint8_t reg_0x0920;

	if (!dio3IsOutput)
	{
		// Configure DIO3 as output

		// Read 0x0580
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_READ_REGISTER);
		SPI0_MasterSendByte((0x0580 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0580 & 0x00FF);
		SPI0_MasterSendByte(0x00);
		reg_0x0580 = SPI0_MasterRecvByte();
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);;

		// Read 0x0583
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_READ_REGISTER);
		SPI0_MasterSendByte((0x0583 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0583 & 0x00FF);
		SPI0_MasterSendByte(0x00);
		reg_0x0583 = SPI0_MasterRecvByte();
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Read 0x0584
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_READ_REGISTER);
		SPI0_MasterSendByte((0x0584 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0584 & 0x00FF);
		SPI0_MasterSendByte(0x00);
		reg_0x0584 = SPI0_MasterRecvByte();
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Read 0x0585
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_READ_REGISTER);
		SPI0_MasterSendByte((0x0585 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0585 & 0x00FF);
		SPI0_MasterSendByte(0x00);
		reg_0x0585 = SPI0_MasterRecvByte();
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Write 0x0580
		// SX126xWriteRegister(0x0580, reg_0x0580 | 0x08);
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0580 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0580 & 0x00FF);
		SPI0_MasterSendByte(reg_0x0580 | 0x08);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Write 0x0583
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0583 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0583 & 0x00FF);
		SPI0_MasterSendByte(reg_0x0583 & ~0x08);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Write 0x0584
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0584 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0584 & 0x00FF);
		SPI0_MasterSendByte(reg_0x0584 & ~0x08);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Write 0x0585
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0585 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0585 & 0x00FF);
		SPI0_MasterSendByte(reg_0x0585 & ~0x08);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		// Write 0x0920
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0920 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0920 & 0x00FF);
		SPI0_MasterSendByte(0x06);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		dio3IsOutput = true;
	}

	if (state)
	{
		// Set DIO3 High
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_READ_REGISTER);
		SPI0_MasterSendByte((0x0920 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0920 & 0x00FF);
		SPI0_MasterSendByte(0x00);
		reg_0x0920 = SPI0_MasterRecvByte();
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0920 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0920 & 0x00FF);
		SPI0_MasterSendByte(reg_0x0920 | 0x08);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);
	}
	else
	{
		// Set DIO3 Low
		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_READ_REGISTER);
		SPI0_MasterSendByte((0x0920 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0920 & 0x00FF);
		SPI0_MasterSendByte(0x00);
		reg_0x0920 = SPI0_MasterRecvByte();
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);

		SX126xWaitOnBusy();
		GPIOA_ResetBits(LORA_SS);
		//SPI_LORA.beginTransaction(spiSettings);
		SPI0_MasterSendByte(RADIO_WRITE_REGISTER);
		SPI0_MasterSendByte((0x0920 & 0xFF00) >> 8);
		SPI0_MasterSendByte(0x0920 & 0x00FF);
		SPI0_MasterSendByte(reg_0x0920 & ~0x08);
		//SPI_LORA.endTransaction();
		GPIOA_SetBits(LORA_SS);
	}
}

void SX126xAntSwOn(void)
{
		SX126xDio3Control(true);
		GPIOB_SetBits(LORA_ANTPWR);
		//digitalWrite(LORA_ANTPWR, HIGH);
}

void SX126xAntSwOff(void)
{

		SX126xDio3Control(false);
		GPIOB_ResetBits(LORA_ANTPWR);

}

void SX126xRXena(void)
{

		//digitalWrite(LORA_ANTPWR, HIGH);
		GPIOB_SetBits(LORA_ANTPWR);

		//printf("SX126xRXena\n");
	
}

void SX126xTXena(void)
{

		//digitalWrite(LORA_ANTPWR, HIGH);
		GPIOB_ResetBits(LORA_ANTPWR);
		//printf("SX126xTXena\n");
}

bool SX126xCheckRfFrequency(uint32_t frequency)
{
	// Implement check. Currently all frequencies are supported
	return true;
}

void SX126xGetStats(uint16_t *nb_pkt_received, uint16_t *nb_pkt_crc_error, uint16_t *nb_pkt_length_error)
{
	uint8_t buf[6];

	SX126xReadCommand(RADIO_GET_STATS, buf, 6);

	*nb_pkt_received = (buf[0] << 8) | buf[1];
	*nb_pkt_crc_error = (buf[2] << 8) | buf[3];
	*nb_pkt_length_error = (buf[4] << 8) | buf[5];
}

void SX126xResetStats(void)
{
	uint8_t buf[6] = {0x00};

	SX126xWriteCommand(RADIO_RESET_STATS, buf, 6);
}
