#include <stdio.h>
#include <stdlib.h>
#include "utilities.h"
#include "board.h"
#include "RegionCommon.h"
#include "cli.h"
#include "Commissioning.h"
#include "LmHandler.h"
#include "LmhpCompliance.h"
#include "LmhpClockSync.h"
#include "LmhpRemoteMcastSetup.h"
#include "LmhpFragmentation.h"
#include "LmHandlerMsgDisplay.h"
#include <config.h>
#include <HAL.h>
#include <stateMachine.h>
#include "CH58x_common.h"
#include "NT3H.h"
#include "app_i2c.h"
#include "DFLora_At.h"
#include "radio.h"
#include "NT3H.h"
#include "app_i2c.h"
#include "flash.h"
#include "csv.h"
#include "LoRaMac.h"
#include "ff.h"
#include "timeScheduler.h"
#include "dfbc.h"
#include "tla2022.h"
#include "pack.h"
#include "proprietary.h"
#include "comm.h"
#include "hw.h"
#include "stdlib.h"

#define LORAWAN_DEFAULT_CLASS                       CLASS_A
#define APP_TX_DUTYCYCLE                            40000
#define APP_TX_DUTYCYCLE_RND                        5000
#define LORAWAN_ADR_STATE                           LORAMAC_HANDLER_ADR_ON
#define LORAWAN_DEFAULT_DATARATE                    DR_0
#define LORAWAN_DEFAULT_CONFIRMED_MSG_STATE         LORAMAC_HANDLER_UNCONFIRMED_MSG
#define LORAWAN_APP_DATA_BUFFER_MAX_SIZE            242
#define LORAWAN_DUTYCYCLE_ON                        true
#define LORAWAN_DUTYCYCLE_OFF                       false

static uint8_t AppDataBuffer[256];
static LmHandlerAppData_t AppData = {
    .Buffer = AppDataBuffer,
    .BufferSize = 0,
    .Port = 0,
};

static volatile uint8_t deepSleepFlag = 0;
/*!
 * Timer to handle the application data transmission duty cycle
 */
//static TimerEvent_t TxTimer;
volatile uint8_t sendBatFlag = 0;
volatile uint8_t adnEnableFlag = 0; // ADN(auto distributing network, receive otaa config from gateway), See
//md5
uint8_t loralock = 0;
//static TimerEvent_t jionTimer;

volatile uint32_t hardwareLoopFlag = 0;
//static TimerEvent_t batTimer;
static uint8_t batSendTime = 0;
static TimerEvent_t dutyTimer;
static void OnMacProcessNotify(void);
static void OnNvmDataChange(LmHandlerNvmContextStates_t state, uint16_t size);
static void OnNetworkParametersChange(CommissioningParams_t *params);
static void OnMacMcpsRequest(LoRaMacStatus_t status, McpsReq_t *mcpsReq,
                             TimerTime_t nextTxIn);
static void OnMacMlmeRequest(LoRaMacStatus_t status, MlmeReq_t *mlmeReq,
                             TimerTime_t nextTxIn);
static void OnJoinRequest(LmHandlerJoinParams_t *params);
static void OnTxData(LmHandlerTxParams_t *params);
static void OnRxData(LmHandlerAppData_t *appData, LmHandlerRxParams_t *params);
static void OnClassChange(DeviceClass_t deviceClass);
// static void OnBeaconStatusChange(LoRaMAcHandlerBeaconParams_t *params);

#if( FRAG_DECODER_FILE_HANDLING_NEW_API == 1 )
//static int8_t FragDecoderWrite( uint32_t addr, uint8_t *data, uint32_t size );
//static int8_t FragDecoderRead( uint32_t addr, uint8_t *data, uint32_t size );
#endif
//static void OnFragProgress( uint16_t fragCounter, uint16_t fragNb, uint8_t fragSize, uint16_t fragNbLost );
//static void OnFragDone( int32_t status, uint32_t size );
//static void UplinkProcess( void );

static uint32_t joinTimes = 1;


static LmHandlerCallbacks_t LmHandlerCallbacks = {
    .GetBatteryLevel = BoardGetBatteryLevel,
    .GetTemperature = NULL,
    .GetRandomSeed = BoardGetRandomSeed,
    .OnMacProcess = OnMacProcessNotify,
    .OnNvmDataChange = OnNvmDataChange,
    .OnNetworkParametersChange = OnNetworkParametersChange,
    .OnMacMcpsRequest = OnMacMcpsRequest,
    .OnMacMlmeRequest = OnMacMlmeRequest,
    .OnJoinRequest = OnJoinRequest,
    .OnTxData = OnTxData,
    .OnRxData = OnRxData,
    .OnClassChange= OnClassChange,
    .OnBeaconStatusChange = NULL,
    .OnSysTimeUpdate = NULL,
};

static LmHandlerParams_t LmHandlerParams = {
    .Region = ACTIVE_REGION,
    .AdrEnable = LORAWAN_ADR_STATE,
    .TxDatarate = LORAWAN_DEFAULT_DATARATE,
    .PublicNetworkEnable = LORAWAN_PUBLIC_NETWORK,
	#ifdef DFROBOT_DUTYCYCLE_ON
    .DutyCycleEnabled = LORAWAN_DUTYCYCLE_ON,
	#else
    .DutyCycleEnabled = LORAWAN_DUTYCYCLE_OFF,
	#endif
    .DataBufferMaxSize = LORAWAN_APP_DATA_BUFFER_MAX_SIZE,
    .DataBuffer = AppDataBuffer};

static LmhpComplianceParams_t LmhpComplianceParams = {
    .AdrEnabled = LORAWAN_ADR_STATE,
#ifdef DFROBOT_DUTYCYCLE_ON
	.DutyCycleEnabled = LORAWAN_DUTYCYCLE_ON,
#else
	.DutyCycleEnabled = LORAWAN_DUTYCYCLE_OFF,
#endif
    .StopPeripherals = NULL,
    .StartPeripherals = NULL,
};

#define UNFRAGMENTED_DATA_SIZE (21 * 50)

/*
 * Un-fragmented data storage.
 */
//static uint8_t UnfragmentedData[UNFRAGMENTED_DATA_SIZE];

/*static LmhpFragmentationParams_t FragmentationParams =
{

    .DecoderCallbacks =
    {
        .FragDecoderWrite = FragDecoderWrite,
        .FragDecoderRead = FragDecoderRead,
    },
    .OnProgress = OnFragProgress,
    .OnDone = OnFragDone
};
*/
volatile sDFBCEvents_t dfbcEvents={0};

volatile uint8_t sendImmediately = 0;
static volatile uint8_t IsMacProcessPending = 0;
static volatile bool IsMcSessionStarted = false;
volatile uint8_t getAck = 0;
volatile uint8_t isRetrans = 0;

// mcu深度睡眠处理
static void mcuDeepSleep(void *context) {
  TimerStop(&sleepTimer);

  if(is_io_count || is_dfbc_valid || isUSBConnection()) return;

  DBG("set-------deepSleepFlag===============1");
  deepSleepFlag = 1;

}

// 大循环
static void hardwareLoop(void *context) {
  TimerStop(&hardwareLoopTimer);
  DBG("hardwareLoopFlag=1\n");
  hardwareLoopFlag = 1;
}

MibRequestConfirm_t mibReq1;

void data_blink(uint16_t ms){
	DBG("led-----on %dms\n",ms);
    GPIOB_SetBits(POWER_IND_LED);
    //指示灯
    led.delayTime = ms;
    led.startTime = getSysTime();
    led.state =1;
}

static uint32_t prev_timestamp=0;
void sendImmediatelyFunc() {
  //if(LmHandlerJoinStatus( ) != LORAMAC_HANDLER_SET) return;
  //如果没有入网成功则不发送
  if(dtuConfig.loraWanPara.joinState == 0) return;
  //两次尝试发送间隔要大于2000ms
  //
  if (getSysTime()- prev_timestamp < 2000) return;
  //if( LmHandlerIsBusy( ) == true ) return;
  prev_timestamp = getSysTime();
  lora_packet_t *node = lora_packet_get_head();
  if(node == NULL) return;
  AppData.BufferSize = node->len;
  memcpy(AppData.Buffer, node->buf, node->len);
  uint8_t len = lora_packet_list_length();
  DBG("list length------->%d,  data=%c%c%c%c\n",len,AppData.Buffer[0],AppData.Buffer[1],AppData.Buffer[2],AppData.Buffer[3]);

  if (LmHandlerSend(&AppData, node->confirmed) == LORAMAC_HANDLER_SUCCESS) {
	  lora_packet_list_del_head();
	  data_blink(300);
	  DBG("--------------------lora send success %c%c%c%c\r\n",AppData.Buffer[0],AppData.Buffer[1],AppData.Buffer[2],AppData.Buffer[3]);
  } else {
    //sysLog("[log] int send fail\r\n");
  }

}

uint32_t sendSuccessNum = 0;
uint32_t ackNum = 0;

// 指令处理 immediate
pDFSerial p;

void processImmediate() {
  // 处理loraWan协议栈的各种事件
  USB_IRQProcessHandler();
  if ( !loralock ) {
    LmHandlerProcess();
  }
  resetWdog();
  RtcProcess();
  // NFC状态机处理
  stateMachine();
  transparentProcess();
  transProcess();
  // 处理poweron 和 poweroff  不能在AT指令里面直接处理需要保存数据
  /*
  if(dtuConfig.hardPara.powerChangeFlag == true){
      sysLog("reseta\n");
      dtuConfig.hardPara.powerChangeFlag = false;
      NvmDataMgmtEvent1(LORAMAC_NVM_NOTIFY_FLAG_MAC_GROUP2);
      LmHandlerProcess( );
      softReset();
  }
  */
  // 处理AT指令
  p = recvAt();
  if (p != NULL) {
    execAtCmd(p);
  }
  sendImmediatelyFunc();

  // 指示灯处理
  led_indicator();
  // 中断产生，需要立即发送给上位机和loraWan的数据
  if (sendBatFlag == 1) {
    sendBatFlag = 0;
    uint8_t bat = getBatteryLevel();
    TimerStop( &sleepTimer );
    deepSleepFlag = 0;
    TimerStop(&hardwareLoopTimer);
    memset(AppDataBuffer, 0, 3);
    uint8_t packetLen = 2;
    sHWMasks_t *appmask = AppDataBuffer;
    uint8_t    *appbuf  = AppDataBuffer+2;
    uint8_t len = packBattery(appmask, appbuf);

    packetLen += len;
    batSendTime--;
    if (batSendTime == 0) {
      //  TimerStop( &batTimer );
      uint32_t duration = randr(10000, dtuConfig.hardPara.loopTime + 10000);
      TimerSetValue(&hardwareLoopTimer,
                    duration);
      TimerStart(&hardwareLoopTimer);
      if (getAck == 0) {
        // 再次入网
        dtuConfig.saveMask.joinType = 1;
        writeLog("bat rejoin\n", getSysTime());
        sysLog("bat rejoin\n");
        DelayMs(randr( 1, 3000 ));
        softReset();
      }

      {  // 统计信息初始 化
        retransNum = 0;
        senaNum = 0;
        ackNum = 0;
      }
      // deepSleepFlag = 1;
      return;
    } else if ((batSendTime == 1) && (getAck == 0)) {
      setDataRate(0);
      sysLog("bat adr\n");
    }
    AppData.Buffer = AppDataBuffer;
    AppData.BufferSize = packetLen;
    DBG("Battery----%02x %02x %02x\n",AppDataBuffer[0], AppDataBuffer[1], AppDataBuffer[2]);
    if (LmHandlerSend(&AppData, true) == LORAMAC_HANDLER_SUCCESS) {
      DBG("[log] bat send success\r\n");
    } else {
      batSendTime++;
      DBG("[log] bat send fail\r\n");
    }
    setDataRate(dtuConfig.loraWanPara.dataRate);
    // TimerStart( &batTimer );
  }
}
//volatile uint8_t chargingInt=0;

// IO 输入口中断
__INTERRUPT
__HIGH_CODE
void GPIOB_IRQHandler() {
  static uint32_t previousTime = 0;
  uint32_t currentTime = getSysTime(); /* 获取当前时间，单位为毫秒 */
  
#if 1
  // ioin1
  if (currentTime - previousTime >= 100) {
    if ((GPIOB_ReadITFlagBit(GPIO_Pin_14) & R16_PB_INT_EN) != 0) {
      if (dtuConfig.loopOperation.ioIn[0].enable) {
        sendImmediately = 1;
      }
      // 使能了IO计数
      // 复位不清零，flash寿命问题 可能不做
      if (dtuConfig.loopOperation.ioCount[0].enable == 0x01) {
        dtuConfig.loopOperation.ioCount[0].Count++;
      }
      GPIOB_ClearITFlagBit(GPIO_Pin_14);
      reverseIntDetect(0);
    }
    previousTime = currentTime;  // 更新上一次触发时间
  }
  // ioin2
  static uint32_t previousTime1 = 0;
  uint32_t currentTime1 = getSysTime(); /* 获取当前时间，单位为毫秒 */
  ;
  if (currentTime1 - previousTime1 >= 100) {
    if ((GPIOB_ReadITFlagBit(GPIO_Pin_15) & R16_PB_INT_EN) != 0) {
      if (dtuConfig.loopOperation.ioCount[1].enable == 0x01) {
        dtuConfig.loopOperation.ioCount[1].Count++;
      }
      if (dtuConfig.loopOperation.ioIn[1].enable) {
        sendImmediately = 2;
      }
      GPIOB_ClearITFlagBit(GPIO_Pin_15);
      reverseIntDetect(1);
    }
    previousTime1 = currentTime1;  // 更新上一次触发时间
  }
  // ioin3
  static uint32_t previousTime2 = 0;
  uint32_t currentTime2 = getSysTime(); /* 获取当前时间，单位为毫秒 */
  ;
  if (currentTime2 - previousTime2 >= 100) {
    if ((GPIOB_ReadITFlagBit(GPIO_Pin_0) & R16_PB_INT_EN) != 0) {
      if (dtuConfig.loopOperation.ioCount[2].enable == 0x01) {
        dtuConfig.loopOperation.ioCount[2].Count++;
      }
      if (dtuConfig.loopOperation.ioIn[2].enable) {
        sendImmediately = 3;
      }
      GPIOB_ClearITFlagBit(GPIO_Pin_0);
      reverseIntDetect(2);
    }
    previousTime2 = currentTime2;  // 更新上一次触发时间
  }

  if ((GPIOB_ReadITFlagBit(NT3H_INT_PIN) & R16_PB_INT_EN) != 0) {
    // PRINT("\r\nINT!\r\n\r\n");
    if (FALSE == NT3HReady) {
      NT3HReady = TRUE;
    }
    GPIOB_ClearITFlagBit(NT3H_INT_PIN);
  }
#endif
  // 保存信息到 eeprom
  // NvmDataMgmtEvent1(LORAMAC_NVM_NOTIFY_FLAG_MAC_GROUP2);
}

// 指示灯处理
//指示灯处理
void led_indicator(){
    if(led.state >0){
      if(getSysTime()- led.startTime > led.delayTime)
      {
    	  DBG("led1----ooooooff curr=%d  startTime=%d  delayTime=%d\n",getSysTime(), led.startTime, led.delayTime);
          led.state = 0;
          GPIOB_ResetBits(POWER_IND_LED);
      }
      if(getSysTime()< led.startTime ){
    	  DBG("led2----ooooooff curr=%d  startTime=%d  delayTime=%d\n",getSysTime(), led.startTime, led.delayTime);
          led.state = 0;
          GPIOB_ResetBits(POWER_IND_LED);
      }
    }
}

sCsvData_t csvData;

// 大循环处理上报
void hardwareLoopFunction() {
  uint16_t bufferLen = 0;

    //如果没有入网，发起入网请求，重启定时器30s，返回
    //如果已经入网，才读取数据发送给网关
  uint8_t lora_busy = LmHandlerIsBusy( );
    if( lora_busy == true ){ ////////////////wanghui 如果没有入网会返回busy吗
        //sysLog("loraWan IsBusy\n");
        if( LmHandlerJoinStatus( ) != LORAMAC_HANDLER_SET ) {    ///如果没有入网
           TimerSetValue( &hardwareLoopTimer, 10000*joinTimes+ randr( 30000, 60000 ) );
           TimerStart( &hardwareLoopTimer );
           hardwareLoopFlag = 0;
           return ;
        }
        /*if(!is_dfbc_valid){  ///如果已经入网 并且 不存在字节码
            TimerSetValue( &hardwareLoopTimer, 20000*joinTimes + randr( 30000, 60000 ));
            TimerStart( &hardwareLoopTimer );
            return;
        }*/
    }

    uint32_t loopTime = dtuConfig.hardPara.loopTime; //getNextTime();//dtuConfig.hardPara.loopTime + randr( -2000, 2000 );
    //如果大循环间隔时间为0,则表示要停止大循环
    //printf("loopTime=%d\n",loopTime);
    if(loopTime == 0){
        TimerStop( &hardwareLoopTimer );
        return;
    }

    TimerTime_t start = getSysTime();

    DBG("batSendTime=%d\n",batSendTime);
    if( batSendTime && (dtuConfig.loraWanPara.joinType == ACTIVATION_TYPE_ABP))
    	goto out;
    //入网次数清零，如果没有入网成功，通过入网次数来慢慢地增加入网间隔
    joinTimes = 0;
    memset(AppDataBuffer, 0, sizeof(AppDataBuffer));
    sHWMasks_t *appmask = AppDataBuffer;
    uint8_t *appdata = AppDataBuffer+2;

    //输出VOUT，为外部传感器预热
    if(is_dfbc_valid) {
    	dtuConfig.loopOperation.power[0].enable = 1;
    	dtuConfig.loopOperation.power[0].supplyTime = 0;
    }

    if(dtuConfig.loopOperation.power[0].enable) {
    	int32_t total = dtuConfig.loopOperation.power[0].supplyTime;
    	power12V(1);
    	//获取系统时间
    	TimerTime_t cur = getSysTime();
    	DBG("vout on total=%d     cur=%d\n",total, cur);
    	while((getSysTime() - cur) < total){
    		processImmediate();
    		DelayMs(2);
    	}
    }

    uint8_t len = packAdc(appmask, appdata+bufferLen);
    bufferLen += len;

    len = packCount(appmask, appdata+bufferLen);
    bufferLen += len;

    len = packIOIN(appmask, appdata+bufferLen);
    bufferLen += len;

    len = packIOOUT(appmask, appdata+bufferLen);
    bufferLen += len;

    len = packModbus(appmask, appdata+bufferLen);
    bufferLen += len;

    //关闭12V电源
    if(dtuConfig.loopOperation.power[0].enable){
    	int32_t total = dtuConfig.loopOperation.power[0].supplyTime;
        if(total != 0){
        	DBG("vout off total=%d\n",total);
    	    power12V(0);
        }
    }

    //reTranNum是需要补传的数据,每次上传失败都会+1；
    //getAck 是上一个包获取了ack,表明能正常和网关通信，则可以开始补传
    if(dtuConfig.loopOperation.reTranNum > 0 && getAck == 1){
        dtuConfig.loopOperation.reTrans = TRUE;
    }

    AppData.BufferSize = bufferLen+2;
    printPacket(&AppData);
    if(is_dfbc_valid || lora_busy){
    	lora_packet_list_append(AppData.Buffer ,AppData.BufferSize, dtuConfig.loraWanPara.isTxConfirmed);
    }else{
    	data_blink(300);
    	if( LmHandlerSend( &AppData, dtuConfig.loraWanPara.isTxConfirmed ) == LORAMAC_HANDLER_SUCCESS ){
    	        sprintf(str,"[log] send success,num= %d\r\n",++sendSuccessNum);
    	        senaNum++;
    	        sysLog(str);
    	        setPackCount();
    	        data_blink(300);
    	    }else{
    	        sysLog("[log] send fail\r\n");
    	        dtuConfig.loopOperation.reTranNum++;
    	    }
    }
out:
    //处理当 供电时间和loop时间接近的情况
    {
    	TimerTime_t end = getSysTime();
    	if(end <= start+loopTime){
    		int32_t nextTime =loopTime-(end-start) ;
    		if(nextTime<4000){
    			nextTime = 4000;
    		}
    		TimerSetValue( &hardwareLoopTimer,  nextTime );
    		TimerStart( &hardwareLoopTimer );
    	}else{
    		TimerSetValue( &hardwareLoopTimer, 4000 );
    		TimerStart( &hardwareLoopTimer );
    	}
    }
}


static void unixEpoch( void* context ){
    dtuConfig.hardPara.unixEpoch += 5;
    TimerStart( &unixEpochTimer );
    printf("u\n");
}
//透传处理程序
uint8_t dataLen = 0;
//占空比标志
volatile uint8_t dutyFlag = 0;

static void transProcessEvent( void* context ){
    TimerStop( &transTimer );
    if( dtuConfig.hardPara.loopTime==0 && dtuConfig.loraWanPara.uartMode == 1 && dtuConfig.loraWanPara.joinState == 1){
        transProFlag = 1;
    }else{
        transProFlag = 0;
   }
}

static void  dutyProcess( void* context ){
    TimerStop( &dutyTimer);
    dutyFlag = 0;
    transProFlag = 1;
}

/////这个是处理串口数据透传的吗
void transProcess(void){
     if(transProFlag && dutyFlag == 0){
         static uint32_t sendAll = 0;
        if(!LoRaMacIsBusy()){
          if(tansFlag == TRANSTART){
            uint8_t maxN =getloraMaxL();
            dataLen = packetDequeue(maxN,AppDataBuffer);
            if(dataLen == 0){
                TimerStop( &transTimer );
                tansFlag = TRANSEND;
                return;
            }
            //有数据则发送给网关

          }else if(tansFlag == RETRANS){

           // printf("RETRANS\n");
          }else if(tansFlag == TRANSEND){
              return ;
          }
          AppData.BufferSize = dataLen;
          LmHandlerErrorStatus_t status = LmHandlerSend( &AppData, false );
          if( status == LORAMAC_HANDLER_SUCCESS ){
                //printf("len=%d\n",dataLen);
                //DelayMs(25);
                //数据统计
                sendAll = sendAll+ dataLen;
                sprintf(str,"len=%d,sendAll=%d\n",dataLen,sendAll);
                sysLog(str);
                //数据已发送 或传入缓冲区待发送
                sysLog("success\r\n");
                tansFlag = TRANSSUCCESS;
            }else if(status == LORAMAC_HANDLER_DUTY){
                tansFlag = RETRANS;
                sprintf(str,"dytyTime=%d\n",dtuConfig.hardPara.dutyTime);
                sysLog(str);
                dutyFlag = 1;
                if(dtuConfig.hardPara.dutyTime <3600)dtuConfig.hardPara.dutyTime = 3600;
                TimerStop( &dutyTimer );
                TimerSetValue( &dutyTimer, dtuConfig.hardPara.dutyTime);
                TimerStart( &dutyTimer );
            }else{
                TimerStop( &transTimer );
                TimerSetValue( &transTimer, 3600);
                TimerStart( &transTimer );
            }
        }else{

        TimerSetValue( &transTimer, 3600);
        TimerStart( &transTimer );
    }
        transProFlag = 0;
   }
}

void DebugInit(void)
{
    GPIOPinRemap(ENABLE, RB_PIN_UART1);
    GPIOB_SetBits(GPIO_Pin_13);
    GPIOB_ModeCfg(GPIO_Pin_12, GPIO_ModeIN_PU);
    GPIOB_ModeCfg(GPIO_Pin_13, GPIO_ModeOut_PP_5mA);
    UART1_DefInit();
    PFIC_DisableIRQ(UART1_IRQn);

    sendSuccessNum = 0;
    ackNum = 0;
    senaNum = 0;
    retransNum = 0;
}

void dumpDtuConfig(void){
	uint8_t * p = (uint8_t *)&dtuConfig;
	uint16_t len = sizeof(dtuConfig);
	printf("-----------------------------const char config[]={\n");
	for(uint16_t i=0; i < len; i++){
		if(i%64 == 0) printf("\n");
		printf("0x%02X, ",p[i]);
	}
	printf("\n}\n");
}
void debugFloat(){
    static char xxx[10]={"3.14"};
    float pi;
    char fbuf[20];
    DBG("x = %s\n",xxx);
    sscanf(xxx,"%f",&pi);
    sprintf(fbuf,"%.2f",pi);
    DBG("pi = %.2f\n",pi);
    DBG("fbuf = %s\n",fbuf);
    float rlt = strtof(xxx, NULL);
    sprintf(fbuf,"%.2f",rlt);
    DBG("rlt = %.2f\n",rlt);
    DBG("fbuf = %s\n",fbuf);
    float fpi = 3.14;
    sprintf(fbuf,"%.2f",fpi);
    DBG("fpi = %.2f\n",fpi);
    DBG("fpi*100 = %d\n",(int)(fpi*100));
    DBG("fbuf = %s\n",fbuf);
}

void sysInit() {
    static uint8_t sn[9]={0};
    SetSysClock(CLK_SOURCE_PLL_60MHz);
    DebugInit();
    printf("\n\n\n\n-----------------Restart----------------------------\n\n");
    RtcInit( );

    debugFloat();


    //初始化sx126x的通信接口和引脚
    BoardInitMcu( );
    ////////////////////放在这里打印就没问题///////////////////////////////////////////
    debugFloat();

    GPIOA_ModeCfg(GPIO_Pin_3, GPIO_ModeOut_PP_5mA);
    GPIOA_ResetBits(GPIO_Pin_3);
    debugFloat();
    DBG("----------sizeof(dtuConfig)=%d\n", sizeof(dtuConfig));
    debugFloat();
    loadDtuConfig();
    debugFloat();

    //dumpDtuConfig();
    DBG("dtuConfig.loraWanPara.antennGain=%.2f\n",dtuConfig.loraWanPara.antennGain);
    dtuConfig.loraWanPara.joinState = 0;
    if ( LmHandlerInit( &LmHandlerCallbacks, &LmHandlerParams ) != LORAMAC_HANDLER_SUCCESS ) {
        //printf( "[log] LoRaMac wasn't properly initialized\n" );
        //writeLog("[ERROR]LoRaMac init failed\n",getSysTime());
    }

    printf("adnDisable===========%d\n",dtuConfig.loopOperation.adnDisable);
    //printf("singleFreq===========%d\n",dtuConfig.loraWanPara.singleFreq);
    //printf("dtuConfig.loopOperation.countMask===========%d\n", dtuConfig.loopOperation.countMask);
    if(dtuConfig.loopOperation.countMask){
        is_io_count = 1;
        enable3V3();
    }else{
        is_io_count = 0;
    }



    //以毫秒为单位的系统总体定时误差
    LmHandlerSetSystemMaxRxError( 20 );
    LmHandlerPackageRegister( PACKAGE_ID_COMPLIANCE, &LmhpComplianceParams );




    //AT指令初始化
    AtBegin();
    //随机延时
    DelayMs(randr(0,200));

    TimerInit(&transTimer,transProcessEvent);
    TimerSetValue( &transTimer, 120);

    //通信接口初始化
    uartUsbInit();
    //透传处理定时器





    GET_UNIQUE_ID( sn );
    DBG("\n------------sn=%02X%02X%02X%02X%02X%02X%02X%02X\n",sn[0],sn[1],sn[2],sn[3],sn[4],sn[5],sn[6],sn[7]);
    md5_md5(sn,8,str);
    memcpy(dtuConfig.hardPara.sn, str, 8);
#ifdef SN_AS_EUI
    memcpy(dtuConfig.loraWanPara.devEui, str, 8);
#endif
    //DBG("------------sn=%02X%02X%02X%02X%02X%02X%02X%02X\n",dtuConfig.hardPara.sn[0],dtuConfig.hardPara.sn[1],dtuConfig.hardPara.sn[2],dtuConfig.hardPara.sn[3],dtuConfig.hardPara.sn[4],dtuConfig.hardPara.sn[5],dtuConfig.hardPara.sn[6],dtuConfig.hardPara.sn[7]);
    printf("dtuConfig.loraWanPara.region = %d dtuConfig.loraWanPara.subBand=%d\n", dtuConfig.loraWanPara.region, dtuConfig.loraWanPara.subBand);

}

void loop() {
        //处理数据储存
        NvmStoreDTUConfig();
        //不能受delay延时影响的逻辑
        processImmediate();
        handleTxtimeout();
        //开始大循环
        if ((AdnStatus() != ADN_DISABLED)  && (AdnStatus() != ADN_EXIT)) {
            //disable otaa when ADN working
            int ret = processAdn();
            if(ret == -1){
                if(is_io_count || is_dfbc_valid || isUSBConnection()){
                    deepSleepFlag = 0;
                }else{
                    DBG("set-------deepSleepFlag===============1");
                    deepSleepFlag = 1;
                }
            }else{
                processImmediate();
            }
        }else{
            if(is_dfbc_valid){ //字节码有效
                static uint32_t prevHW=0, prevModbus=0;
                uint32_t time = getSysTime();
                deepSleepFlag = 0;
                if(time - prevHW >= 1000){
                    pollHW();
                    //DBG("time=%d   prev=%d\n", time, prevHW);
                    dfbc_loop();
                    prevHW = time;
                }
                if(time - prevModbus >= 5000){
                    pollModbus();
                    prevModbus = time;
                }
            }
            if (hardwareLoopFlag == 1) {
                        printf("\n\n\n----------------hardwareLoop----------------------\n");
                        hardwareLoopFlag = 0;
                        hardwareLoopFunction();
            }
        }

        //if(chargingInt){
        //  chargingInt = 0;
        //  processCharging();
        //}
        //在空闲时间保存配置
        if(dtuConfig.saveMask.value != 0){
            //查看loraWan是否处于繁忙状态
            if( LoRaMacIsBusy( ) != true ) {
                //同步loraWan配置
                syncConfig();
            }
        }

         //处理低功耗
        if(deepSleepFlag == 1) {
            TimerStop( &sleepTimer );
#ifdef  SLEEP_ENABLE
         if(isUSBConnection() == 0) {
           sysLog("[log] deep sleep\n");

           mDelaymS(10);
           //射频芯片进入低功耗
           Radio.Sleep();
           if(getMcuSleep() == 1){
               TimerStop( &hardwareLoopTimer );
           }
           //引脚置位
           DBG("--------alllPinsPowerDown-------------\n");
           alllPinsPowerDown();

           //唤醒条件
           PWR_PeriphWakeUpCfg( ENABLE, RB_SLP_RTC_WAKE, Long_Delay );

           //mcu进入低功耗
           LowPower_Sleep( RB_PWR_RAM30K | RB_PWR_RAM2K );

           HSECFG_Current( HSE_RCur_100 );

           //重新初始化USB UART RS485
           uartUsbInit();
           sysLog("[log] To wake up from low power, please enter AT instruction\n");
           //SX126 io重新口初始化
           SX126xIoReInit();
           IOinit();
           startWdog();
           //writeLog(" deep sleep\r\n",getSysTime());
          }
          deepSleepFlag = 0;
#endif
        }
}

void sysInit2(){
    is_dfbc_valid = dfbc_valid();
    DBG("is_dfbc_valid=%d\n",is_dfbc_valid);
    if(is_dfbc_valid){
        enable3V3();
        builtin_init();
        dfbc_init();
    }

    //通过配置来恢复IO口的配置和状态
    IOinit();
    //MD5校验
    checkMD5();
    USB_IRQProcessHandler();

    //getBatteryPower();
    //低功耗控制定时器
    TimerInit( &sleepTimer, mcuDeepSleep );
    //大循环的控制定时器 ，在工作模式下打开，在入库模式下不打开，串口打印一条信息后直接睡眠
    //退出入库模式，插入USB或刷NFC,发启用该timer
    //对应的指令是AT+POWERON

    //都叫hardwareLoop
    TimerInit(&hardwareLoopTimer, hardwareLoop);
    TimerSetValue( &hardwareLoopTimer, dtuConfig.hardPara.loopTime);

    TimerInit(&dutyTimer, dutyProcess);
    TimerSetValue( &dutyTimer, 100000);

    //上电即开始进入大循环，主要是为了入网
    if(getMcuSleep() == 1){ //入库模式
        TimerSetValue( &sleepTimer, 3000 );
        TimerStart( &sleepTimer );
        //writeLog("power off\r\n",getSysTime());
    } else/* if(dtuConfig.loraWanPara.joinType == ACTIVATION_TYPE_OTAA)*/ { //先凑合着用，后面要把入网逻辑和大循环和发送数据逻辑相分离
        hardwareLoop(NULL);
    }/* else if(dtuConfig.loraWanPara.joinType == ACTIVATION_TYPE_ABP){
            DBG("ABP JOIN.........\n");
            LmHandlerSend( &AppData, false );
        }*/

    initAdn();
    startWdog();
    initLog();
    writeLog("--reset---\n",getSysTime());
    //chargingInt = 1;
}
//主程序
int main( void )
{
    sysInit();
    while(1); /////去掉这个while(1) 打印浮点数就出错了  ////请帮忙看下  //打印口是UART1 (PB12 PB13)
	sysInit2();

	while(1){
        loop();
    }

}

static void OnMacProcessNotify( void )
{
    IsMacProcessPending = 1;
}

static void OnNvmDataChange( LmHandlerNvmContextStates_t state, uint16_t size )
{
    DisplayNvmDataChange( state, size );
}

static void OnNetworkParametersChange( CommissioningParams_t* params )
{
    DisplayNetworkParametersUpdate( params );
}

static void OnMacMcpsRequest( LoRaMacStatus_t status, McpsReq_t *mcpsReq, TimerTime_t nextTxIn )
{
   // DisplayMacMcpsRequestUpdate( status, mcpsReq, nextTxIn );
}

static void OnMacMlmeRequest( LoRaMacStatus_t status, MlmeReq_t *mlmeReq, TimerTime_t nextTxIn )
{
   // DisplayMacMlmeRequestUpdate( status, mlmeReq, nextTxIn );
   // printf("status=%d\n",status);
}

static void OnJoinRequest( LmHandlerJoinParams_t* params )
{
    //DisplayJoinRequestUpdate( params );
    if( params->CommissioningParams->IsOtaaActivation == true ) {
        if( params->Status == LORAMAC_HANDLER_SUCCESS ) {
            DBG("[log] OTAA SUCCESS\n");
            joinTimes = 1;
            dtuConfig.loraWanPara.joinState =1;
            batSendTime = 2;
            data_blink(5000);
            getAck = 0;
            DBG("\n");
            prev_timestamp = getSysTime();
            //USB1Print("[log] OTAA SUCCESS\n");
            //TimerSetValue( &TxTimer, APP_TX_DUTYCYCLE +5000);
            //TimerStart( &TxTimer );
            dtuConfig.loopOperation.fakeJoinFlag = 1;
        }else{
            joinTimes++;
            sysLog("[log]JOIN FAIL!\n");
            dtuConfig.loraWanPara.joinState = 0;
        }

    } else {
        sysLog("[log] ABP SUCCESS\n");
        dtuConfig.loraWanPara.joinState =1;
        prev_timestamp = getSysTime();
        // USB1Print("[log] ABP SUCCESS\n");
        batSendTime = 2;
        getAck = 1;
        /*GPIOB_ResetBits(POWER_IND_LED);
        GPIOB_SetBits(POWER_IND_LED);
        //指示灯
        led.delayTime = 5000;
        led.startTime = getSysTime();
        led.state =1;*/
       // return ;
    }

    //在入网时会更改速率
    MibRequestConfirm_t mibReq;
    mibReq.Type = MIB_CHANNELS_DATARATE;
    mibReq.Param.ChannelsDatarate = dtuConfig.loraWanPara.dataRate;
    LoRaMacMibSetRequestConfirm( &mibReq );
    //dtuConfig.saveMask.dataRate = 0;

    if( params->Status == LORAMAC_HANDLER_ERROR ){
        //入网失败则进入低功耗，等待下一次唤醒 再入网
        TimerSetValue( &sleepTimer, 1000);
        TimerStart( &sleepTimer );
    } else {
        LmHandlerRequestClass( dtuConfig.loraWanPara.class );
         //入网成功则立即开始发送电池包
        sendBatFlag =1;
    }
}

//当完成一次传输会进入到此函数(包括重传)
//上一个包的数据处理完成,立马进行下一个包的处理
static void OnTxData( LmHandlerTxParams_t* params )
{
   // printf( "TX POWER    : %d\n", params->TxPower );
      if(batSendTime ==0 ){
          if(dtuConfig.loraWanPara.class == CLASS_A && !isTxPending()){
              TimerSetValue( &sleepTimer, 600);
              TimerStart( &sleepTimer );
          }else{
              TimerStop( &sleepTimer );
          }
      }else{
          TimerStop( &sleepTimer );
          sendBatFlag = 1;
      }
   //  printf("isTxPending=%d\n",);
    //当开启透传时,需要执行此函数处理代码
    if( (dtuConfig.hardPara.loopTime == 0) && (tansFlag == TRANSSUCCESS)){
        //printf("txdone\n")
        TimerSetValue( &transTimer, 300);
        TimerStart( &transTimer );
        tansFlag = TRANSTART;
    }
    sprintf(str,"send all num=%d,retry=%d\n",senaNum,retransNum);
    sysLog(str);
    sprintf(str,"Loss Rate (%d/%d)\n",ackNum,sendSuccessNum);
    sysLog(str);
    retransNum = 0;
}

static uint32_t ackCount = 0;
//当接收到数据会进入此函数
static void OnRxData( LmHandlerAppData_t* appData, LmHandlerRxParams_t* params )
{
    ackCount++;
    if( appData->BufferSize != 0 )
    {
        sysLog("[log] receive a data:");
        if(appData->Port == 20){
            dfserials[2].dataFlag = 1;
            dfserials[2].datalength = appData->BufferSize+2;
            strncpy(dfserials[2].recvBuffer,appData->Buffer,appData->BufferSize);
            dfserials[2].recvBuffer[dfserials[2].datalength-2]='\r';
            dfserials[2].recvBuffer[dfserials[2].datalength-1]='\n';
            sprintf(str,"at=%s\n",dfserials[2].recvBuffer);
            sysLog(str);
        }else{
            transToTTL((uint8_t*)appData->Buffer,appData->BufferSize);
            uint8 len = appData->BufferSize;
            if(len > 19) len = 19;
            dfbcEvents.LORAWAN_RECV = 1;
            memcpy(dfbcEvents.lorawan_buf, appData->Buffer, len);
            dfbcEvents.lorawan_buf[len] = 0;
            data_blink(300);
        }
    }else{
        sprintf(str,"[log] receive a ack,num= %d\n",++ackNum);
        sysLog(str);

        if(params->DownlinkCounter < 3){
            writeLog("ack re\n",getSysTime());
            getAck = 1;
            dtuConfig.loopOperation.fakeJoinFlag = 0;
        }
        sprintf(str,"[re]Loss Rate =(%d/%d)\n",ackNum,sendSuccessNum);
        sysLog(str);
        setAckCount();
    }

    sprintf(str,"[Downlink fnCnt] %d\n",params->DownlinkCounter);
    // writeLog(str, getSysTime());
    sysLog(str);
}

static void OnClassChange( DeviceClass_t deviceClass )
{
    DisplayClassUpdate( deviceClass );

    switch( deviceClass )
    {
        default:
        case CLASS_A:
        {
            IsMcSessionStarted = false;
            break;
        }
        case CLASS_B:
        {
            // Inform the server as soon as possible that the end-device has switched to ClassB
            LmHandlerAppData_t appData =
            {
                .Buffer = NULL,
                .BufferSize = 0,
                .Port = 0
            };
            sysLog("OnClassChange\r\n");
            LmHandlerSend( &appData, LORAMAC_HANDLER_UNCONFIRMED_MSG );
            IsMcSessionStarted = true;
            break;
        }
        case CLASS_C:
        {
            IsMcSessionStarted = true;
            break;
        }
    }
}

