/*
 * pack.c
 *
 *  Created on: May 19, 2024
 *      Author: Administrator
 */

#include "pack.h"
#include "dfbc.h"
#include "comm.h"

// 交换16位整数的字节序
uint16_t swap16(uint16_t value) {
    return (value >> 8) | (value << 8);
}

// 交换32位整数的字节序
uint32_t swap32(uint32_t value) {
    return ((value >> 24) & 0x000000FF) |
           ((value >> 8)  & 0x0000FF00) |
           ((value << 8)  & 0x00FF0000) |
           (value << 24);
}

uint8_t packAdc(sHWMasks_t *appmask, uint8_t *buf){
    if(dtuConfig.loopOperation.adc[0].enable == 1){
    	DBG("before appmask=%2x\n", appmask->hw0Mask);
    	enable3V3();
        tla202xBegin();
        uint16_t adc_value = tla202xADC0()*5;
        disable3V3();
        appmask->hw0Mask.bits.ADC0 = 1;

        buf[0] = adc_value&0xFF;
        buf[1] = adc_value>>8;
        DBG("after appmask=%2x\n", appmask->hw0Mask);
        return 2;
    }
    return 0;
}

uint8_t packCount(sHWMasks_t *appmask, uint8_t *buf){
	uint8_t ret = 0;
	for(uint8_t i = 0; i < 2; i++){
        if(dtuConfig.loopOperation.ioCount[i].enable == 1){
        	DBG("before appmask=%2x\n", appmask->hw0Mask);
        	if(i == 0){
        		appmask->hw0Mask.bits.IOCNT0 = 1;
        	}else{
        		appmask->hw0Mask.bits.IOCNT1 = 1;
        	}
            buf[ret++] = dtuConfig.loopOperation.ioCount[i].Count;
            buf[ret++] = (dtuConfig.loopOperation.ioCount[i].Count>>8)&0xff;
            DBG("after appmask=%2x\n", appmask->hw0Mask);
        }
    }
	return ret;
}

uint8_t packCountN(sHWMasks_t *appmask, uint8_t *buf, uint8_t i){
	uint8_t ret = 0;

    if(dtuConfig.loopOperation.ioCount[i].enable == 1){
    	DBG("before appmask=%2x\n", appmask->hw0Mask);
    	if(i == 0){
    		appmask->hw0Mask.bits.IOCNT0 = 1;
        } else {
        	appmask->hw0Mask.bits.IOCNT1 = 1;
        }
        buf[ret++] = dtuConfig.loopOperation.ioCount[i].Count;
        buf[ret++] = (dtuConfig.loopOperation.ioCount[i].Count>>8)&0xff;
        DBG("after appmask=%2x\n", appmask->hw0Mask);
    }

	return ret;
}

uint8_t packIOIN(sHWMasks_t *appmask, uint8_t *buf){
	uint8_t ret = 0;
    uint8_t ioValue = buf[0];
    uint8_t value;
    enable3V3();
    DelayMs(50);
    for(uint8_t i=0 ;i<2; i++){
        if(dtuConfig.loopOperation.ioIn[i].enable == 1){
        	DBG("before appmask=%2x\n", appmask->hw0Mask);
        	ret = 1;
        	if(i == 0){
        		appmask->hw0Mask.bits.IOIN0 = 1;
        	}else{
        		appmask->hw0Mask.bits.IOIN1 = 1;
        	}
            if(GPIOB_ReadPortPin(dtuConfig.loopOperation.ioIn[i].pin) !=0){
                value = 0;
            }else{
                value = 1;
            }
            ioValue |= value << (i+2);
            buf[0] = ioValue;
            DBG("after appmask=%2x\n", appmask->hw0Mask);
        }
    }
    disable3V3();
    return ret;
}

uint8_t packIOINN(sHWMasks_t *appmask, uint8_t *buf, uint8_t i){
	uint8_t ret = 0;
    uint8_t ioValue = buf[0];
    uint8_t value = 1;
    enable3V3();
    DelayMs(50);
    if(dtuConfig.loopOperation.ioIn[i].enable == 1) {
        DBG("before appmask=%2x\n", appmask->hw0Mask);
        ret = 1;
        if(i == 0){
        	appmask->hw0Mask.bits.IOIN0 = 1;
        } else {
        	appmask->hw0Mask.bits.IOIN1 = 1;
        }
        if(GPIOB_ReadPortPin(dtuConfig.loopOperation.ioIn[i].pin) !=0){
            value = 0;
        }
        ioValue |= value << (i+2);
        buf[0] = ioValue;
        DBG("after appmask=%2x\n", appmask->hw0Mask);
    }
    disable3V3();
    return ret;
}

uint8_t packBattery(sHWMasks_t *appmask, uint8_t *buf){
	appmask->hw0Mask.bits.BAT = 1;
    buf[0] = 100;
    return 1;
}

uint8_t packIOOUT(sHWMasks_t *appmask, uint8_t *buf){
	uint8_t ret = 0;
    uint8_t ioValue = buf[0];
    uint8_t value;
    return ret;
}

uint8_t packModbus(sHWMasks_t *appmask, uint8_t *buf){
	uint8_t ret = 0;
	uint8_t index = 0;
	uint8_t dataLen;
	DBG("---dtuConfig.loopOperation.rs485Mask=0x%04X\n",dtuConfig.loopOperation.rs485Mask);
    //最大支持八个指令，多测几个 和不同的数据类型
	if(modbuslock){

	}
    for (uint8_t i = 0; i < 8; i++) {
        if (dtuConfig.loopOperation.rs485item[i].enable == 1) {
        	uint8_t rawdata[30],rawdatalen;
        	float value;

        	appmask->modbusMask |= (1 << i);
        	if(execModbusItem(i, rawdata, &rawdatalen, &value) == 0){
                uint8_t *_p = (uint8_t *)&value;
                buf[index++] = (1<<6) + (dtuConfig.loopOperation.rs485item[i].byteOrder << 3) + 4;

                for (uint8_t k = 0; k < 4; k++) {
                    buf[index++] = _p[k];
                }
        	}else{
        		buf[index++] = 0x78;
        	}
        }
    }
    DBG("return index=%d\n",index);
    return index;
}


uint8_t packModbusN(sHWMasks_t *appmask, uint8_t *buf, uint8_t i){
	uint8_t dataLen;
	uint8_t index = 0;
    //最大支持八个指令，多测几个 和不同的数据类型

    if (dtuConfig.loopOperation.rs485item[i].enable == 1) {
        	uint8_t rawdata[30],rawdatalen;
        	float value;

        	appmask->modbusMask |= (1 << i);
        	if(execModbusItem(i, rawdata, &rawdatalen, &value) == 0){
                uint8_t *_p = (uint8_t *)&value;
                buf[index++] = (1<<6) + (dtuConfig.loopOperation.rs485item[i].byteOrder << 3) + 4;

                for (uint8_t k = 0; k < 4; k++) {
                    buf[index++] = _p[k];
                }
        	}else{
        		buf[index++] = 0x78;
        	}
    }

    DBG("return index=%d\n",index);
    return index;
}


