#include "proprietary.h"

#include "DFLora_At.h"
#include "LmHandler.h"
#include "aes.h"
#include "cmac.h"

#define KEYREQ_LENGTH sizeof(KeyReq_t)
#define KEYANS_LENGTH sizeof(KeyAns_t)

#define UINT32_2_BYTES(x) (uint8_t) x, (uint8_t)(x >> 8), (uint8_t)(x >> 16), (uint8_t)(x >> 24)

void onReceiveKeyAns(ProprietaryPkt_t *pkt);
void ScheduleAdn(void *context);
void ScheduleRbt(void *context);
void disableAdn();

bool aes128_encrypt(const uint8_t *in, uint8_t *out, uint8_t size, const uint8_t *key);
bool aes128_clacmic(uint32_t *mic, const uint8_t *dat, uint8_t size, const uint8_t *key);

uint8_t genkey[16] = {0xDF, 0xDF, 0xDF, 0xDF, 0xDF, 0xDF, 0xDF, 0xDF, 0, 0, 0, 0, 0, 0, 0, 0};
uint8_t enc_buffer[128];
uint8_t ptx_buffer[128];

/* * * * * * * * * * * * * * * * * * * *
 *         Propprietary Functions
 * * * * * * * * * * * * * * * * * * * */
void OnReceiveProprietary(McpsIndication_t *mcpsIndication) {
    printf("RECV PROPERTIES");
    ProprietaryPkt_t pkt = {
        *(mcpsIndication->Buffer),
        mcpsIndication->Buffer,
        mcpsIndication->BufferSize,
    };

    switch (pkt.PType) {
    case KeyAns:
        onReceiveKeyAns(&pkt);
        break;
    default:
        return;
    }
}

/* * * * * * * * * * * * * * * * * * * *
 *             ADN Functions
 * * * * * * * * * * * * * * * * * * * */
AdnContext_t adnCtx;

int initAdn() {
    if (dtuConfig.loopOperation.adnDisable) {
        printf("ADN DISABLED");
        adnCtx.Status = ADN_DISABLED;
    } else {
        printf("ADN RUNING...");
        memset(&adnCtx, 0, sizeof(adnCtx));
        adnCtx.Status   = ADN_IDLE;
        adnCtx.MaxCount = 5;
        memcpy(adnCtx.GenKey, genkey, sizeof(adnCtx.GenKey));

        TimerInit(&adnCtx.scheduler, ScheduleAdn);
    }
}

int processAdn() {
	if (adnCtx.Counter > adnCtx.MaxCount) {
        adnCtx.Status = ADN_EXIT;
        return -1;
    } // Stop ADN after 2 minutes, avoiding run out of power

    if (adnCtx.Status != ADN_IDLE) {
        return -2;
    }

    if (LoRaMacIsBusy()) {
        return -3;
    }
    adnCtx.Counter++;
    uint8_t nonce[8] = {UINT32_2_BYTES((uint32_t)rand1()), UINT32_2_BYTES((uint32_t)rand1())};
    SET_ZERO(adnCtx.Nonce);
    memcpy(adnCtx.Nonce, nonce, sizeof(nonce));
    adnCtx.Status        = ADN_WAITING_ANS;
    MlmeReq_t mlmeKeyReq = {
        .Type                = MLME_KEY_REQUEST,
        .Req.KeyReq.DevNonce = adnCtx.Nonce,
        .Req.KeyReq.GenKey   = adnCtx.GenKey,
    };
    int ret = LoRaMacMlmeRequest(&mlmeKeyReq); // LoRaMAC do not send Mcps msg when device is not joined,
                                               // so we use Mlme rather than Mcps
    if (ret != 0) {
        printf("sendKeyReq fail with err: %d", ret);
    }


    uint32_t dcTime = mlmeKeyReq.ReqReturn.DutyCycleWaitTime;
    if (dcTime < 7000) {
        dcTime = 7000;
    }
    printf("NEXT TX AT: %d ms   Counter=%d  adnCtx.MaxCount=%d\n", dcTime,adnCtx.Counter,adnCtx.MaxCount);
    TimerSetValue(&adnCtx.scheduler, dcTime);
    TimerStart(&adnCtx.scheduler);

    return adnCtx.Counter;
}

void ScheduleAdn(void *context) {
    printf("Scheduling...");
    switch (adnCtx.Status) {
    case ADN_WAITING_ANS:
        adnCtx.Status = ADN_IDLE;
        break;
    case ADN_REBOOT:
        softReset();
        break;
    }
}

void ScheduleRbt(void *context) {
    uint32_t dcTime;
    printf("Rebooting after: %d", dcTime);
}

void onReceiveKeyAns(ProprietaryPkt_t *pkt) {
    if (adnCtx.Status != ADN_WAITING_ANS) {
        printf("did not expect keyans");
        return;
    }

    DBG_HEX("RECV KEYANS:", pkt->PropPayload, pkt->PayloadSize);

    KeyAns_t keyans;
    uint8_t mic[4], pld[16];

    // check size
    if (pkt->PayloadSize != 21) {
        printf("wrong payload size");
        return;
    }

    // verify mic
    if (!aes128_clacmic(mic, pkt->PropPayload, 16 + 1, adnCtx.GenKey)) {
        printf("MIC fail");
        return;
    }

    if (memcmp(mic, pkt->PropPayload + pkt->PayloadSize - 4, 4)) {
        printf("MIC fail: expect: %08x, got: %02x%02x%02x%02x", mic, *(pkt->PropPayload + pkt->PayloadSize - 4),
            *(pkt->PropPayload + pkt->PayloadSize - 4 + 1), *(pkt->PropPayload + pkt->PayloadSize - 4 + 2),
            *(pkt->PropPayload + pkt->PayloadSize - 4 + 3));
        return;
    }

    // decrypt
    if (!aes128_encrypt(pkt->PropPayload + 1, pld, 16, adnCtx.GenKey)) {
        printf("decrypt fail");
        return;
    }

    memcpyr(keyans.DevEUI, pld, sizeof(keyans.DevEUI));
    memcpyr(keyans.AppNonce, pld + sizeof(keyans.DevEUI), sizeof(keyans.AppNonce));

    DBG_HEX("DEVEUI:   ", keyans.DevEUI, 8);
    DBG_HEX("APPNONCE: ", keyans.AppNonce, 8);

    if (memcmp(keyans.DevEUI, dtuConfig.loraWanPara.devEui, sizeof(keyans.DevEUI))) {
        DBG_HEX("KEY ANS NOT FOR ME:", keyans.DevEUI, sizeof(keyans.DevEUI));
    }

    uint8_t appkey[16];
    memcpy(adnCtx.Nonce + sizeof(keyans.AppNonce), keyans.AppNonce, sizeof(keyans.AppNonce));

    if (!aes128_encrypt(adnCtx.Nonce, appkey, sizeof(adnCtx.Nonce), adnCtx.GenKey)) {
        printf("generate appkey fail");
        return;
    }

    DBG_HEX("NONCE:   ", adnCtx.Nonce, 16);
    DBG_HEX("APPKEY:  ", appkey, sizeof(appkey));

    memcpy(dtuConfig.loraWanPara.appKey, appkey, sizeof(appkey));
    dtuConfig.cfgCnt++;
    dtuConfig.saveMask.appKey = 1;
    NvmDataMgmtEvent1(LORAMAC_NVM_NOTIFY_FLAG_MAC_GROUP2);

    adnCtx.Status = ADN_REBOOT;
    disableAdn();

    // Todo: recover after dutycycle
}

void disableAdn() {
    printf("KEY RECEIVED, STOP ADN");
    dtuConfig.cfgCnt++;
    dtuConfig.loopOperation.adnDisable = true;
    NvmDataMgmtEvent1(LORAMAC_NVM_NOTIFY_FLAG_MAC_GROUP2);
    NvmStoreDTUConfig();
}

/* * * * * * * * * * * * * * * * * * * *
 *            Local Functions
 * * * * * * * * * * * * * * * * * * * */
bool aes128_encrypt(const uint8_t *in, uint8_t *out, uint8_t size, const uint8_t *key) {
    if (in == NULL || out == NULL || key == NULL) {
        return false;
    }
    if ((size % 16) != 0) {
        return false;
    }

    aes_context aesContext;
    memset1(aesContext.ksch, '\0', 240);
    aes_set_key(key, 16, &aesContext);

    uint8_t block = 0;
    while (size != 0) {
        if (aes_encrypt(&in[block], &out[block], &aesContext)) {
            return false;
        }
        block = block + 16;
        size  = size - 16;
    }
    return true;
}

bool aes128_clacmic(uint32_t *mic, const uint8_t *dat, uint8_t size, const uint8_t *key) {
    if (mic == NULL || dat == NULL || key == NULL) {
        return false;
    }

    uint8_t cmac[16];
    AES_CMAC_CTX aesCmacCtx;

    AES_CMAC_Init(&aesCmacCtx);
    AES_CMAC_SetKey(&aesCmacCtx, key);
    AES_CMAC_Update(&aesCmacCtx, dat, size);
    AES_CMAC_Final(cmac, &aesCmacCtx);

    uint8_t temp[4] = {0};
    memcpyr(temp, cmac, sizeof(temp));
    memcpy(mic, temp, sizeof(temp));

    return true;
}

// void reverse_array(uint8_t *dct, uint8_t *src, uint8_t len)
// {
//     for (int i = 0; i < len; i++)
//     {
//         dct[i] = src[len - 1 - i];
//     }
// }
