#ifndef __PROPRIETARY__
#define __PROPRIETARY__
#include "CH58x_common.h"
#include "LoRaMac.h"

#define PROP_DBG
#ifdef PROP_DBG
#define DBG_DTL(fmt, args...) printf("FILE: %s, LINE: %d " fmt "\n", __FILE__, __LINE__, ##args)
#define DBG_HEX(notify, array, size)                                                                                   \
    do {                                                                                                               \
        printf(notify);                                                                                                \
        for (int i = 0; i < size; i++) {                                                                               \
            printf("%02x ", array[i]);                                                                                 \
        }                                                                                                              \
        printf("\n");                                                                                                  \
    } while (0)

#else
#define DBG_DTL(fmt, args...)
#define DBG_HEX(notify, array, size)
#endif // PROP_DBG

#define SET_ZERO(x) memset(x, 0, sizeof(x))

typedef struct sProprietaryPkt {
    uint8_t PType;
    uint8_t *PropPayload;
    uint8_t PayloadSize;
    uint8_t Mic[4];

} ProprietaryPkt_t;

enum eLoRaMacVersion {
    LoRaWAN101,
    LoRaWAN102,
    LoRaWAN103,
    LoRaWAN104,
    LoRaWAN110,
};
enum ePType {
    KeyReq = 0x01,
    KeyAns = 0x02,
};

void OnReceiveProprietary(McpsIndication_t *mcpsIndication);

// ADN Function, send keyreq to gateway to ask for a AppKEY

// typedef struct sKeyReq
//{
//     uint8_t version;
//     uint8_t class;
//     uint8_t DevEUI[8];
//     uint8_t DevNonce[8];
//
// } KeyReq_t;

typedef struct sKeyAns {
    uint8_t DevEUI[8];
    uint8_t AppNonce[8];

} KeyAns_t;

typedef enum eAdnStatus {
    ADN_DISABLED,
    ADN_IDLE,
    ADN_WAITING_ANS,
    ADN_EXIT,
    ADN_REBOOT,

} AdnStatus_t;

typedef struct sAdnContext {
    AdnStatus_t Status;

    uint8_t Counter;
    uint8_t MaxCount;

    uint8_t Nonce[16];
    uint8_t GenKey[16];

    TimerEvent_t scheduler;
    TimerEvent_t reboot;

} AdnContext_t;

extern AdnContext_t adnCtx;

#define AdnStatus() (adnCtx.Status)

int initAdn();
int processAdn();

#endif // __PROPRIETARY__
