#include <stateMachine.h>
#include "DFLora_At.h"
#include "MD5.h"
#include "NT3H.h"
#include "NvmDataMgmt.h"
#include "app_i2c.h"
#include "nvmm.h"
#include <stdbool.h>
#include <stdio.h>
#include "comm.h"

BOOL        NT3HReady = false;
BOOL        flagReady = false;
PTHRU_STATE state     = NT3H_IDLE;
PTHRU_STATE nextState = NT3H_IDLE;
uint8_t     nfcAck[64];

static uint32_t packFrameCount = 0;

uint32_t frameCount = 0;

uint8_t  appData[960];
uint32_t appAddr     = 0;

uint16_t dataIndex   = 0;

uint32_t csvFileIndex   = 0;
uint32_t requestPacket  = 0;
uint32_t csvPacketCount = 0;


#define jumpBoot            ((void (*)(void))((int *)BOOT_CODE_ADDR))
uint8_t ackDone = 0;
void PWDGenerater(uint8_t *uid, uint8_t *pwd, uint8_t *pack)
{
    uint8_t md5[16];
    md5_md5(uid, 7, md5);
    memcpy(pwd, md5, 4);
    memcpy(pack, md5 + 8, 2);
}

void flagTimeOut(void)
{
    flagReady = true;
}

void NonBlockingDelayMs(unsigned int ms)
{
    uint32_t  startTimestamp  = getSysTime();
    uint32_t  targetTimestamp = startTimestamp + ms;
    pDFSerial p;
    while (getSysTime() < targetTimestamp)
    {
        // 这里可以添加其他任务的执行
        USB_IRQProcessHandler();
        resetWdog();
        // 处理AT指令
        p = recvAt();
        if (p != NULL)
        {
            execAtCmd(p);
        }
        DelayMs(1);
    }
}

void nfcI2cInit(void)
{
#ifdef  NFC_ENABLE
    // 初始化 IIC
    // 初始化 NT3H

    NT3H_init();
    NT3H_POWER_ON();
    DelayMs(10);
    uint8_t nxpId[7];

    // 检查 ID
    int8_t error = 0;
    GPIOPinRemap(ENABLE, RB_PIN_I2C);
    i2c_app_init(MASTER_ADDR << 1);

    if (NT3H_SUCCESS != (error = NT3H_check_id(nxpId)))
    {
       // PRINT("check_id Fail with err: %d\r\n", error);
       // writeLog("[ERROR]nfc communication EEROR\r\n",getSysTime());
        DelayMs(10);
        return;
    }

    // 写入 CC File
    uint8_t CCFile[4] = NT3H_DEFAULT_CC_FILE_1K;
    if (NT3H_SUCCESS != NT3H_check_cc(CCFile))
    {
        //PRINT("Init CC File Fail, Retry!\r\n");
    }

    // 配置 FD 引脚事件
    if (NT3H_SUCCESS != NT3H_FD_config(NT3H_FD_COMMUNICATION_START, NT3H_FD_STATE_HALT, 0x37))
    {
       // PRINT("Init FDPin Fail, Retry!\r\n");
    }

    // 配置加密
    uint8_t password[NT3H_PWD_LENGTH], pack[NT3H_PACK_LENGTH];
    PWDGenerater(nxpId, password, pack);

    if (NT3H_SUCCESS != NT3H_auth(0xFE, NT3H_ACCESS_NFC_PORT, password, pack, 0))
    {
      //  PRINT("Init Fail, Retry!\r\n");
    }

    if (NT3H_SUCCESS != NT3H_clear_blocks(0x37, 0x37))
    {
      //  PRINT("Clean fail, retrying\r\n");
    }

    // 读取nfc里面的配置
    uint8_t *recvBuf = (uint8_t *)malloc(1024);
    memset(recvBuf,0,1024);

    int         len = NT3H_read_NDEF(NT3H_BLOCK_USER0, recvBuf, 1024);
    nfcConfig_t config;
    memcpy(&config, recvBuf + 5, sizeof(nfcConfig_t));

    //dtuConfig.loraWanPara.devEui;
    //config.basic.devEUI;
    //查询DEVEUI 是否相同 ，DEVEUI必须以DTU flash为准
    uint8_t change = strcmp(dtuConfig.loraWanPara.devEui,config.basic.devEUI);
    //printf("change=%d\n",change);
    if (dtuConfig.cfgCnt > config.basic.cfgCnt || change!= 0) // 如果eeprom里面的配置计数较大，则将eeprom同步到nfc
    {
        // 当配置计数大于10000，则将值改为0，并储存到eeprom
        if (dtuConfig.cfgCnt >= 10000)
        {
            dtuConfig.cfgCnt = 0;
            NvmDataMgmtEvent1(LORAMAC_NVM_NOTIFY_FLAG_MAC_GROUP2);
            NvmStoreDTUConfig();
            //NvmStore();
        }

        int         error = 0;
        uint8_t     nfcDa[800];
        nfcConfig_t NFC   = flashToNfc();
        uint16_t crc = calculateCRC((uint8_t *)&NFC, sizeof(nfcConfig_t));
        nfcDa[0]     = crc >> 8;
        nfcDa[1]     = crc & 0xff;
        memcpy(nfcDa + 2, (uint8_t *)&NFC, sizeof(nfcConfig_t));


        if (NT3H_SUCCESS != NT3H_write_NDEF(NT3H_BLOCK_USER0, nfcDa, sizeof(nfcConfig_t) + 2))
        {
            //PRINT("The NFC config sync fail=%d\r\n", error);
            sprintf(str,"The NFC config sync fail=%d\r\n", error);
            //writeLog(str,1000);
        }
        else
        {
            //sysLog("The NFC config sync success\r\n");
        }
    }
    else if (config.basic.cfgCnt > dtuConfig.cfgCnt) // 如果nfc的配置计数大于eeprom，则将nfc里面的数据同步到eeprom
    {
        if (config.basic.cfgCnt >= 10000)
        {
            config.basic.cfgCnt = 0;
            NT3H_write_NDEF(NT3H_BLOCK_USER0, (uint8_t *)&config, sizeof(nfcConfig_t));
        }
        if (len > 0)
        {
            uint16_t crc = calculateCRC((uint8_t *)&config, sizeof(nfcConfig_t));
            uint16_t crcValue = recvBuf[3]<<8|recvBuf[4];

            // sysLog("nfc to flash\n");
            // PRINT("nfc to dtu 0x%04x,0x%04x\r\n",crc,crcValue);
             if(crc == crcValue){
                //writeLog("nfc to dtu in init\r\n",1000);
                nfcToFlash(config);
                NvmDataMgmtEvent1(4);
             }else{
                 uint8_t     nfcDa[800];
                 nfcConfig_t NFC   = flashToNfc();
                 uint16_t crc = calculateCRC((uint8_t *)&NFC, sizeof(nfcConfig_t));
                 nfcDa[0]     = crc >> 8;
                 nfcDa[1]     = crc & 0xff;
                 memcpy(nfcDa + 2, (uint8_t *)&NFC, sizeof(nfcConfig_t));
                 NT3H_write_NDEF(NT3H_BLOCK_USER0, nfcDa, sizeof(nfcConfig_t) + 2);
                // writeLog("[ERROR]nfc to dtu crc error\r\n",1000);
             }
        }
        else
        {
           // PRINT("nfc to dtu fail\r\n");
        }
    }
    else
    {
        //sysLog("Configuration unchanged\r\n");
        DelayMs(10);
    }
    NT3H_POWER_OFF();
    free(recvBuf);
#endif
}

void stateMachine()
{

}

PTHRU_STATE nfcCommand(uint8_t *command)
{
    PTHRU_STATE state;
    uint8_t     mainCom     = command[0] >> 5;
    uint8_t     slaveCom    = (command[0] >> 2) & 0x07;
    uint8_t     ackFlag     = command[0] & 0x01;
    uint8_t     playLoadLen = command[1] >> 2;
    frameCount  = (uint32_t)(command[1]) << 16 | (uint32_t)(command[2]) << 8 | command[3];
    frameCount &= 0x3FFFF;

    // static
    switch (mainCom)
    {
    case TRANS_AT:
        //把数据封装成一个AT指令，并通过AT指令的规则发到AT数据通道里面
        if (slaveCom == 1)
        {
            dfserials[NFC_INTTERFACE_NUM].datalength = playLoadLen;
            dfserials[NFC_INTTERFACE_NUM].dataFlag   = 1;
            memcpy(dfserials[NFC_INTTERFACE_NUM].recvBuffer, command + 4, playLoadLen);
        }
        NT3H_close_pthru();
        NT3HReady = TRUE;
        NT3H_open_pthru(I2C_TO_NFC);
        state = NT3H_PTHRU_WRITE;
        break;
    case TRANS_UPGRADE:
        // 传输了16个包 或者传输结束
        // PRINT("Sequence: %u\r\n", frameCount);
        if (frameCount % 16 == 0 || slaveCom == 3)
        {
            // 从命令为3，表示升级文件校验通过，写最后一段数据，然后跳转boot程序
            if (slaveCom == 3)
            {
                // PRINT("Write Compelete with %u pkg\n", packFrameCount);
                PRINT("jump to boot\n");
                FLASH_ROM_WRITE(APP_CODE_START_ADDR + packFrameCount * 60, appData, 960);
                DelayMs(10);
                // 跳转到boot
                jumpBoot();
            }
            else
            {
                // 开始时先将flash擦除，之后才能正常写
                if (frameCount == 0)
                {
                    // 先把整块区域擦除
                    FLASH_ROM_ERASE(APP_CODE_START_ADDR, APP_CODE_END_ADDR - APP_CODE_START_ADDR);
                    PRINT("Wiping...\n");
                }
                // 如果新传来的数据帧比之前写入flash的大，表示 正常数据包 16个包的校验通过
                if (packFrameCount < frameCount)
                {
                    if (frameCount != 0)
                    {
                        // 验证成功后,把app数据写到flash 存储区域
                        FLASH_ROM_WRITE(APP_CODE_START_ADDR + packFrameCount * 60, appData, 960);
                    }
                    memset(appData, 0, 960);
                    packFrameCount += 16;
                }
                else
                { // 发生了重传
                    packFrameCount = frameCount;
                }
                dataIndex = 0;
            }
        }
        memcpy(appData + dataIndex, command + 4, 60);
        dataIndex += 60; // appData的储存位置
        // 表示需要回传一个校验包
        if (ackFlag == 1 || slaveCom == 2)
        {
            NT3H_close_pthru();
            NT3HReady = TRUE;
            NT3H_open_pthru(I2C_TO_NFC);
            state = NT3H_PTHRU_WRITE;
            uint8_t md5Value[16];
            md5(appData, 960, md5Value);
            nfcACK(TRANS_UPGRADE, md5Value, 16);
        }
        else
        {
            state = NT3H_PTHRU_READ;
        }
        break;
    case TRANS_READCSV:
        PRINT("Command: %u, %u\r\n", mainCom, slaveCom);
        if (slaveCom == 1)
        {
            uint16_t csvLen1   = (dtuConfig.hardPara.addr / 60) +1;
            uint8_t  csvLen[4] = {csvLen1>>32, csvLen1>>24, csvLen1 >> 8, csvLen1};
            PRINT("CSV Length: %u, %02X, %02X\r\n", csvLen1, csvLen[2], csvLen[3]);
            NT3H_close_pthru();
            NT3HReady = TRUE;
            NT3H_open_pthru(I2C_TO_NFC);
            state = NT3H_PTHRU_WRITE;
            nfcACK(TRANS_READCSV, csvLen, 4);
        }
        else if (slaveCom == 2)
        {
            NT3H_close_pthru();
            NT3HReady = TRUE;
            NT3H_open_pthru(I2C_TO_NFC);

            state         = NT3H_PTHRU_WRITE;
            requestPacket = command[4];
            csvFileIndex  = (uint32_t)command[5] << 16 | (uint32_t)command[6] << 8 | (uint32_t)command[7];
            // 激活csv传输
            nfcACK(TRANS_READCSV_1, NULL /*csvFile + csvFileIndex * 60*/, 60);
        }
        break;
    case TRANS_EXIT:
        break;
    default:
        break;
    }

    return state;
}

void nfcACK(uint8_t cmd, uint8_t *ack, uint16_t len)
{
    uint8_t dataLen = strlen(ack);
    switch (cmd)
    {
    case TRANS_AT:
        nfcAck[0] = (1 << 5) | (1 << 2);
        nfcAck[1] = len << 2;
        memcpy(nfcAck + 4, ack, len);
        nextState = NT3H_PTHRU_READ;
        ackDone   = 1;
        break;
    case TRANS_UPGRADE:
        nfcAck[0] = (2 << 5) | (1 << 2);
        nfcAck[1] = 16 << 2;
        memcpy(nfcAck + 4, ack, 16);
        ackDone   = 1;
        nextState = NT3H_PTHRU_READ;
        break;
    case TRANS_UPGRADE_1:
        nfcAck[0] = (2 << 5) | (2 << 2);
        nfcAck[1] = dataLen << 2;
        memcpy(nfcAck + 4, ack, dataLen);
        ackDone = 1;
        break;
    case TRANS_READCSV:
        nfcAck[0] = (3 << 5) | (1 << 2);
        nfcAck[1] = 4 << 2;
        memcpy(nfcAck + 4, ack, 4);
        ackDone   = 1;
        nextState = NT3H_PTHRU_READ;
        break;
    case TRANS_READCSV_1:
        nfcAck[0] = (4 << 5) | (2 << 2);
        csvPacketCount++;
        static uint32_t csvDataIndex = 0;
        // 如果发送包的计数小于请求包的个数，则继续发包给主机
         //组包
        if (csvPacketCount <= requestPacket)
        {
            // PRINT("PKG: csvPC: %u, request: %u\n", csvPacketCount, requestPacket);
            if (dataLen > 64)
                dataLen = 64;
            nfcAck[1] = 60 << 2;
            nfcAck[1] = nfcAck[1] | csvFileIndex >> 16;
            nfcAck[2] = csvFileIndex >> 8;
            nfcAck[3] = (uint8_t)csvFileIndex;

            //读取csv文件
            flashReadAddr(str,csvFileIndex * 60,60);

            // 返回具体数据
           // uint8_t *data       = (uint8_t *)(APP_CODE_START_ADDR + csvFileIndex * 60);
            uint8_t *pkgPayload = nfcAck + 4;
            for (int j = 0; j < 60; j++)
            {
                pkgPayload[j] = str[j];
            }

            // 把发送的包存入csvData，等待校验
            memcpy(appData + csvDataIndex, pkgPayload, 60);
            csvDataIndex += 60;
            // for (int j = 0; j < 60; j++)
            // {
            //     PRINT("%02X", *(data + j));
            // }
            // PRINT("\n");

            nextState = NT3H_PTHRU_WRITE;
            ackDone   = 1;
            csvFileIndex++;
        }
        else // 请求的包发送完成，返回主机校验码
        {
            // PRINT("MD5: csvPC: %u, request: %u\n", csvPacketCount, requestPacket);
            uint8_t md5Value[16];
            md5(appData, requestPacket * 60, md5Value);
            //            PRINT("==========================\n");
            //            for (int i = 0; i < 16; i++)
            //            {
            //                for (int j = 0; j < 60; j++)
            //                {
            //                    PRINT("%02X", *(appData + 60 * i + j));
            //                }
            //                PRINT("\n");
            //            }
            //            PRINT("==========================\n");
            nfcAck[0] = (4 << 5) | (3 << 2);
            nfcAck[1] = 16 << 2;

            memcpy(nfcAck + 4, md5Value, 16);
            nextState      = NT3H_PTHRU_READ;
            csvPacketCount = 0;
            ackDone        = 1;
            csvDataIndex   = 0;
        }
        break;
    case TRANS_EXIT:
        NT3H_close_pthru();
        state = NT3H_HALT;
        break;
    default:
        break;
    }
}
