#include "utils.h"
#include "CH58x_common.h"
#include <stdlib.h>

void relay(uint8_t value){
    GPIOB_ModeCfg(GPIO_Pin_0, GPIO_ModeOut_PP_5mA);
    if(value == 2) {
       	value = !GPIOB_ReadPortPin(GPIO_Pin_0);
    }
    if(value==1) {
   		GPIOB_SetBits(GPIO_Pin_0);
   	} else {
   		GPIOB_ResetBits(GPIO_Pin_0);
   	}
}

void floatToString(float number, char *str, int precision) {
    int intPart = (int)number;       // 提取整数部分
     float decimalPart = number - intPart;   // 提取小数部分

     // 处理负数
     bool isNegative = false;
     if (number < 0) {
         isNegative = true;
         intPart = abs(intPart);
         decimalPart = fabs(decimalPart);
     }

     // 转换整数部分为字符串
     int index = 0;
     if (intPart == 0) {
         str[index++] = '0';
     }
     else {
         while (intPart > 0) {
             int digit = intPart % 10;
             str[index++] = digit + '0';
             intPart /= 10;
         }
     }

     // 反转整数部分字符串
     reverseString(str, index);

     // 添加负号
     if (isNegative) {
         // 将字符串整体右移一位，为负号腾出空间
         for (int i = index; i >= 0; i--) {
             str[i + 1] = str[i];
         }
         str[0] = '-';
         index++;
     }

     // 添加小数点
     if (precision > 0) {
         str[index++] = '.';
     }

     // 转换小数部分为字符串
     for (int i = 0; i < precision; i++) {
         decimalPart *= 10;
         int digit = (int)decimalPart;
         str[index++] = digit + '0';
         decimalPart -= digit;
     }

     str[index] = '\0';   // 添加字符串结束符
}


lora_packet_t *lora_packet_head=NULL;
lora_packet_t *lora_packet_tail=NULL;
#include "DFLora_At.h"

void lora_packet_list_append(uint8_t *buf, uint8_t len, uint8_t confirmed) {
	if(dtuConfig.loraWanPara.joinState == 0) return;
	if((buf == 0 ) || (len == 0)) return;
	if(lora_packet_list_length() > 99) return;
	lora_packet_t *node = (lora_packet_t *)malloc(sizeof(lora_packet_t));
	if(node == NULL) return;
	node->next = NULL;
	node->buf = (uint8_t*)malloc(len);
	if(node->buf == NULL) {free(node);return;}
	memcpy(node->buf, buf, len);
	DBG("%X%X%X%X\n",buf[0], buf[1], buf[2], buf[3]);
	node->len = len;
	node->confirmed = confirmed;
    if (lora_packet_tail) {
    	lora_packet_tail->next = node;
    	lora_packet_tail = node;
    } else {
    	lora_packet_head = node;
    	lora_packet_tail = node;
    }
}

// 删除链表中的一个节点
void lora_packet_list_del_head() {
    lora_packet_t *current = lora_packet_head;
    if(lora_packet_head)
    	lora_packet_head = lora_packet_head->next;
    if(lora_packet_head == NULL) lora_packet_tail=NULL;
    if(current && current->buf)
    	free(current->buf);
    if(current)
    	free(current);
}

void lora_packet_list_print() {
    lora_packet_t *current = lora_packet_head;
    while (current) {
        printf("Packet length: %d\n", current->len);
        current = current->next;
    }
}

uint8_t lora_packet_list_length() {
	uint8_t ret = 0;
    lora_packet_t *current = lora_packet_head;
    while (current) {
        current = current->next;
        ret ++;
    }
    return ret;
}

lora_packet_t * lora_packet_get_head() {
    return lora_packet_head;
}

